/*	contact - LiPS Address Book Application
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <glib.h>
#include <glib-object.h>

#include <gemapplication.h>
#include <hiker/exgmgr.h>

#include <addressbook.h>
#include <abcontact.h>

#include <enabler.h>
#include <threads.h>

#include <uiwincntedit.h>
#include <uiwincntlistsel.h>

#include <uicompskwindow.h>

#include "exchange.h"
#include "main-hiker.h"

static void
_exg_request_cnt_edit_window_hide_cb (UIWinCntEdit		*editwin,
																			AlpExgRequest		 request)
{
	alp_exg_request_complete(request, 0);
	
	/*
	 * Terminate application, if launched by
	 * exchange server. 
	 */
	if (abook_main_is_relaunched() == FALSE)
	{
		gtk_widget_destroy(GTK_WIDGET(editwin));
	}
	
	return;
}

/*
 * exchange manager service handler
 * 
 * SERVICE: 
 *    mime: "application/x-vcard"
 * 		verb: store
 *    para: string, temporary vCard file name;
 * 	return: <none>
 * 
 */
gboolean
abook_exg_request_store_contact (AlpExgRequest	request)
{
	ab_contact_t *contact = NULL;
	ab_err_t error;
	const gchar *tmp_fname = NULL;
	gint32 fd;
	GemApplication *app = NULL;
	GtkWidget *window = NULL;
	GtkWidget *view = NULL;
	
	alp_exg_request_get_string_parameter_const(request, 
																						 "vcard-file",
																						 &tmp_fname);
	if (NULL == tmp_fname)
	{
		alp_exg_request_complete(request,
														 ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR);
		return FALSE;
	}							
																 
	fd = open(tmp_fname, O_RDONLY, S_IRUSR);
	g_print("%s(): tmp_fname(%s), fd(%d)\n",
					__FUNCTION__,
					tmp_fname,
					fd);

	ab_import_contact(AB_ENABLER_SESS_LOCAL,
										fd,
										&contact,
										AB_VCARD_2_1);
	close(fd);
	
	window = ui_win_cnt_edit_new(contact);
//	ab_contact_free(contact);
	
	if (abook_main_is_relaunched() == FALSE)
	{
		app = gem_application_get_instance();
		gem_application_set_main_window(app,
																		window);
	}
	else
	{
		gtk_widget_show_all(window);
		gtk_window_present(GTK_WINDOW(window));
	}
	
	g_signal_connect(G_OBJECT(window),
									 "hide",
									 G_CALLBACK(_exg_request_cnt_edit_window_hide_cb),
									 (gpointer)request);
	
	return;
}

gboolean
abook_exg_request_unwarp_identifier_string (const gchar	 	*idstr,
																						ab_uid_t			*uid,
																						ab_location_t	*loc)
{
	ab_uid_t tmp_uid = AB_ID_INVALID;
	ab_location_t tmp_loc = AB_LOCAL;
	gchar tmp_loc_str[256];
	gboolean passed = TRUE;
			
	g_return_val_if_fail(idstr, FALSE);
	
	memset(tmp_loc_str, 0, 256);
	 
	sscanf(idstr, 
				 "cnt://%d@%s",
				 &tmp_uid,
				 &tmp_loc_str);

	if (g_utf8_collate(tmp_loc_str, "local") == 0)
	{
		tmp_loc = AB_LOCAL;
	}
	else if (g_utf8_collate(tmp_loc_str, "simcard") == 0)
	{
		tmp_loc = AB_LOCAL;
	}
	else if (g_utf8_collate(tmp_loc_str, "network") == 0)
	{
		tmp_loc = AB_NETWROK;
	}
	else
	{
		passed = FALSE;
	}
	
	if (loc)
	{
		*loc = tmp_loc;
	}
	
	if (uid)
	{
		*uid = tmp_uid;
	}
	
	return passed;
}

/*
 * exchange manager service handler
 * 
 * SERVICE: 
 *    mime: "application/x-contact"
 * 		verb: view
 *    para: string, temporary vCard file name;
 * 	return: <none>
 * 
 */
gboolean
abook_exg_request_view_contact (AlpExgRequest	request)
{
	GemApplication *app = NULL;
	GtkWidget *window = NULL;
	GtkWidget *view = NULL;
	const gchar *identifier = NULL;
	ab_contact_t *contact = NULL;
	ab_uid_t uid;
	ab_location_t loc;
	ab_err_t error;
	
	alp_exg_request_get_string_parameter_const(request, 
																						 "identifier",
																						 &identifier);
	if (NULL == identifier)
	{
		alp_exg_request_complete(request,
														 ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR);
		return FALSE;
	}
	
	g_print("%s(): identifier(%s)\n",
					__FUNCTION__,
					identifier);
	if (abook_exg_request_unwarp_identifier_string(identifier,
																								&uid,
																								&loc) == FALSE)
	{
		alp_exg_request_complete(request,
														 ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR);
		return FALSE;
	}
	g_print("%s(): parsed id(%d), loc(%d)\n",
					__FUNCTION__,
					uid,
					loc);
	
	switch (loc)
	{
		case AB_LOCAL:
			error = ab_get_contact(AB_ENABLER_SESS_LOCAL, uid, &contact);
			break;		

		case AB_SIM:
			error = ab_get_contact(AB_ENABLER_SESS_LOCAL, uid, &contact);
			break;		

		case AB_NETWROK:
			error = ab_get_contact(AB_ENABLER_SESS_LOCAL, uid, &contact);
			break;
		
		default:
			error = AB_ERROR_INVALID;
			break;		
	}
	g_print("%s(): contact(%p)\n",
					__FUNCTION__,
					contact);
	
	if (error != AB_ERROR_NONE ||
			contact == NULL)
	{
		alp_exg_request_complete(request,
														 ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR);
		return FALSE;
	}
														 
	window = ui_win_cnt_edit_new(contact);
//	ab_contact_free(contact);
	
	if (abook_main_is_relaunched() == FALSE)
	{
		app = gem_application_get_instance();
		gem_application_set_main_window(app,
																		window);
	}
	else
	{
		gtk_widget_show_all(window);
		gtk_window_present(GTK_WINDOW(window));
	}
	
	g_signal_connect(G_OBJECT(window),
									 "hide",
									 G_CALLBACK(_exg_request_cnt_edit_window_hide_cb),
									 (gpointer)request);
	
	return;
}

static void
_exg_request_cnt_list_window_hide_cb (UIWinCntListSel	*window,
																			AlpExgRequest		 request)
{
	gchar *sel_str = NULL;
	
	g_object_get(G_OBJECT(window),
							 "selection",
							 &sel_str,
							 NULL);
							 
/*	g_print("%s(): sel_str(%s)\n",
					__FUNCTION__,
					sel_str ? sel_str : "null");
*/					
	alp_exg_request_set_string_parameter(request, 
																			 "contact-uid",
																			 sel_str);
	alp_exg_request_complete(request, 0);
	
	/*
	 * Terminate application, if launched by
	 * exchange server. 
	 */
	if (abook_main_is_relaunched() == FALSE)
	{
		gtk_widget_destroy(GTK_WIDGET(window));
	}
	
	return;
}

void
_exg_request_cnt_list_window_show_cb (UIWinCntListSel	*window,
																			gpointer				 user_data)
{
	ab_thread_t *thread1 = NULL;
	GtkWidget *view = NULL;
	
	view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));	

	thread1 = ab_thread_create((ab_thread_func)ab_enabler_cnt_lst_view_load_contacts,
														 (gpointer)view,
														 TRUE);
																	
  gem_notification_set_text(GEM_NOTIFICATION(thread1->notifier),
                            "Loading ...");
	ab_thread_run(thread1);

	return;
}

/*
 * exchange manager service handler
 * 
 * SERVICE: 
 *    mime: "application/x-contact"
 * 		verb: get
 *    para: <none>
 * 	return: Contact UID
 * 
 */
gboolean
abook_exg_request_get_contact (AlpExgRequest	request)
{
	GemApplication *app = NULL;
	GtkWidget *window = NULL;
	GtkWidget *view = NULL;
	
	window = ui_win_cnt_list_sel_new();
	
	if (abook_main_is_relaunched() == FALSE)
	{
		app = gem_application_get_instance();
		gem_application_set_main_window(app,
																		window);
	}
	else
	{
		gtk_widget_show_all(window);
		gtk_window_present(GTK_WINDOW(window));
	}
	
	g_signal_connect(G_OBJECT(window),
									 "hide",
									 G_CALLBACK(_exg_request_cnt_list_window_hide_cb),
									 (gpointer)request);
	
	g_signal_connect(G_OBJECT(window),
									 "show",
									 G_CALLBACK(_exg_request_cnt_list_window_show_cb),
									 (gpointer)request);
	
	return;
}

/* 
vi:ts=2:nowrap:ai:expandtab
*/
