/*	contact - LiPS Address Book Application
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#include <gtk/gtk.h>
#include <gem.h>
#include <gemnotification.h>
#include <uicompmainwindow.h>
#include <uicompskwindow.h>

#include <hiker/appmgr.h>
#include <hiker/notifymgr.h>
#include <hiker/exgmgr.h>

#include "dbus.h"
#include "enabler.h"

#include "exchange.h"

static char *bundle_name = NULL;
static gboolean relaunched = FALSE;

static gboolean 
exchange_handler (char* id_str)
{
	AlpExgRequest request;
	alp_status_t err;
	char verb_buf[64];
	char type_buf[64];
	long int id_val;
	gboolean ret = FALSE;

	g_return_if_fail(id_str);
	
	memset (verb_buf, 0, sizeof(verb_buf));
	memset (type_buf, 0, sizeof (type_buf));
	
 	id_val= strtol(id_str, NULL,16);
  
	err = alp_exg_request_receive(&request,
																(AlpExgRequestId)id_val);
	if (err)
	{
		return;
	}

	err = alp_exg_request_get_verb(request,
																 verb_buf,
																 sizeof(verb_buf));
	if (err)
	{
		return;
	}
	
	err = alp_exg_request_get_data_type(request,
																			type_buf,
																			sizeof (type_buf));
	if (err)
	{
		return;
	}
	
	g_print("%s(): id_str(%s), id_val(%d), verb(%s), mime-type(%s)\n",
					__FUNCTION__,
					(id_str ? id_str : "null"),
					id_val,
					verb_buf,
					type_buf);
	
	if (strcmp(type_buf, "application/x-vcard") == 0)
	{
		if (strcmp(verb_buf, "store") == 0)
		{
			ret = abook_exg_request_store_contact(request);
		}
		else if (strcmp(verb_buf, "view") == 0)
		{
			ret = abook_exg_request_view_contact(request);
		}
	}
	else if (strcmp(type_buf, "application/x-contact") == 0)
	{
		if (strcmp(verb_buf, "get") == 0)
		{
			ret = abook_exg_request_get_contact(request);
		}
		if (strcmp(verb_buf, "view") == 0)
		{
			ret = abook_exg_request_view_contact(request);
		}
	}
	
	if (ret == FALSE)
	{
		g_print("%s(): UN-HANDLED exchange Service: verb(%s), mime-type(%s)\n",
						__FUNCTION__,
						verb_buf,
						type_buf);
		alp_exg_request_complete(request,
														 ALP_EXG_MGR_ERR_HANDLER_NOT_FOUND_ERROR);
	}

	return ret;
}

static void
prv_contact_launch_handler (int				 argc,
														char			*argv[],
														gboolean	 relaunch)
{
	static char *bundle_name = NULL;
	
	bundle_name = argv[0];
	g_print("%s(): bundle(%s), %s, code(%s)\n",
					__FUNCTION__,
					(bundle_name ? bundle_name : "null"),
					(relaunch ? "relaunched" : "launched"),
					argv[1]);
	
	if (argc > 1)
	{
		if (strcmp(argv[1], ALP_APP_BACKGROUNDED) ==0)
		{
			g_print("%s(): %d\n", __FUNCTION__, __LINE__);
			/* switch to backgroud */
		}
		else if (strcmp(argv[1], ALP_APP_EXCHANGE) == 0)
		{
			g_print("%s(): %d\n", __FUNCTION__, __LINE__);
			/* exchange handling */
			if (exchange_handler(argv[2]) == FALSE)
			{
				exit(-1);
			} 
		}
		else if (argc > 2 && strcmp (argv[1], ALP_APP_NOTIFY) == 0)
		{
			AlpNotifyLaunch notification;
			int notify_size;
			alp_status_t err = 0;
			
			g_print("%s(): %d\n", __FUNCTION__, __LINE__);
			notify_size = alp_notify_details(argv[2],
																			&notification);

      g_print("%s: notification type is %s", 
							__FUNCTION__,
							notification.notifyType);

			if (strcmp(notification.notifyType, 
								 ALP_NOTIFY_EVENT_NOTIFY_REGISTER) == 0)
			{
								
				/*
				 * register for exchange service: 
				 * 
				 * store, application/x-vcard" 
				 */
				err = alp_exg_handler_check_registration("store", "application/x-vcard");
				if (err == ALP_EXG_MGR_ERR_HANDLER_NOT_FOUND_ERROR)
				{
					err = alp_exg_handler_register("store",
																				 NULL,
																				 "application/x-vcard",
																				 bundle_name,
																				 TRUE,
																				 TRUE,
																				 TRUE,
																				 TRUE);
					if (err)
					{
//						return err;
						exit(-1);
					}
				}
	
				/*
				 * register for exchange service: 
				 * 
				 * get, application/x-contact" 
				 */
				err = alp_exg_handler_check_registration("get", "application/x-contact");
				if (err == ALP_EXG_MGR_ERR_HANDLER_NOT_FOUND_ERROR)
				{
					err = alp_exg_handler_register("get",
																				 NULL,
																				 "application/x-contact",
																				 bundle_name,
																				 TRUE,
																				 TRUE,
																				 TRUE,
																				 TRUE);
					if (err)
					{
//						return err;
						exit(-1);
					}
				}
				
				/*
				 * register for exchange service: 
				 * 
				 * view, application/x-contact" 
				 */
				err = alp_exg_handler_check_registration("view", "application/x-contact");
				if (err == ALP_EXG_MGR_ERR_HANDLER_NOT_FOUND_ERROR)
				{
					err = alp_exg_handler_register("view",
																				 NULL,
																				 "application/x-contact",
																				 bundle_name,
																				 TRUE,
																				 TRUE,
																				 TRUE,
																				 TRUE);
					if (err)
					{
//						return err;
						exit(-1);
					}
				}
				alp_notify_done(bundle_name, 0, 0);
			}
			else
			{
				alp_notify_done(bundle_name, 0, 0);
			}
			
			exit(0);
		}
		else/* (strcmp(argv[1], ALP_APP_PRIMARY) == 0) */
		{
			/* switch to foreground */
			
			GemApplication *app = NULL;
			GtkWidget *window = NULL;
			GtkWidget *view = NULL;
			
			window = ui_comp_main_window_new();
											 	
			app = gem_application_get_instance();
				
			gem_application_set_main_window(app,
																			window);
		}
	}
	
	return;
}														

static void
prv_contact_relaunch_handler (int				 argc,
															char			*argv[],
															gpointer	 cbData)
{
	relaunched = TRUE;
	
	return prv_contact_launch_handler(argc, argv, TRUE);
}

static void
prv_contact_exit_handler (gpointer cb_data)
{
	GemApplication *app = NULL;
	alp_status_t ret;
	
	g_print("%s(): exiting ... \n", __FUNCTION__);
	
	app = gem_application_get_instance();

	gem_application_quit(app);
	
	return;
}

static alp_status_t
prv_hiker_init (void)
{
	alp_status_t err = 0;

	err = alp_app_set_relaunch_handler(prv_contact_relaunch_handler,
																		 NULL);
	if (err)
	{
		return err;
	}
	
	err = alp_app_add_exit_handler(prv_contact_exit_handler,
																 NULL);
	if (err)
	{
		return err;
	}

	/* this part is move into relaunch handler
	 */
#if 0	
	/*
	 * register for exchange service: 
	 * 
	 * store, application/x-vcard" 
	 */
	err = alp_exg_handler_check_registration("store", "application/x-vcard");
	if (err == ALP_EXG_MGR_ERR_HANDLER_NOT_FOUND_ERROR)
	{
		err = alp_exg_handler_register("store",
																	 NULL,
																	 "application/x-vcard",
																	 bundle_name,
																	 TRUE,
																	 TRUE,
																	 TRUE,
																	 TRUE);
		if (err)
		{
			return err;
		}
	}

	/*
	 * register for exchange service: 
	 * 
	 * get, application/x-contact" 
	 */
	err = alp_exg_handler_check_registration("get", "application/x-contact");
	if (err == ALP_EXG_MGR_ERR_HANDLER_NOT_FOUND_ERROR)
	{
		err = alp_exg_handler_register("get",
																	 NULL,
																	 "application/x-contact",
																	 bundle_name,
																	 TRUE,
																	 TRUE,
																	 TRUE,
																	 TRUE);
		if (err)
		{
			return err;
		}
	}
#endif
	
	return 0;
}

int alp_main (int argc, char *argv[])
{
	GemApplication *app = NULL;
  gchar *file_path = NULL;
	
	gtk_init(&argc, &argv);

	bundle_name = argv[0];
	prv_hiker_init();
	
	if (abook_iac_initialize () == FALSE)
  {
    g_print ("%s(): abook_dbus_init() failure\n", __FUNCTION__);
    return EXIT_FAILURE;
  }
	
	if (!g_thread_supported())
	{
		g_thread_init(NULL);
		gdk_threads_init();
	}
	
	ab_enabler_initialize();
	
	prv_contact_launch_handler(argc,
														 argv,
														 FALSE);
	
	app = gem_application_get_instance();
	gem_application_run(app);
	
	gem_application_destroy(app);
	ab_enabler_finalize();
	
	return EXIT_SUCCESS;
}

gboolean
abook_main_is_relaunched (void)
{
	return relaunched;
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
