/*	test-transmit.c - Unit Test for data transmit of Transport Local
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com>
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <CUnit/Basic.h>
#include <hiker/exgmgr.h>
#include <hiker/appmgr_pdk.h>

#include <addressbook.h>
#include <abcontact.h>

#include "test-exchange.h"

#define SPEC_VER 1
int 
init_suite_exchange (void)
{
	mimedir_init();
	
	return 0;
}

int 
clean_suite_exchange (void)
{
	return 0;
}

void
testEXCHANGE_STORE_VCARD (void)
{
	AlpExgRequest request;
	ab_contact_t *contact = NULL;
  ab_err_t error;
  ab_sid_t sid;
  ab_fid_t fid;
	alp_status_t ret = 0;
	gchar tmp_fname[] = "/tmp/tmp-exg-vcard-XXXXXX";
	gint32 fd;
  
	error = ab_open_session(0, 
													&sid,
													SPEC_VER);
	CU_ASSERT(AB_ERROR_NONE == error);
	
	fd = mkstemp(tmp_fname);
	g_print("%s(): tmp_fname(%s), fd(%d)\n",
					__FUNCTION__,
					tmp_fname,
					fd);
	CU_ASSERT(-1 != fd);
	
	error = ab_create_contact(sid, &contact);
	CU_ASSERT(NULL != contact);

	ab_contact_set_full_name(contact, "Person01");
	ab_contact_set_mobile_num(contact, "+86139xxxx87820");
	ab_contact_set_home_num(contact, "62xxxx26");
	ab_contact_set_business_num(contact, "82xxxx58");

	ab_contact_add_field(contact, 
											 AB_FTID_BUSINESS_NUM,
											 &fid);
	ab_contact_set_string_field(contact,
															fid,
															"62xxxx88");

	ab_contact_set_email(contact, 
											 "person01@xxxx.com");
	ab_contact_add_field(contact, 
											 AB_FTID_EMAIL,
											 &fid);
	ab_contact_set_string_field(contact,
															fid,
															"person01@yyy.com");
	
	ab_contact_set_department(contact,
														"Department001");
	ab_contact_set_company(contact,
												 "Company001");
		
	error = ab_export_contact(sid, 
														contact,
														fd,
														AB_VCARD_2_1);
	CU_ASSERT(AB_ERROR_NONE == error);

	ab_contact_free(contact);
	
	ret = alp_exg_request_create(&request);
	ret = alp_exg_request_set_verb(request, "store");
	ret = alp_exg_request_set_data_type(request, "application/x-vcard");
 
	ret = alp_exg_request_set_string_parameter(request, "vcard-file", tmp_fname);
	
/*  ret = alp_exg_request_set_string_parameter(request, "Content", "Nan YE");
	ret = alp_exg_request_set_string_parameter(request, "Type", "string");
	ret = alp_exg_request_set_string_parameter(request, "PhoneNum", "139xxxx7820");
*/
	ret = alp_exg_request_execute(request);
	CU_ASSERT(ret == ALP_EXGMGR_ERR_NO_ERROR);
	
	ret = alp_exg_request_delete(request);
	
	close(fd);
	
	return;	
}

void
testEXCHANGE_GET_CONTACT (void)
{
	AlpExgRequest request;
	alp_status_t ret = 0;
  const gchar *cnt_uid = NULL;
  
	ret = alp_exg_request_create(&request);
	ret = alp_exg_request_set_verb(request, "get");
	ret = alp_exg_request_set_data_type(request, "application/x-contact");
 
	ret = alp_exg_request_execute(request);
	CU_ASSERT(ret == ALP_EXGMGR_ERR_NO_ERROR);
	
	alp_exg_request_get_string_parameter_const(request, 
																						 "contact-uid",
																						 &cnt_uid);
	g_print("%s(): cnt_uid(%s)\n",
					__FUNCTION__,
					cnt_uid ? cnt_uid : "null");
	
	ret = alp_exg_request_delete(request);
	
	return;	
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
