/*  addressbook - Address book
 *abookdbus.c - 
 *
 *Authors: YE Nan <nan.ye@orange-ftgroup.com>
 *        ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 *
 *This software and associated documentation files (the "Software") 
 *are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *All Rights Reserved. 
 *
 *A copyright license is hereby granted for redistribution and use of 
 *the Software in source and binary forms, with or without modification, 
 *provided that the following conditions are met: 
 *- Redistributions of source code must retain the above copyright notice, 
 *this copyright license and the following disclaimer. 
 *- Redistributions in binary form must reproduce the above copyright 
 *notice, this copyright license and the following disclaimer in the 
 *documentation and/or other materials provided with the distribution. 
 *- Neither the name of LiPS  nor the names of its Members may be used 
 *to endorse or promote products derived from the Software without 
 *specific prior written permission. 
 *
 *A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *distribute any implementation compliant with the any specification adopted 
 *by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *and non-discriminatory terms and conditions and under reciprocity, as 
 *regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *POSSIBILITY OF SUCH DAMAGE. 
 */


#include <gtk/gtk.h>

#include "dbus-shared.h"
#include "internal.h"
#include "uicompskwindow.h"
#include "abservices.h"
#include "enabler.h"
#include <uicompwinmgr.h>
#include <iac.h>

static iac_context_t *context = NULL;
static gboolean dbus_invoke = FALSE;

//extern void send_message_mw_hide_cb (GtkWidget *widget, ABookMarkView *view);

static gboolean
abook_iac_service_func (const gchar *interface,
                        const gchar *method,
                        iac_arguments_t *arguments,
                        iac_arguments_t *retval, gpointer user_data)
{
  g_print ("%s(): service entering\n", __FUNCTION__);
  g_print ("%s(): request method : %s\n", __FUNCTION__, method);
  g_print ("%s(): iac_invoked    : %s\n",
           __FUNCTION__, (abook_iac_invoked_app ()? "true" : "false"));

  if (g_strcasecmp (method, ADDRESSBOOK_IAC_METHOD_VOICE_FIND) == 0)
  {
    GtkWidget *main_window = NULL;

    if (!abook_iac_invoked_app ())
    {
      main_window = main_window_service_send_message_init ();
    }
    else
    { 
//      main_window = abook_get_app_main_window ();
    }

    gtk_widget_show_all (main_window);
    gtk_window_present (GTK_WINDOW (main_window));

    dbus_invoke = TRUE;

    return IAC_OK;
  }
  else if (g_strcasecmp (method, ADDRESSBOOK_IAC_METHOD_MULTI_SEL) == 0)
  {
    GtkWidget *main_window = NULL;

    if (!abook_iac_invoked_app ())
    {
      main_window = main_window_service_send_message_init ();

      if (FALSE)
      {
        UICompSKWindow *sk_window =
          ui_comp_winmgr_get_window (UI_COMP_SKW_MAKR_VIEW_SIMPLE);
        GtkWidget *view = NULL;
        view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(sk_window));

//        abook_mark_view_parse_contacts (ABOOK_MARKVIEW (sk_window->view), TRUE);
//        g_signal_connect (G_OBJECT (sk_window->window),
//                          "hide",
//                          G_CALLBACK (service_mw_send_message_hide_cb),
//                          sk_window->view);
      }
    }
    else
    {
//      main_window = abook_get_app_main_window ();
    }

    gtk_widget_show_all (main_window);
    gtk_window_present (GTK_WINDOW (main_window));

    return IAC_OK;
  }
  else if (g_strcasecmp (method, ADDRESSBOOK_IAC_METHOD_ADD_CONTACT) == 0)
  {
    GtkWidget *main_window = NULL;

    if (!abook_iac_invoked_app ())
    {

//      main_window = main_window_service_add_contact_init ();

      if (iac_arguments_size (arguments) > 1)
      {
        UICompSKWindow *window = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW_EDIT);
        GtkWidget *view = NULL;
        view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));
//        ABookContactView *view = ABOOK_CONTACT_VIEW (window->view);
        gchar *num = NULL;

        iac_arguments_get_args (arguments,
                                IAC_TYPE_STRING, &num, IAC_TYPE_INVALID);

        g_print ("%s(): number = %s\n", __FUNCTION__, num);

//        abook_contact_view_set_item_value (view, CNT_MOBILENUM, num);

        g_free (num);
      }
    }
    else
    {
//      main_window = abook_get_app_main_window ();
    }

    gtk_widget_show_all (main_window);
    gtk_window_present (GTK_WINDOW (main_window));

    return IAC_OK;
  }

  return IAC_ERROR;
}

gboolean
abook_iac_initialize (void)
{
  context = iac_initialize ();

  if (context == NULL)
  {
    g_print ("%s(): iac_context initialization failure\n", __FUNCTION__);

    return FALSE;
  }

  if (iac_register_service (context, ADDRESSBOOK_IAC_SERVICE) == IAC_ERROR)
  {
    g_print ("%s(): cannot register service\n", __FUNCTION__);

    return FALSE;
  }

  if (iac_register_listener (context,
                             ADDRESSBOOK_IAC_OBJECT,
                             ADDRESSBOOK_IAC_INTERFACE,
                             abook_iac_service_func, NULL) != IAC_OK)
  {
    iac_deinitialize (context);
    return FALSE;
  }

  g_print ("%s(): iac_context = 0x%08x\n", __FUNCTION__, (guint32) context);

  return TRUE;
}

void
abook_iac_finalize (void)
{
  if (context)
  {
    iac_deinitialize (context);
  }

  return;
}

void
abook_iac_set_invoked ()
{
  dbus_invoke = TRUE;

  return;
}

gboolean
abook_iac_invoked_app (void)
{
  return dbus_invoke;
}

gboolean
abook_iac_dialer (const gchar *name, const gchar *picpath,
                  const gchar *dialnum)
{
  iac_arguments_t *args = NULL;

  g_print ("%s(): iac_context = 0x%08x\n", __FUNCTION__, (guint32) context);
  g_print ("%s(): name    = %s\n", __FUNCTION__, name ? name : "null");
  g_print ("%s(): picpath = %s\n", __FUNCTION__, picpath ? picpath : "null");
  g_print ("%s(): dialnum = %s\n", __FUNCTION__, dialnum ? dialnum : "null");
  g_print ("%s(): invokded = %s\n",
           __FUNCTION__, (abook_iac_invoked_app ()? "true" : "false"));
  args = iac_arguments_new_with_args (IAC_TYPE_STRING,
                                      dialnum,
                                      IAC_TYPE_STRING,
                                      name,
                                      IAC_TYPE_STRING,
                                      picpath, IAC_TYPE_INVALID);

  GDK_THREADS_LEAVE ();
  if (iac_start_service (context,
                         VOICECALL_APP_SERVICE,
                         VOICECALL_IAC_METHOD_CALLING_INVOKE) != IAC_OK)
  {
    g_print ("%s(): start app service %s failure\n",
             __FUNCTION__, VOICECALL_APP_SERVICE);

    return FALSE;
  }
  GDK_THREADS_ENTER ();

  gdk_threads_leave ();

  if (abook_iac_invoked_app ())
  {
    g_print ("%s(): call VOICECALL_IAC_METHOD_NUM_RETURN\n", __FUNCTION__);
    iac_execute_service (context,
                         VOICECALL_IAC_SERVICE,
                         VOICECALL_IAC_OBJECT,
                         VOICECALL_IAC_INTERFACE,
                         VOICECALL_IAC_METHOD_NUM_RETURN, args, NULL);
  }
  else
  {
    g_print ("%s(): call VOICECALL_IAC_METHOD_CALLING_INVOKE\n", __FUNCTION__);
    iac_execute_service (context,
                         VOICECALL_IAC_SERVICE,
                         VOICECALL_IAC_OBJECT,
                         VOICECALL_IAC_INTERFACE,
                         VOICECALL_IAC_METHOD_CALLING_INVOKE, args, NULL);
  }

  iac_arguments_free (args);

  return TRUE;
}

gboolean
abook_iac_single_sms (const gchar *name, const gchar *number)
{
  iac_arguments_t *args = NULL;

  g_print ("%s(): iac_context = 0x%08x\n", __FUNCTION__, (guint32) context);
  g_print ("%s(): invokded = %s\n",
           __FUNCTION__, (abook_iac_invoked_app ()? "true" : "false"));
  g_print ("%s(): name = %s, number = %s\n",
           __FUNCTION__, name ? name : "null", number ? number : "null");

  args = iac_arguments_new_with_args (IAC_TYPE_INT32,
                                      1,
                                      IAC_TYPE_STRING,
                                      name,
                                      IAC_TYPE_STRING,
                                      number, IAC_TYPE_INVALID);
  GDK_THREADS_LEAVE ();
  if (iac_start_service (context,
                         SMS_APP_SERIVCE, SMS_IAC_METHOD_SEND_SMS) != IAC_OK)
  {
    g_print ("%s(): start app service %s failure\n",
             __FUNCTION__, SMS_APP_SERIVCE);

    return FALSE;
  }
  GDK_THREADS_ENTER ();

  g_print ("%s(): call SMS_IAC_METHOD_SEND_SMS\n", __FUNCTION__);
  iac_execute_service (context,
                       SMS_IAC_SERVICE,
                       SMS_IAC_OBJECT,
                       SMS_IAC_INTERFACE, SMS_IAC_METHOD_SEND_SMS, args, NULL);

  iac_arguments_free (args);

  return TRUE;
}

gboolean
abook_iac_multi_sms (const GPtrArray *pairs)
{
  iac_arguments_t *args = NULL;
  gint i;

  g_print ("%s(): iac_context = 0x%08x\n", __FUNCTION__, (guint32) context);
  g_print ("%s(): invokded = %s\n",
           __FUNCTION__, (abook_iac_invoked_app ()? "true" : "false"));

  g_print ("%s(): %d pairs\n", __FUNCTION__, pairs->len);
  args = iac_arguments_new_with_args (IAC_TYPE_INT32,
                                      pairs->len, IAC_TYPE_INVALID);

  for (i = 0; i < pairs->len; i++)
  {
    NumberPair *pair = (NumberPair *) g_ptr_array_index (pairs, i);

    g_print ("%s(): append pair [%20s, %s]\n",
             __FUNCTION__, pair->name, pair->number);
    iac_arguments_append_args (args,
                               IAC_TYPE_STRING,
                               pair->name,
                               IAC_TYPE_STRING, pair->number, IAC_TYPE_INVALID);
  }

  GDK_THREADS_LEAVE ();
  if (iac_start_service (context,
                         SMS_APP_SERIVCE, SMS_IAC_METHOD_SEND_SMS) != IAC_OK)
  {
    g_print ("%s(): start app service %s failure\n",
             __FUNCTION__, SMS_APP_SERIVCE);

    return FALSE;
  }
  GDK_THREADS_ENTER ();

  g_print ("%s(): call SMS_IAC_METHOD_SEND_SMS\n", __FUNCTION__);
  iac_execute_service (context,
                       SMS_IAC_SERVICE,
                       SMS_IAC_OBJECT,
                       SMS_IAC_INTERFACE, SMS_IAC_METHOD_SEND_SMS, args, NULL);

  iac_arguments_free (args);

  return TRUE;
}

gboolean
abook_iac_ps_add_contact (const gchar *name,
                          const gchar *num, const gchar *uidstr)
{
  iac_arguments_t *args = NULL;
  iac_arguments_t *retval = NULL;

  g_print ("%s(): name = %s, num = %s, uidstr = %s\n",
           __FUNCTION__, name, num, uidstr);
  args = iac_arguments_new_with_args (IAC_TYPE_STRING,
                                      name,
                                      IAC_TYPE_STRING,
                                      num,
                                      IAC_TYPE_STRING,
                                      uidstr, IAC_TYPE_INVALID);
  retval = iac_arguments_new ();

  g_print ("%s(): call PHONE_SERVER_PBK_METHOD_ADD\n", __FUNCTION__);
  iac_execute_service (context,
                       PHONE_SERVER_SERVICE,
                       PHONE_SERVER_PBK_OBJECT,
                       PHONE_SERVER_PBK_INTERFACE,
                       PHONE_SERVER_PBK_METHOD_ADD, args, retval);

  iac_arguments_free (args);
  iac_arguments_free (retval);

  return TRUE;
}

gboolean
abook_iac_ps_update_contact (gint index, const gchar *name, const gchar *num)
{
  iac_arguments_t *args = NULL;
  iac_arguments_t *retval = NULL;

  g_print ("%s(): index = %d, name = %s, num = %s\n",
           __FUNCTION__, index, name, num);

  args = iac_arguments_new_with_args (IAC_TYPE_INT32,
                                      index,
                                      IAC_TYPE_STRING,
                                      name,
                                      IAC_TYPE_STRING, num, IAC_TYPE_INVALID);
  retval = iac_arguments_new ();

  g_print ("%s(): call PHONE_SERVER_PBK_METHOD_ADD\n", __FUNCTION__);
  iac_execute_service (context,
                       PHONE_SERVER_SERVICE,
                       PHONE_SERVER_PBK_OBJECT,
                       PHONE_SERVER_PBK_INTERFACE,
                       PHONE_SERVER_PBK_METHOD_UPDATE, args, retval);

  iac_arguments_free (args);
  iac_arguments_free (retval);

  return TRUE;
}

gboolean
abook_iac_ps_delete_contact (gint index)
{
  iac_arguments_t *args = NULL;
  iac_arguments_t *retval = NULL;

  args = iac_arguments_new_with_args (IAC_TYPE_INT32, index, IAC_TYPE_INVALID);
  iac_arguments_print (args);

  retval = iac_arguments_new ();

  g_print ("%s(): call PHONE_SERVER_PBK_METHOD_DELETE\n", __FUNCTION__);
  iac_execute_service (context,
                       PHONE_SERVER_SERVICE,
                       PHONE_SERVER_PBK_OBJECT,
                       PHONE_SERVER_PBK_INTERFACE,
                       PHONE_SERVER_PBK_METHOD_DELETE, args, retval);

  iac_arguments_free (args);
  iac_arguments_free (retval);

  return TRUE;
}

/* 
vi:ts=2:nowrap:ai:expandtab
*/

