/*	contact - LiPS Address Book Application
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gem.h>
#include <gdk/gdkkeysyms.h>
#include <uicompcnteditview.h>
#include <gemhashview.h>
#include <gemhashviewcellimg.h>
#include <abcontact.h>
#include <abtemplate.h>
#include "enabler.h"
#include "gemhashviewcell.h"
#include "gemhashviewcellstd.h"

enum {
  PROP_UI_COMP_CNT_EDIT_VIEW_0,
  PROP_CONTACT,
};

typedef struct _UICompCntEditViewPriv UICompCntEditViewPriv;

struct _UICompCntEditViewPriv
{
	gboolean 			 initializing;
	/* Only be used as a parameter, NOT holds real information */
	ab_contact_t	*contact;	
};

#define UI_COMP_CNT_EDIT_VIEW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), \
																								UI_COMP_TYPE_CNT_EDIT_VIEW, \
																								UICompCntEditViewPriv));

static void 			ui_comp_cnt_edit_view_class_init			(UICompCntEditViewClass		*klass);
static void 			ui_comp_cnt_edit_view_init						(UICompCntEditView				*cntview);
static void				ui_comp_cnt_edit_view_destroy					(GtkObject								*object);

static void				ui_comp_cnt_edit_view_set_property		(GObject									*object,
																												 guint										 prop_id,
																												 const GValue							*value,
																												 GParamSpec								*pspec);
static void				ui_comp_cnt_edit_view_get_property		(GObject									*object,
																												 guint										 prop_id,
																												 GValue										*value,
																												 GParamSpec								*pspec);

static GObject *	ui_comp_cnt_edit_view_constructor			(GType										 type,
																												 guint										 n_construct_properties,
																												 GObjectConstructParam		*construct_params);

static void				ui_comp_cnt_edit_view_update_contact	(UICompCntEditView				*cntview);

static GemHashViewClass	*parent_class = NULL;

GType
ui_comp_cnt_edit_view_get_type(void)
{
	static GType cnt_edit_view_type = 0;
	
  if (!cnt_edit_view_type)
	{
		static const GTypeInfo cnt_edit_view_info =
		{
			sizeof (UICompCntEditViewClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) ui_comp_cnt_edit_view_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (UICompCntEditView),
			0,			/* n_preallocs */
			(GInstanceInitFunc) ui_comp_cnt_edit_view_init,
			NULL,
		};

		cnt_edit_view_type = g_type_register_static(GEM_TYPE_HASH_VIEW,
																								"UICompCntEditView",
																								&cnt_edit_view_info,
																								0);
	}

	return cnt_edit_view_type;
}

static void
ui_comp_cnt_edit_view_class_init (UICompCntEditViewClass *klass)
{
	GObjectClass *object_class;
//  GtkObjectClass *object_class;
	GParamSpec *contact_param;

  object_class = G_OBJECT_CLASS(klass);
  parent_class = g_type_class_peek_parent(klass);
  
//  object_class->destroy = ui_comp_cnt_edit_view_destroy;
	object_class->constructor = ui_comp_cnt_edit_view_constructor;		 
 	object_class->set_property = ui_comp_cnt_edit_view_set_property;
  object_class->get_property = ui_comp_cnt_edit_view_get_property;

 	g_type_class_add_private(klass, sizeof(UICompCntEditViewPriv));
 
	contact_param = g_param_spec_pointer("contact",
																			 "Contact",
																			 "Contact",
																			 G_PARAM_CONSTRUCT | G_PARAM_READWRITE);
 
	g_object_class_install_property(object_class,
                                  PROP_CONTACT,
                                  contact_param);
		
	return;
}

static void
ui_comp_cnt_edit_view_destroy (GtkObject *object)
{
	UICompCntEditView *cnt_edit_view = UI_COMP_CNT_EDIT_VIEW(object);

  (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
  
  return;
}

static gboolean
ui_comp_cnt_edit_view_category_entry_key_press_cb (GtkWidget *widget,
                             	                  	 GdkEventKey *event,
                             	                   	 UICompCntEditView *view)
{

  if (event->keyval == GDK_Return)
  {
		action_add_to_category_cb();
  }

  if (event->keyval == GDK_Left ||
      event->keyval == GDK_Right ||
      event->keyval == GDK_Up || event->keyval == GDK_Down)
  {
    return FALSE;
  }

  return TRUE;
}

static void
ui_comp_cnt_edit_view_set_property (GObject					*object,
																		guint						 prop_id,
																		const GValue		*value,
																		GParamSpec			*pspec)
{
  UICompCntEditViewPriv *priv;
	gpointer pointer_val;
	
  priv = UI_COMP_CNT_EDIT_VIEW_GET_PRIVATE(UI_COMP_CNT_EDIT_VIEW(object));

  switch(prop_id)
  {
		case PROP_CONTACT:
			pointer_val = g_value_get_pointer(value);
			if (priv->contact != pointer_val)
			{
				priv->contact = pointer_val;
				ui_comp_cnt_edit_view_update_contact(UI_COMP_CNT_EDIT_VIEW(object));
			}
			g_object_notify(object, "contact");
			break;

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
  
  return;
}

static void
ui_comp_cnt_edit_view_get_property (GObject			*object,
																		guint				 prop_id,
																		GValue			*value,
																		GParamSpec	*pspec)
{
  UICompCntEditViewPriv *priv;
	
  priv = UI_COMP_CNT_EDIT_VIEW_GET_PRIVATE(UI_COMP_CNT_EDIT_VIEW(object));

  switch(prop_id)
  {
		case PROP_CONTACT:
			g_value_set_object(value, (gpointer)priv->contact);
			break;
			
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
  
  return;
}

static void
ui_comp_cnt_edit_view_update_contact (UICompCntEditView		*cntview)
{
  UICompCntEditViewPriv *priv = UI_COMP_CNT_EDIT_VIEW_GET_PRIVATE(cntview);
	GemHashView *base;
	GtkWidget *seperator = NULL;
	GtkWidget *cell = NULL;
	GtkWidget *image = NULL;
	GtkWidget *hbox = NULL;	
	gint i;
	
	if (!priv->initializing)
	{
		gem_hash_view_clear(GEM_HASH_VIEW(cntview));
	}

	base = GEM_HASH_VIEW(cntview);

	/* Name Cell */
	cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Name</span>");	 
	gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
												 AB_IV_FULL_NAME * 1000,
												 GEM_HASH_VIEW_CELL(cell));
												 												 
	gtk_widget_set_size_request(gem_hash_view_cell_std_get_entry(GEM_HASH_VIEW_CELL_STD(cell)),
                              100,
                              -1);
	
	/* Image */
  image = gtk_image_new ();
  if (TRUE)
  {  	
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_set_size_request (hbox, -1, 70);
    gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 10);
    gtk_box_pack_end (GTK_BOX (hbox), image,  FALSE, FALSE, 10);
    gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 10);
  }
  
  /* Seperator */
  seperator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (base->layout), seperator, FALSE, FALSE, 0);
  
	if(NULL == priv->contact)
	{ 
		g_print("\n%s() : entering\n",__FUNCTION__);
	
	  /* Mobile Number Cell */
	  cell = gem_hash_view_cell_std_new ("<span color=\"Blue\">Mobile Number</span>");
	  gem_hash_view_set_item (GEM_HASH_VIEW(cntview),
	                          AB_IV_MOBILE_NUM * 1000,
	                          GEM_HASH_VIEW_CELL(cell));	                                 
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                              (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);	
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);    
	  
	  /* Home Number */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Home Number</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  											 AB_IV_HOME_NUM*1000,
	  											 GEM_HASH_VIEW_CELL(cell)); 
		gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                              (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);  											  											 
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);   	
		
		/* Business Number */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Business Number</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  											 AB_IV_BUSINESS_NUM*1000,
	  											 GEM_HASH_VIEW_CELL(cell));
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);  		
		
		/* FAX Number */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Fax Number</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  									     AB_IV_FAX_NUM*1000,
	  											 GEM_HASH_VIEW_CELL(cell));
		gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);  
		
		/* Company */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Company</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  											 AB_IV_COMPANY*1000,
	  											 GEM_HASH_VIEW_CELL(cell));
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0); 
		
		/* Department */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Department</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  											 AB_IV_DEPARTMENT*1000,
	  											 GEM_HASH_VIEW_CELL(cell));
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);
	  
	  /* Website */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Website</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  											 AB_IV_WEBSITE*1000,
	  											 GEM_HASH_VIEW_CELL(cell));
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0); 
	  
	  /* Email */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Email</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  											 AB_IV_EMAIL*1000,
	  											 GEM_HASH_VIEW_CELL(cell));
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0); 
	
		/* Image Cell */
	  cell = gem_hash_view_cell_image_new ("<span color=\"Blue\">Picture</span>");
	  
	  gem_hash_view_cell_set_visible (GEM_HASH_VIEW_CELL(cell),
	 		    												  TRUE);
	  gem_hash_view_cell_image_set_buddy (GEM_HASH_VIEW_CELL_IMAGE (cell), GTK_IMAGE(image));  
	 
	  gem_hash_view_cell_set_value (GEM_HASH_VIEW_CELL(cell), "icon01.png");
	  gem_hash_view_set_item (GEM_HASH_VIEW (cntview),
	                          AB_IV_PICTURE*1000,
	                          GEM_HASH_VIEW_CELL (cell));
		                   
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);  
	   
	  /* Ringtone */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Ringtone</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  											 AB_IV_RINGTONE*1000,
	  											 GEM_HASH_VIEW_CELL(cell));
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0); 
	  
	  /* Home Address */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Home Address</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  											 AB_IV_HOME_STREET*1000,
	  											 GEM_HASH_VIEW_CELL(cell));
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0); 
	  
	  /* Business Address */
	  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Business Address</span>");
	  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
	  											 AB_IV_BUSI_STREET*1000,
	  											 GEM_HASH_VIEW_CELL(cell));
	 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
	                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
		hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_set_size_request (hbox, -1, -1);
	  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
	  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);
	}
	else
	{
		gint i = 0;
	  ab_fid_t *fid = NULL;
		
		ab_contact_list_fields(priv->contact, &fid);
		 
		for (i = 0; fid[i] != AB_ID_INVALID; i++)
		{
			g_print("%s(): %d\n", __FUNCTION__, fid[i]);
			if (fid[i] == AB_IV_FULL_NAME * 1000 ||
					fid[i] == AB_IV_UID * 1000)
			{
				continue;
			}
			else if(fid[i] == AB_IV_PICTURE * 1000)
			{
				/* Image Cell */
			  cell = gem_hash_view_cell_image_new ("<span color=\"Blue\">Picture</span>");
			  
			  gem_hash_view_cell_set_visible (GEM_HASH_VIEW_CELL(cell),
			 		    												  TRUE);
			  gem_hash_view_cell_image_set_buddy (GEM_HASH_VIEW_CELL_IMAGE (cell), GTK_IMAGE(image));  
			 
			  gem_hash_view_cell_set_value (GEM_HASH_VIEW_CELL(cell), "icon01.png");
			  gem_hash_view_set_item (GEM_HASH_VIEW (cntview),
			                          AB_IV_PICTURE*1000,
			                          GEM_HASH_VIEW_CELL (cell));
				                   
			 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
			                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
				hbox = gtk_hbox_new (FALSE, 0);
			  gtk_widget_set_size_request (hbox, -1, -1);
			  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
			  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);
			}
			else if (fid[i] == AB_IV_RINGTONE * 1000)
			{
				/* Ringtone */
			  cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Ringtone</span>");
			  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
			  											 AB_IV_RINGTONE*1000,
			  											 GEM_HASH_VIEW_CELL(cell));
			 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
			                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
				hbox = gtk_hbox_new (FALSE, 0);
			  gtk_widget_set_size_request (hbox, -1, -1);
			  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
			  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);
			}
			else
			{
		    ab_ftid_t ftid = 0;
		   	gchar *label = NULL;
		   	gchar *name =NULL;
		   	
				ab_contact_get_field_label(priv->contact,
																	 fid[i],
																	 &label);
																	 
				name = g_strdup_printf("%s%s%s","<span color=\"Blue\">",label,"</span>");
			 	cell = gem_hash_view_cell_std_new (name);
			  gem_hash_view_set_item (GEM_HASH_VIEW(cntview),
			                          fid[i],
			                          GEM_HASH_VIEW_CELL (cell));
			 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
			                              (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);	
				hbox = gtk_hbox_new (FALSE, 0);
			  gtk_widget_set_size_request (hbox, -1, -1);
			  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
			  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0);
			  free(name);
			  free(label);
			}  
		}
		 /* Category */
	  
	}
	cell = gem_hash_view_cell_std_new("<span color=\"Blue\">Category</span>");
  gem_hash_view_set_item(GEM_HASH_VIEW(cntview),
  											 AB_IV_CATEGORY_ID*1000,
  											 GEM_HASH_VIEW_CELL(cell));
 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
                               (GEM_HASH_VIEW_CELL_STD(cell)), 180, -1);
  g_signal_connect (G_OBJECT (GEM_HASH_VIEW_CELL_STD(cell)->entry), "key-press-event",
                    G_CALLBACK
                    (ui_comp_cnt_edit_view_category_entry_key_press_cb),
                    (gpointer) cntview);
  image = gtk_image_new ();

  if (TRUE)
  {
    GdkPixbuf *pixbuf = NULL;
    gchar *picpath = NULL;
    picpath = g_build_filename (gem_env_get_resource_path(),
															  "pixmaps",
													  	   "categories.png",
															   NULL);
		gem_resource_load_pic(picpath,
													GEM_PIC_SMALL);
		pixbuf = gem_resource_find_pic(picpath,
																	 GEM_PIC_SMALL);
    gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
    
    g_free (picpath);
  }

  if (TRUE)
  {  	
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_set_size_request (hbox, -1, -1);
    gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 10);
  }
  
  /*UID*/
	cell = gem_hash_view_cell_std_new("<span color=\"Blue\">UID</span>");	 
	gem_hash_view_set_item (GEM_HASH_VIEW(cntview),
												  AB_IV_UID*1000,
												  GEM_HASH_VIEW_CELL(cell));		
	gem_hash_view_cell_set_visible(GEM_HASH_VIEW_CELL(cell),FALSE);																				 
 	gtk_widget_set_size_request (gem_hash_view_cell_std_get_entry
                               (GEM_HASH_VIEW_CELL_STD(cell)), 215, -1);
	hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_size_request (hbox, -1, -1);
  gtk_box_pack_start (GTK_BOX (hbox), cell, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (base->layout), hbox,  FALSE, FALSE, 0); 
    
  GEM_HASH_VIEW(cntview)->focused_item = AB_IV_FULL_NAME*1000;
  
  priv->initializing = FALSE;
  
	return;
}

static GObject *
ui_comp_cnt_edit_view_constructor	(GType										 type,
																	 guint										 n_construct_properties,
																	 GObjectConstructParam		*construct_params)
{
  UICompCntEditViewPriv *priv;
  GObject *object;
  gint i;

  object = (*G_OBJECT_CLASS(parent_class)->constructor)(type,
																												n_construct_properties,
																												construct_params);
	
 	priv = UI_COMP_CNT_EDIT_VIEW_GET_PRIVATE(object);
 	
 	for (i = 0; i < n_construct_properties; i++)
 	{
 		if (g_strcasecmp(construct_params[i].pspec->name, "contact") == 0)
 		{
 			g_object_set(object,
 									 construct_params[i].pspec->name,
 									 g_value_get_pointer(construct_params[i].value),
 									 NULL);
 			break;
 		}
 	}

	ui_comp_cnt_edit_view_update_contact(UI_COMP_CNT_EDIT_VIEW(object));
	
	return object;
}

static void
ui_comp_cnt_edit_view_init (UICompCntEditView *cntview)
{
  UICompCntEditViewPriv *priv = UI_COMP_CNT_EDIT_VIEW_GET_PRIVATE(cntview);
  
  priv->initializing = TRUE;
  priv->contact = NULL;
  
	return;
}

GtkWidget *
ui_comp_cnt_edit_view_new (ab_contact_t *contact)
{
	return gtk_widget_new(UI_COMP_TYPE_CNT_EDIT_VIEW,
												"contact",
												contact,
												NULL);
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
