/*	contact - LiPS Address Book Application
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <glib.h>
#include <glib-object.h>

#include <gem.h>

#include <uicompskwindow.h>
#include <uicompwinmgr.h>

#include <enabler.h>
#include <addressbook.h>
#include <abcontact.h>

#include "uicompcntlstview.h"

enum {
	STG_PBUF_LOCAL,
	STG_PBUF_SIM,
	STG_PBUF_NETWORK,
	STG_PBUF_NUMS,
};

typedef struct _UICompCntLstViewPriv UICompCntLstViewPriv;

struct _UICompCntLstViewPriv
{
	GdkPixbuf					*stg_pbufs[STG_PBUF_NUMS];
	
	GtkWidget					*number_area;
	GtkWidget					*list_area;
	
	GtkTreeViewColumn	*name_col;
	GtkTreeViewColumn	*storage_col;
};

#define UI_COMP_CNT_LST_VIEW_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE((obj), \
																							UI_COMP_TYPE_CNT_LST_VIEW, \
																							UICompCntLstViewPriv));

static void 	ui_comp_cnt_lst_view_class_init			(UICompCntLstViewClass		*klass);
static void 	ui_comp_cnt_lst_view_init						(UICompCntLstView					*cntlview);
static void		ui_comp_cnt_lst_view_finalize				(GObject									*object);

static void		ui_comp_cnt_lst_view_set_property		(GObject									*object,
																									 guint										 prop_id,
																									 const GValue							*value,
																									 GParamSpec								*pspec);
static void		ui_comp_cnt_lst_view_get_property		(GObject									*object,
																									 guint										 prop_id,
																									 GValue										*value,
																									 GParamSpec								*pspec);

static GtkVBoxClass * parent_class = NULL;

GType
ui_comp_cnt_lst_view_get_type (void)
{
	static GType cnt_list_view_type = 0;

  if (!cnt_list_view_type)
	{
		static const GTypeInfo cnt_list_view_info =
		{
			sizeof (UICompCntLstViewClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) ui_comp_cnt_lst_view_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (UICompCntLstView),
			0,			/* n_preallocs */
			(GInstanceInitFunc) ui_comp_cnt_lst_view_init,
			NULL,
		};

		cnt_list_view_type = g_type_register_static(GTK_TYPE_VBOX,
																								"UICompCntLstView",
																								&cnt_list_view_info,
																								0);
	}

	return cnt_list_view_type;
}

void
ui_comp_cnt_lst_view_update (UICompCntLstView	*cntlview)
{
	UICompCntLstViewPriv *priv = UI_COMP_CNT_LST_VIEW_GET_PRIVATE(cntlview);
	GtkTreePath *path = NULL;
	GtkTreeView *treeview = NULL;

  treeview = GTK_TREE_VIEW(gem_tree_view_get_view(GEM_TREE_VIEW(priv->list_area)));  
  
	path = gtk_tree_path_new_first();
	gtk_tree_view_set_cursor(treeview,
													 path,
													 NULL,
													 TRUE);
  
  gtk_tree_path_free(path);
  
	return;
}

static void
ui_comp_cnt_lst_view_class_init (UICompCntLstViewClass * klass)
{
  GObjectClass *object_class;
  GtkWidgetClass *widget_class;
 
	parent_class = g_type_class_peek_parent(klass);
  object_class = G_OBJECT_CLASS(klass);
	widget_class = GTK_WIDGET_CLASS(klass);
	
  object_class->finalize			= ui_comp_cnt_lst_view_finalize;
	object_class->set_property	= ui_comp_cnt_lst_view_set_property;
	object_class->get_property	= ui_comp_cnt_lst_view_get_property;

	g_type_class_add_private(klass,
													 sizeof(UICompCntLstViewPriv));
	
	return;
}

static void
ui_comp_cnt_lst_view_finalize (GObject *object)
{
/*	UICompCntLstViewPriv *priv = UI_COMP_CNT_LST_VIEW_GET_PRIVATE(object);
*/	
	G_OBJECT_CLASS(parent_class)->finalize(object);

	return;
}

static void
ui_comp_cnt_lst_view_set_property (GObject				*object,
																	guint						 prop_id,
																	const GValue		*value,
																	GParamSpec			*pspec)
{
  UICompCntLstViewPriv *priv;

  priv = UI_COMP_CNT_LST_VIEW_GET_PRIVATE(object);

  switch(prop_id)
  {
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
	
  return;
}

static void
ui_comp_cnt_lst_view_get_property (GObject			*object,
																	 guint				 prop_id,
																	 GValue				*value,
																	 GParamSpec		*pspec)
{
  UICompCntLstViewPriv *priv;

  priv = UI_COMP_CNT_LST_VIEW_GET_PRIVATE(object);

  switch(prop_id)
  {
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
  
  return;
}

static void
_ui_comp_cnt_lst_view_cursor_changed_cb (GtkTreeView 			*treeview,
																				 UICompCntLstView	*cntlview)
{
	UICompCntLstViewPriv *priv = UI_COMP_CNT_LST_VIEW_GET_PRIVATE(cntlview);
	GtkTreeModel *model = NULL;
	GtkTreePath *path = NULL;
	
	/*
	 * TODO: maybe, this part SHOULD be moved 
	 *       out from UICompCntLstView in the future. 
	 */

	gtk_tree_view_get_cursor(treeview,
													 &path,
													 NULL);

	model = gtk_tree_view_get_model(treeview);
	if (model && path)
	{
		GtkTreeIter iter;
		ab_uid_t uid = AB_ID_INVALID;
		gint storage;
		
		gtk_tree_model_get_iter(model,
														&iter,
														path);
		
		gtk_tree_model_get(model,
											 &iter,
											 UI_COMP_CLV_UID,
											 &uid,
											 UI_COMP_CLV_STORAGE,
											 &storage,
											 -1);
		
/*		g_print("%s(): uid(%d), storage(%d)\n",
						__FUNCTION__,
						uid,
						storage);
*/						
		if (STORAGE_LOCAL == storage)
		{
			ab_contact_t *contact = NULL;
			
			ab_get_contact(AB_ENABLER_SESS_LOCAL, 
										 uid,
										 &contact);
			if (contact)
			{
				gchar *number;
				
				ab_contact_get_mobile_num(contact, &number);
				
				if (number)
				{
					gtk_label_set_text(GTK_LABEL(priv->number_area),
														 number);
					g_free(number);
				}
				
				ab_contact_free(contact);
			}
		}
		else if (STORAGE_SIM == storage)
		{
			ab_contact_t *contact = NULL;
			
			ab_get_contact(AB_ENABLER_SESS_SIM, 
										 uid,
										 &contact);
			if (contact)
			{
				gchar *number;
				
				ab_contact_get_mobile_num(contact, &number);
				
				if (number)
				{
					gtk_label_set_text(GTK_LABEL(priv->number_area),
														 number);
					g_free(number);
				}
				
				ab_contact_free(contact);
			}
		}
	}
	
	gtk_tree_path_free(path);

	return;	
}	
																
static void
_ui_comp_cnt_lst_view_stg_cell_data_func (GtkTreeViewColumn		*tree_column,
																					GtkCellRenderer			*cell,
																					GtkTreeModel				*tree_model,
																					GtkTreeIter					*iter,
																					UICompCntLstView		*cntlview)
{
	UICompCntLstViewPriv *priv = UI_COMP_CNT_LST_VIEW_GET_PRIVATE(cntlview);
	
	gint storage;
	
	gtk_tree_model_get(tree_model,
										 iter,
										 UI_COMP_CLV_STORAGE,
										 &storage,
										 -1);
	g_object_set(cell,
							 "pixbuf",
							 priv->stg_pbufs[storage],
							 NULL);

	return;
}

static void
ui_comp_cnt_lst_view_init (UICompCntLstView * cntlview)
{
	UICompCntLstViewPriv *priv = UI_COMP_CNT_LST_VIEW_GET_PRIVATE(cntlview);
	GtkWidget *treeview = NULL;
	GtkListStore * liststore = NULL;
	GtkTreeViewColumn	*col = NULL;
	GtkCellRenderer *renderer = NULL;
	gchar *stg_pbuf_names[STG_PBUF_NUMS] = {
		"storage-phone.png",
		"storage-card.png",
		"storage-net.png"
	};
	gint i;
	
	priv->list_area = gem_tree_view_new();
	
	treeview = gem_tree_view_get_view(GEM_TREE_VIEW(priv->list_area));
	
	/* column name */
	priv->name_col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(priv->name_col, "Name");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), 
															priv->name_col);
	
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_renderer_set_fixed_size(renderer,
																	 130,
																	 -1);

	gtk_tree_view_column_pack_start(priv->name_col,
																	renderer,
																	TRUE);
	gtk_tree_view_column_set_alignment(priv->name_col,
																		 1);
	gtk_tree_view_column_add_attribute(priv->name_col,
																		 renderer,
																		 "text",
																		 UI_COMP_CLV_NAME);

	/* column storage */
	priv->storage_col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(priv->storage_col, "Storage");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), 
															priv->storage_col);
	
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_cell_renderer_set_fixed_size(renderer,
																	 32,
																	 32);
	gtk_tree_view_column_pack_start(priv->storage_col,
																	renderer,
																	TRUE);
/*	gtk_tree_view_column_add_attribute(col,
																		 renderer,
																		 "pixbuf",
																		 UI_COMP_CLV_STORAGE);
*/
	gtk_tree_view_column_set_cell_data_func(priv->storage_col,
																					renderer,
																					(GtkTreeCellDataFunc)_ui_comp_cnt_lst_view_stg_cell_data_func,
																					(gpointer)cntlview,
																					NULL);
	
	liststore = gtk_list_store_new(NUM_UI_COMP_CLV_COLS,
																 G_TYPE_UINT,
																 G_TYPE_STRING,
																 G_TYPE_INT);
	
	gem_tree_view_set_model(GEM_TREE_VIEW(priv->list_area),
													GTK_TREE_MODEL(liststore));
											 
	g_object_unref(liststore);

	gem_tree_view_set_search_column(GEM_TREE_VIEW(priv->list_area),
																	UI_COMP_CLV_NAME,
																	GTK_SORT_ASCENDING,
																	NULL);
	
	g_signal_connect(G_OBJECT(treeview),
									 "cursor-changed",
									 G_CALLBACK(_ui_comp_cnt_lst_view_cursor_changed_cb),
									 (gpointer)cntlview);
	
	for (i = 0; i < STG_PBUF_NUMS; i++)
	{
		gchar * abs_fname = NULL;
		
		priv->stg_pbufs[i] = NULL;
		
		if (stg_pbuf_names[i])
		{
			abs_fname = g_build_filename(gem_env_get_resource_path(),
																	 "pixmaps",
															  	 stg_pbuf_names[i],
																	 NULL);
			if (gem_resource_load_pic(abs_fname,
																GEM_PIC_NORMAL) == TRUE)
			{
				priv->stg_pbufs[i] = gem_resource_find_pic(abs_fname,
																									 GEM_PIC_NORMAL);
			}
			
			g_free(abs_fname);
		}
	}
	
	priv->number_area = gtk_label_new("TODO: Number");
	gtk_widget_set_size_request(priv->number_area,
															-1,
															30);
	
	gtk_box_pack_start(GTK_BOX(cntlview),
										 priv->number_area,
										 FALSE,
										 FALSE,
										 0);

	gtk_box_pack_start(GTK_BOX(cntlview),
										 priv->list_area,
										 TRUE,
										 TRUE,
										 0);
	
	return;
}

GtkWidget *
ui_comp_cnt_lst_view_new (void)
{
	return gtk_widget_new(UI_COMP_TYPE_CNT_LST_VIEW,
												NULL);
}

GemTreeView	*
ui_comp_cnt_lst_view_get_tree_view (UICompCntLstView *cntlview)
{
	UICompCntLstViewPriv *priv = UI_COMP_CNT_LST_VIEW_GET_PRIVATE(cntlview);
	
	return GEM_TREE_VIEW(priv->list_area);
}


/*
vi:ts=2:nowrap:ai:expandtab 
*/
