/*	contact - LiPS Address Book Application
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdio.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <gem.h>

#include <uicomptreeviewcnt.h>
#include <uicompskwindow.h>
#include <gdk/gdkkeysyms.h>

static void ui_comp_tree_view_cnt_class_init		(UICompTreeViewCntClass	*klass);
static void ui_comp_tree_view_cnt_init					(UICompTreeViewCnt			*cntview);
static void	ui_comp_tree_view_cnt_destroy				(GtkObject							*object);

static GemTreeViewClass * parent_class = NULL;

GType
ui_comp_tree_view_cnt_get_type (void)
{
	static GType tree_view_cnt_type = 0;

  if (!tree_view_cnt_type)
	{
		static const GTypeInfo tree_view_cnt_info =
		{
			sizeof (UICompTreeViewCntClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) ui_comp_tree_view_cnt_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (UICompTreeViewCnt),
			0,			/* n_preallocs */
			(GInstanceInitFunc) ui_comp_tree_view_cnt_init,
			NULL,
		};

		tree_view_cnt_type = g_type_register_static(GEM_TYPE_TREE_VIEW,
																								"UICompTreeViewCnt",
																								&tree_view_cnt_info,
																								0);
	}

	return tree_view_cnt_type;
}

static void
ui_comp_tree_view_cnt_class_init (UICompTreeViewCntClass * klass)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*)klass;

  parent_class = g_type_class_peek_parent(klass);

  object_class->destroy = ui_comp_tree_view_cnt_destroy;

	return;
}

static void
ui_comp_tree_view_cnt_destroy (GtkObject *object)
{
	UICompTreeViewCnt * tree_view_cnt = UI_COMP_TREE_VIEW_CNT(object);

  (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
  
  return;
}

static void
ui_comp_tree_view_cnt_toggled_cb (GtkCellRendererToggle *celltoggle,
                         				  gchar *pathstr, 
                         				  GtkTreeView *tree)
{
  GtkTreeModel *model = NULL;
  GtkTreePath *path = NULL;
  GtkTreeIter iter;
  gboolean active;

  model = gtk_tree_view_get_model (tree);

  path = gtk_tree_path_new_from_string (pathstr);
  if (!gtk_tree_model_get_iter (model, &iter, path))
  {
    g_print ("%s(): bad path %s\n", __FUNCTION__, pathstr);
    return;
  }

  if (model)
  {
    gtk_tree_model_get (GTK_TREE_MODEL (model),
                        &iter, COL_MV_CHECK, &active, -1);

    active = !active;

    gtk_tree_store_set (GTK_TREE_STORE (model),
                        &iter, COL_MV_CHECK, active, -1);

    /* we also need to modify the active state
     *of relative parent of children
     */
    if (gtk_tree_path_get_depth (path) > 1)
    {
      /* means the current node is a child node */
      GtkTreePath *parent_path = NULL;
      GtkTreeIter parent_iter;

      parent_path = gtk_tree_path_copy (path);
      if (gtk_tree_path_up (parent_path) &&
          gtk_tree_model_get_iter (model, &parent_iter, parent_path))
      {
        gint n_children = 0;
        gint n_active = 0;
        gint i;

        n_children = gtk_tree_model_iter_n_children (model, &parent_iter);

        for (i = 0; i < n_children; i++)
        {
          GtkTreeIter child_iter;

          if (gtk_tree_model_iter_nth_child
              (model, &child_iter, &parent_iter, i))
          {
            gboolean child_active;

            gtk_tree_model_get (GTK_TREE_MODEL (model),
                                &child_iter, COL_MV_CHECK, &child_active, -1);
            if (child_active)
            {
              n_active++;
            }
          }
        }

        gtk_tree_store_set (GTK_TREE_STORE (model),
                            &parent_iter,
                            COL_MV_CHECK, (n_active >= n_children), -1);
      }

      gtk_tree_path_free (parent_path);

      if (TRUE)
      {
        GtkTreeIter first_iter;

        if (gtk_tree_model_get_iter_first (model, &first_iter))
        {
          gtk_tree_store_set (GTK_TREE_STORE (model),
                              &first_iter, COL_MV_CHECK, FALSE, -1);
        }
      }
    }
    else
    {
      /* else it is a top-level node */
      GtkTreePath *root = NULL;

      root = gtk_tree_path_new_first ();
      if (!gtk_tree_path_compare (path, root))
      {
//                              g_print("%s(): Select all\n", __FUNCTION__);
        gint n_persons = 0;
        gint i;

        n_persons = gtk_tree_model_iter_n_children (model, NULL);
//                              g_print("%s(): n_person = %d\n", __FUNCTION__, n_persons);
        for (i = 1; i < n_persons; i++)
        {
          GtkTreeIter person_iter;
          gint n_numbers = 0;
          gint j;

          if (gtk_tree_model_iter_nth_child (model, &person_iter, NULL, i))
          {
            gtk_tree_store_set (GTK_TREE_STORE (model),
                                &person_iter, COL_MV_CHECK, active, -1);
          }

          n_numbers = gtk_tree_model_iter_n_children (model, &person_iter);
          for (j = 0; j < n_numbers; j++)
          {
            GtkTreeIter num_iter;

            if (gtk_tree_model_iter_nth_child
                (model, &num_iter, &person_iter, j))
            {
              gtk_tree_store_set (GTK_TREE_STORE (model),
                                  &num_iter, COL_MV_CHECK, active, -1);
            }
          }
        }
      }
      else
      {
        gint n_children = 0;
        gint i;

//                              g_print("%s(): Select one\n", __FUNCTION__);

        if (TRUE)
        {
          GtkTreeIter first_iter;

          if (gtk_tree_model_get_iter_first (model, &first_iter))
          {
            gtk_tree_store_set (GTK_TREE_STORE (model),
                                &first_iter, COL_MV_CHECK, FALSE, -1);
          }
        }

        n_children = gtk_tree_model_iter_n_children (model, &iter);

        for (i = 0; i < n_children; i++)
        {
          GtkTreeIter child_iter;

          if (gtk_tree_model_iter_nth_child (model, &child_iter, &iter, i))
          {
            gtk_tree_store_set (GTK_TREE_STORE (model),
                                &child_iter, COL_MV_CHECK, active, -1);
          }
        }
      }
    }
  }

  gtk_tree_path_free (path);

  return;
}

static gboolean
ui_comp_tree_view_cnt_key_press_event_cb (GtkWidget *widget,
                                          GdkEventKey *event,
                                          UICompTreeViewCnt *view)
{
  GtkTreeView *treeview = NULL;
  treeview = gem_tree_view_get_view(GEM_TREE_VIEW(view));
	
  g_print("%s(): key = %d \n",__FUNCTION__,event->keyval);

 
  if (event->keyval == GDK_Right)
  {
    GtkTreePath *path = NULL;
    GtkTreeViewColumn *col = NULL;

    gtk_tree_view_get_cursor (treeview, &path, &col);
    if (path && gtk_tree_path_get_depth (path) == 1)
    {
      GtkTreeModel *model = NULL;

      gtk_tree_view_expand_row (treeview, path, FALSE);

#if 0
      model = gtk_tree_view_get_model (treeview);
      if (model)
      {
        GtkTreeIter parent_iter;

        if (gtk_tree_model_get_iter (model, &parent_iter, path))
        {
          GtkTreeIter iter;

          if (gtk_tree_model_iter_children (model, &iter, &parent_iter))
          {
            GtkTreePath *focus_path = NULL;

            focus_path = gtk_tree_model_get_path (model, &iter);
            gtk_tree_view_set_cursor (treeview, focus_path, col, FALSE);

            gtk_tree_path_free (focus_path);
          }
        }
      }
#endif
    }

    return TRUE;
  }
  else if (event->keyval == GDK_Left)
  {
    GtkTreePath *path = NULL;
    GtkTreeViewColumn *col = NULL;

    gtk_tree_view_get_cursor (treeview, &path, &col);

    if (path)
    {
      if (gtk_tree_path_get_depth (path) > 1)
      {
        gtk_tree_path_up (path);
        if (path)
        {
          gtk_tree_view_set_cursor (treeview, path, col, FALSE);
        }
      }
      else
      {
        gtk_tree_view_collapse_row (treeview, path);
      }
    }

    return TRUE;
  }
  else if (event->keyval == GDK_Up)
  {
    GtkTreePath *path = NULL;
    GtkTreePath *first = NULL;
    GtkTreeViewColumn *col = NULL;
    gboolean result = FALSE;

    gtk_tree_view_get_cursor (treeview, &path, &col);

    first = gtk_tree_path_new_first ();
    result = !gtk_tree_path_compare (path, first);
    gtk_tree_path_free (first);

    if (path && result)
    {
      GtkTreeModel *model = NULL;

			g_print("%s(): path(%s)\n",
							__FUNCTION__,
							gtk_tree_path_to_string(path));

      model = gtk_tree_view_get_model (treeview);
      if (model)
      {
        GtkTreeIter iter;

        if (gtk_tree_model_iter_nth_child
            (model, &iter, NULL,
             gtk_tree_model_iter_n_children (model, NULL) - 1))
        {
          GtkTreePath *last = NULL;

          last = gtk_tree_model_get_path (model, &iter);

          if (gtk_tree_view_row_expanded (treeview, last))
          {
          	g_print("%s(): has child \n",__FUNCTION__);
				          	
            GtkTreeIter child_iter;

            if (gtk_tree_model_iter_nth_child
                (model, &child_iter, &iter,
                 gtk_tree_model_iter_n_children (model, &iter) - 1))
            {
              gtk_tree_path_free (last);

              last = gtk_tree_model_get_path (model, &child_iter);
            }
          }

          gtk_tree_view_set_cursor (treeview, last, NULL, FALSE);

          gtk_tree_path_free (last);
        }
      }
    }

    gtk_tree_path_free (path);
    return result;
  }
  else if (event->keyval == GDK_Down)
  {
    GtkTreePath *path = NULL;
    GtkTreeViewColumn *col = NULL;
    GtkTreeModel *model = NULL;
    gboolean result = FALSE;

    gtk_tree_view_get_cursor (treeview, &path, &col);

    model = gtk_tree_view_get_model (treeview);
    if (model)
    {
      GtkTreeIter iter;

      if (gtk_tree_model_iter_nth_child
          (model, &iter, NULL,
           gtk_tree_model_iter_n_children (model, NULL) - 1))
      {
        GtkTreePath *last = NULL;
        last = gtk_tree_model_get_path (model, &iter);
        if (gtk_tree_view_row_expanded (treeview, last))
        {
 					g_print("%s(): has child \n",__FUNCTION__);
        	
          GtkTreeIter child_iter;

          if (gtk_tree_model_iter_nth_child
              (model, &child_iter, &iter,
               gtk_tree_model_iter_n_children (model, &iter) - 1))
          {
            gtk_tree_path_free (last);

            last = gtk_tree_model_get_path (model, &child_iter);
          }
        }

        result = !gtk_tree_path_compare (path, last);

        if (result)
        {
          GtkTreePath *first = NULL;

          first = gtk_tree_path_new_first ();
          gtk_tree_view_set_cursor (treeview, first, NULL, FALSE);
          gtk_tree_path_free (first);
        }

        gtk_tree_path_free (last);
      }
    }

    gtk_tree_path_free (path);

    return result;
  }

//  if (!event->state && event->string &&
//      (isalpha (event->string[0]) || isalnum (event->string[0])))
//  {
//    if (!GTK_WIDGET_HAS_FOCUS (view->search_entry))
//    {
//      GdkEvent *new_event;
//
//      gtk_widget_grab_focus (view->search_entry);
//
//      new_event = gdk_event_copy ((GdkEvent *) event);
//      ((GdkEventKey *) new_event)->window = view->search_entry->window;
//      gtk_widget_event (view->search_entry, new_event);
//
//      return FALSE;
//    }
//  }

  return FALSE;
}

void
ui_comp_tree_view_cnt_string_cell_data_func (GtkTreeViewColumn *col,
			                                      GtkCellRenderer *cell,
			                                      GtkTreeModel *model,
			                                      GtkTreeIter *iter, 
			                                      GtkTreeView *view)
{
  GtkTreeIter parent_iter;

  if (gtk_tree_model_iter_parent (model, &parent_iter, iter))
  {
    gchar *origtxt = NULL;
    gchar *markuptxt = NULL;

    gtk_tree_model_get (GTK_TREE_MODEL (model),
                        iter, COL_MV_STRING, &origtxt, -1);

//              g_print("%s(): entering, str = %s\n", __FUNCTION__, origtxt);
    if (TRUE)
    {
      GtkTreePath *curr_path = NULL;
      GtkTreePath *cursor_path = NULL;
      GtkTreeViewColumn *unused = NULL;

      gtk_tree_view_get_cursor (view, &cursor_path, &unused);
      curr_path = gtk_tree_model_get_path (model, iter);
      if (gtk_tree_path_compare (curr_path, cursor_path) == 0)
      {
        markuptxt =
          g_strdup_printf ("<span color=\"White\" style=\"italic\">%s</span>",
                           origtxt);
      }
      else
      {
        markuptxt =
          g_strdup_printf ("<span color=\"Blue\" style=\"italic\">%s</span>",
                           origtxt);
      }

      gtk_tree_path_free (curr_path);
      gtk_tree_path_free (cursor_path);
    }

    g_object_set (G_OBJECT (cell), "markup", markuptxt, NULL);

    g_free (markuptxt);
    g_free (origtxt);
  }
#if 1
  else
  {
    GtkTreePath *path = NULL;
    GtkTreePath *first = NULL;

    path = gtk_tree_model_get_path (model, iter);
    first = gtk_tree_path_new_first ();

    if (path && first && !gtk_tree_path_compare (path, first))
    {
      gchar *origtxt = NULL;
      gboolean active;

      gtk_tree_model_get (GTK_TREE_MODEL (model),
                          iter,
                          COL_MV_STRING, &origtxt, COL_MV_CHECK, &active, -1);

      //              g_print("%s(): entering, str = %s\n", __FUNCTION__, origtxt);

      if (active)
      {
        gchar *markuptxt = NULL;

        markuptxt = g_strdup_printf ("<span color=\"Red\">%s</span>", origtxt);
        g_object_set (G_OBJECT (cell), "markup", markuptxt, NULL);
        g_free (markuptxt);
      }

      g_free (origtxt);
    }

    gtk_tree_path_free (path);
    gtk_tree_path_free (first);
  }
#endif

  return;
}

gboolean
ui_comp_tree_view_cnt_focus_in_event_cb (GtkWidget *widget, 
																				 GdkEventFocus *event,
																				 gpointer user_data)
{
  UICompTreeViewCnt *view = NULL;
  GtkWidget *treeview = NULL;
  view = UI_COMP_TREE_VIEW_CNT(widget);
	treeview = gem_tree_view_get_view(GEM_TREE_VIEW(view));
  gtk_widget_grab_focus (treeview);

  return FALSE;
}

static void
ui_comp_tree_view_cnt_init (UICompTreeViewCnt * cnt_treeview)
{
	GtkWidget *treeview = NULL;
	GtkTreeViewColumn	*col = NULL;
	GtkCellRenderer *renderer = NULL;
	GtkTreeStore * treestore = NULL;
	
	treeview = gem_tree_view_get_view(GEM_TREE_VIEW(cnt_treeview));
	
	/* column check */
  col = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title (col, "Check");
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);

  renderer = gtk_cell_renderer_toggle_new ();
  gtk_tree_view_column_pack_start (col, renderer, TRUE);
  gtk_tree_view_column_add_attribute (col, renderer, "active", COL_MV_CHECK);
  g_signal_connect (G_OBJECT (renderer), "toggled",
                    G_CALLBACK (ui_comp_tree_view_cnt_toggled_cb), treeview);
                    
  /* column string */
  col = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title (col, "String");
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
  gtk_tree_view_column_set_clickable (col, FALSE);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_column_pack_start (col, renderer, TRUE);
  gtk_cell_renderer_set_fixed_size (renderer, 110, -1);
  g_object_set (G_OBJECT (renderer), "ypad", 5, NULL);
  gtk_tree_view_column_add_attribute (col, renderer, "text", COL_MV_STRING);
  gtk_tree_view_set_expander_column (GTK_TREE_VIEW (treeview), col);
  gtk_tree_view_column_set_cell_data_func (col, renderer,
                                           (GtkTreeCellDataFunc)
                                           ui_comp_tree_view_cnt_string_cell_data_func,
                                           treeview, NULL);                    
	
 /* column picture */
  col = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title (col, "Picutre");
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
  gtk_tree_view_column_set_clickable (col, FALSE);

  renderer = gtk_cell_renderer_pixbuf_new ();
  gtk_tree_view_column_pack_start (col, renderer, TRUE);
  gtk_tree_view_column_add_attribute (col, renderer, "pixbuf", COL_MV_PIC);

  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview), FALSE);
//  g_signal_connect (G_OBJECT (treeview), "key-press-event",
//                    G_CALLBACK (ui_comp_tree_view_cnt_key_press_event_cb),
//                    cnt_treeview);
                    
/*	g_object_set(G_OBJECT(renderer),
							 "xalign",
							 1.0,
							 NULL);
	g_object_set(G_OBJECT(renderer),
							 "ypad",
							 5,
							 NULL);
*/	
	
	treestore = gtk_tree_store_new(NUM_MV_COLS,
                                G_TYPE_STRING,
                                G_TYPE_STRING,
                                G_TYPE_BOOLEAN, 
                                GDK_TYPE_PIXBUF);
	
	gem_tree_view_set_model(GEM_TREE_VIEW(cnt_treeview),
													GTK_TREE_MODEL(treestore));													 
	g_object_unref(treestore);
	
  GTK_WIDGET_SET_FLAGS (cnt_treeview, GTK_CAN_FOCUS);

  g_signal_connect (G_OBJECT (cnt_treeview), "focus-in-event",
                    G_CALLBACK (ui_comp_tree_view_cnt_focus_in_event_cb), NULL);

	gem_tree_view_set_search_column(cnt_treeview,COL_MV_STRING,GTK_SORT_ASCENDING,NULL);
	return;
}

void ui_comp_tree_view_set_checkcolumn_visible(UICompTreeViewCnt * cnt_treeview,gboolean visible)
{
	GtkWidget *treeview = NULL;
	treeview = gem_tree_view_get_view(GEM_TREE_VIEW(cnt_treeview));
	GtkTreeViewColumn *tree_column = gtk_tree_view_get_column (GTK_TREE_VIEW(treeview),0);
	gtk_tree_view_column_set_visible(tree_column,visible);
}

void ui_comp_tree_view_add_headline(UICompTreeViewCnt * cnt_treeview,gchar *content)
{
  GtkTreeStore *tree = NULL;
	GtkTreeIter parnet_iter;
	tree = gem_tree_view_get_model(GEM_TREE_VIEW(cnt_treeview));
  gtk_tree_store_append (tree, &parnet_iter, NULL);
  gtk_tree_store_set (tree, &parnet_iter,
                    COL_MV_STRING, content, COL_MV_CHECK, FALSE, -1);

}

GtkWidget *
ui_comp_tree_view_cnt_new (void)
{
	return gtk_widget_new(UI_COMP_TYPE_TREE_VIEW_CNT,
												NULL);
}

void
ui_comp_tree_view_cnt_update (UICompTreeViewCnt * cntview)
{
  GtkTreePath *path = NULL;
  path = gtk_tree_path_new_first ();
  gtk_tree_view_set_cursor (GTK_TREE_VIEW (gem_tree_view_get_view(cntview)), path, NULL, TRUE);
  gtk_tree_path_free (path);
  return;
}
					

/*
vi:ts=2:nowrap:ai:expandtab 
*/
