/*	uiwincntedit.c - UI Window Contact Edit View
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gemhashview.h>

#include <uicompskwindow.h>
#include <uicompcnteditview.h>

#include "uiwincntedit.h"
#include "uiwinfilechooser.h"

#include <abcontact.h>
#include <enabler.h>

typedef enum {
	NORM_OPT,
	PIC_OPT,
	RING_OPT,
} WindowOptionType;

typedef struct _UIWinCntEditPriv UIWinCntEditPriv;

struct _UIWinCntEditPriv
{
	WindowOptionType	 opt_type;
	ab_contact_t			*contact;
};

#define UI_WIN_CNT_EDIT_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE((obj), \
																					 UI_WIN_TYPE_CNT_EDIT, \
																					 UIWinCntEditPriv));

static void ui_win_cnt_edit_class_init		(UIWinCntEditClass	*klass);
static void ui_win_cnt_edit_init					(UIWinCntEdit				*editwin);
static void	ui_win_cnt_edit_destroy				(GtkObject					*object);


static void	ui_win_cnt_edit_softkey_cb									(UIWinCntEdit				*editwin,
																												 gint								 nr,
																												 gpointer						 storage);
static void	ui_win_cnt_edit_file_chooser_pic_hide_cb		(UIWinFileChooser		*fcwin,
																												 UIWinCntEdit				*editwin);
static void	ui_win_cnt_edit_file_chooser_ring_hide_cb		(UIWinFileChooser		*fcwin,
																												 UIWinCntEdit				*editwin);

static UICompSKWindowClass * parent_class = NULL;

GType
ui_win_cnt_edit_get_type (void)
{
	static GType window_type = 0;

  if (!window_type)
	{
		static const GTypeInfo window_info =
		{
			sizeof (UIWinCntEditClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) ui_win_cnt_edit_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (UIWinCntEdit),
			0,			/* n_preallocs */
			(GInstanceInitFunc) ui_win_cnt_edit_init,
			NULL,
		};
		
		window_type = g_type_register_static(UI_COMP_TYPE_SKWINDOW,
																				 "UIWinCntEdit",
																				 &window_info,
																				 0);
	}
	return window_type;
}

static void
ui_win_cnt_edit_class_init (UIWinCntEditClass * klass)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*)klass;

  parent_class = g_type_class_peek_parent(klass);

	g_type_class_add_private(klass,
													 sizeof(UIWinCntEditPriv));

	return;
}

static gboolean
_ui_win_cht_edit_pic_cell_focus_in_cb (GtkWidget			*widget,
																			 GdkEventFocus	*event,
																			 UIWinCntEdit		*editwin)
{
	UIWinCntEditPriv *priv = UI_WIN_CNT_EDIT_GET_PRIVATE(editwin);

	priv->opt_type = PIC_OPT;
	
	gpe_skwindow_set_softkey_label(GPE_SKWINDOW(editwin), 
																 UI_COMP_SK_LEFT,
																 "Select");
	
	return FALSE;
}

static gboolean
_ui_win_cht_edit_pic_cell_focus_out_cb (GtkWidget				*widget,
																				GdkEventFocus		*event,
																				UIWinCntEdit		*editwin)
{
	UIWinCntEditPriv *priv = UI_WIN_CNT_EDIT_GET_PRIVATE(editwin);

	priv->opt_type = NORM_OPT;
	
	gpe_skwindow_set_softkey_label(GPE_SKWINDOW(editwin), 
																 UI_COMP_SK_LEFT,
																 "Save");
	
	return FALSE;
}

static gboolean
_ui_win_cht_edit_ring_cell_focus_in_cb (GtkWidget				*widget,
																				GdkEventFocus		*event,
																				UIWinCntEdit		*editwin)
{
	UIWinCntEditPriv *priv = UI_WIN_CNT_EDIT_GET_PRIVATE(editwin);

	priv->opt_type = RING_OPT;
	
	gpe_skwindow_set_softkey_label(GPE_SKWINDOW(editwin), 
																 UI_COMP_SK_LEFT,
																 "Select");
	
	return FALSE;
}

static gboolean
_ui_win_cht_edit_ring_cell_focus_out_cb (GtkWidget				*widget,
																				 GdkEventFocus		*event,
																				 UIWinCntEdit			*editwin)
{
	UIWinCntEditPriv *priv = UI_WIN_CNT_EDIT_GET_PRIVATE(editwin);

	priv->opt_type = NORM_OPT;
	
	gpe_skwindow_set_softkey_label(GPE_SKWINDOW(editwin), 
																 UI_COMP_SK_LEFT,
																 "Save");
	
	return FALSE;
}

static void
ui_win_cnt_edit_init (UIWinCntEdit *editwin)
{
	UIWinCntEditPriv *priv = UI_WIN_CNT_EDIT_GET_PRIVATE(editwin);
	GtkWidget *view = NULL;
	GemHashViewCell	*cell = NULL;
	GtkWidget *entry = NULL;
	
	ui_comp_skwindow_parse_sk_conf(UI_COMP_SKWINDOW(editwin),
																 "Save",
																 "Cancel",
																 NULL,
																 NULL,
																 0);

	view = ui_comp_cnt_edit_view_new(NULL);
	ui_comp_skwindow_set_view(UI_COMP_SKWINDOW(editwin),
														view,
														"Edit Contact");
	
	/*
	 * Need to change to correct softkey configuration
	 * when different cell is focused in.
	 */
	cell = gem_hash_view_get_item(GEM_HASH_VIEW(view),
																AB_IV_PICTURE * 1000);
	entry = gem_hash_view_cell_std_get_entry(GEM_HASH_VIEW_CELL_STD(cell));
	g_signal_connect(G_OBJECT(entry),
									 "focus-in-event",
									 G_CALLBACK(_ui_win_cht_edit_pic_cell_focus_in_cb),
									 (gpointer)editwin);
	g_signal_connect(G_OBJECT(entry),
									 "focus-out-event",
									 G_CALLBACK(_ui_win_cht_edit_pic_cell_focus_out_cb),
									 (gpointer)editwin);
		
	cell = gem_hash_view_get_item(GEM_HASH_VIEW(view),
																AB_IV_RINGTONE * 1000);
	entry = gem_hash_view_cell_std_get_entry(GEM_HASH_VIEW_CELL_STD(cell));
	g_signal_connect(G_OBJECT(entry),
									 "focus-in-event",
									 G_CALLBACK(_ui_win_cht_edit_ring_cell_focus_in_cb),
									 (gpointer)editwin);
	g_signal_connect(G_OBJECT(entry),
									 "focus-out-event",
									 G_CALLBACK(_ui_win_cht_edit_ring_cell_focus_out_cb),
									 (gpointer)editwin);

	/*
	 * TODO:	last parameter specify the storage,
	 * 				need to change to meet different 
	 * 				requirement.
	 */
	g_signal_connect(G_OBJECT(editwin),
									 "softkey-activated",
									 G_CALLBACK(ui_win_cnt_edit_softkey_cb),
									 (gpointer)GINT_TO_POINTER(0));							 

	priv->contact = NULL;
	
	return;
}

GtkWidget *
ui_win_cnt_edit_new (ab_contact_t *contact)
{
	UIWinCntEditPriv *priv = NULL;
	GtkWidget *window = NULL;
	GtkWidget *cntview = NULL;
	
	window = gtk_widget_new(UI_WIN_TYPE_CNT_EDIT,
													NULL);
	
	cntview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));

	priv = UI_WIN_CNT_EDIT_GET_PRIVATE(window);
	if (contact)
	{
		priv->contact = contact;
		ab_enabler_cnt_edit_view_set_contact(UI_COMP_CNT_EDIT_VIEW(cntview),
																				 contact);
	}
	
	return window;
}

static void
ui_win_cnt_edit_softkey_cb (UIWinCntEdit	*editwin,
														gint					 nr,
														gpointer			 storage)
{
	UIWinCntEditPriv *priv = UI_WIN_CNT_EDIT_GET_PRIVATE(editwin);
	gboolean updated = FALSE;
  gboolean ret = FALSE;
 	
  if (nr == UI_COMP_SK_LEFT)
  {
  	switch (priv->opt_type)
  	{
  		case NORM_OPT:
	  		{
		      GtkWidget *editview;
			  	
			  	editview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(editwin)); 
		      
		      /*
		       * TODO: FIXME to pass contact through function parameter
		       */
		      switch(GPOINTER_TO_INT(storage))
		      {
		      	case 0:			
//		      		ab_create_contact(AB_ENABLER_SESS_LOCAL, &contact);
							ab_enabler_cnt_parse_from_edit_view(GEM_HASH_VIEW(editview),
																									&(priv->contact));
							break;
							
		      	case 1:
							ab_enabler_cnt_parse_from_edit_view_sim(GEM_HASH_VIEW(editview),
																											&(priv->contact));
							break;
		      }
		      
		      ab_contact_print ((priv->contact));      
		      
		      if ((priv->contact))
		      {
		        ab_uid_t uid = AB_ID_INVALID;    
		        gchar *tmpuid = NULL;
		        
		        ab_add_contact(ab_enabler_get_session((gint)storage),
		        							 (priv->contact),
		        							 &uid);
		        ab_contact_get_uid((priv->contact), &uid);
		  			
		  			tmpuid = g_strdup_printf("%d", uid);
		  			g_print("%s(): uid(%d)\n",
		  							__FUNCTION__,
		  							uid);

		        gem_hash_view_set_value(GEM_HASH_VIEW(editview),
		        												AB_IV_UID * 1000,
		        												tmpuid);
						g_free (tmpuid);
					}
					
		      ab_contact_free (priv->contact);
		      priv->contact = NULL;
		  		ret = TRUE;
	  		}
	  		break;
	  		
  		case PIC_OPT:
	  		{
  				GtkWidget *window = NULL;
		      GtkWidget *fcview = NULL;
	  			g_print("%s(): Picture Select Option\n", __FUNCTION__);

					window = ui_win_file_chooser_new();
					fcview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));
					
					gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fcview),
																						"vroot://photo");
					g_signal_connect(G_OBJECT(window),
													 "hide",
													 G_CALLBACK(ui_win_cnt_edit_file_chooser_pic_hide_cb),
													 (gpointer)editwin);
																						
					gtk_widget_show_all(window);	  			
	  		}
	  		break;
	  		
  		case RING_OPT:
	  		{
  				GtkWidget *window = NULL;
		      GtkWidget *fcview = NULL;
	  			g_print("%s(): Ringtone Select Option\n", __FUNCTION__);

					window = ui_win_file_chooser_new();
					fcview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));
					
					gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fcview),
																						"vroot://photo");
					g_signal_connect(G_OBJECT(window),
													 "hide",
													 G_CALLBACK(ui_win_cnt_edit_file_chooser_ring_hide_cb),
													 (gpointer)editwin);

					gtk_widget_show_all(window);	  			
	  		}
	  		break;
  	}
  }
	else if (nr == UI_COMP_SK_RIGHT)
  {
    ret = TRUE;
  }

  if (ret)
  {
		gtk_widget_hide_all(GTK_WIDGET (editwin));			
	}

  return;
}

static void
ui_win_cnt_edit_file_chooser_pic_hide_cb (UIWinFileChooser	*fcwin,
																					UIWinCntEdit			*editwin)
{
	const gchar *fname = NULL;
	
	fname = ui_win_file_chooser_get_file_name(fcwin);
	if (fname)
	{
		GtkWidget *editview = NULL;
		
		editview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(editwin));
		
		gem_hash_view_set_value(GEM_HASH_VIEW(editview),
														AB_IV_PICTURE * 1000,
														fname);
	}
	
	g_object_disconnect(G_OBJECT(fcwin),
											"any_signal::hide",
											G_CALLBACK(ui_win_cnt_edit_file_chooser_pic_hide_cb),
											(gpointer)editwin,
											NULL);	
	
	return;	
}

static void
ui_win_cnt_edit_file_chooser_ring_hide_cb (UIWinFileChooser	*fcwin,
																					 UIWinCntEdit			*editwin)
{
	const gchar *fname = NULL;
	
	fname = ui_win_file_chooser_get_file_name(fcwin);
	if (fname)
	{
		GtkWidget *editview = NULL;
		
		editview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(editwin));
		
		gem_hash_view_set_value(GEM_HASH_VIEW(editview),
														AB_IV_RINGTONE * 1000,
														fname);
	}

	g_object_disconnect(G_OBJECT(fcwin),
											"any_signal::hide",
											G_CALLBACK(ui_win_cnt_edit_file_chooser_pic_hide_cb),
											(gpointer)editwin,
											NULL);	
	
	return;	
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
