/*	uiwincntlistsel.c - UI Window Contact Selection List View
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gemhashview.h>

#include <addressbook.h>

#include <uicompskwindow.h>
#include <uicompcntlstview.h>

#include "uiwincntlistsel.h"

enum {
	PROP_UI_WIN_CNT_LIST_SEL_0,
	PROP_SELECTION,
};

typedef struct _UIWinCntListSelPriv UIWinCntListSelPriv;

struct _UIWinCntListSelPriv
{
	gchar	*sel_string;
};

#define UI_WIN_CNT_LIST_SEL_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE((obj), \
																							 UI_WIN_TYPE_CNT_LIST_SEL, \
																							 UIWinCntListSelPriv));

static void		ui_win_cnt_list_sel_class_init		(UIWinCntListSelClass			*klass);
static void		ui_win_cnt_list_sel_init					(UIWinCntListSel					*listwin);
static void		ui_win_cnt_list_sel_finalize			(GObject									*object);

static void		ui_win_cnt_list_sel_set_property	(GObject									*object,
																								 guint										 prop_id,
																								 const GValue							*value,
																								 GParamSpec								*pspec);
static void		ui_win_cnt_list_sel_get_property	(GObject									*object,
																								 guint										 prop_id,
																								 GValue										*value,
																								 GParamSpec								*pspec);

static void		ui_win_cnt_list_sel_softkey_cb		(UIWinCntListSel				*listwin,
																								 gint										 nr,
																								 gpointer								 storage);

static UIWinCntListClass * parent_class = NULL;

GType
ui_win_cnt_list_sel_get_type (void)
{
	static GType window_type = 0;

  if (!window_type)
	{
		static const GTypeInfo window_info =
		{
			sizeof (UIWinCntListSelClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) ui_win_cnt_list_sel_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (UIWinCntListSel),
			0,			/* n_preallocs */
			(GInstanceInitFunc) ui_win_cnt_list_sel_init,
			NULL,
		};
		
		window_type = g_type_register_static(UI_WIN_TYPE_CNT_LIST,
																				 "UIWinCntListSel",
																				 &window_info,
																				 0);
	}
	return window_type;
}

static void
ui_win_cnt_list_sel_finalize (GObject *object)
{
	UIWinCntListSelPriv *priv = UI_WIN_CNT_LIST_SEL_GET_PRIVATE(object)
	
	if (priv->sel_string)
	{
		g_free(priv->sel_string);
		priv->sel_string = NULL;
	}
	
	G_OBJECT_CLASS(parent_class)->finalize(object);

	return;
}

static void
ui_win_cnt_list_sel_class_init (UIWinCntListSelClass * klass)
{
  GObjectClass *object_class;
  GParamSpec *sel_param;

  object_class = G_OBJECT_CLASS(klass);
  parent_class = g_type_class_peek_parent(klass);
  
	sel_param = g_param_spec_string("selection",
																	"Selection",
																	"Selection",
																	NULL,
																	G_PARAM_READABLE);

  object_class->finalize	= ui_win_cnt_list_sel_finalize;
	object_class->set_property = ui_win_cnt_list_sel_set_property;
  object_class->get_property = ui_win_cnt_list_sel_get_property;

	g_type_class_add_private(klass,
													 sizeof(UIWinCntListSelPriv));

	g_object_class_install_property(object_class,
                                  PROP_SELECTION,
                                  sel_param);

	return;
}

static void
ui_win_cnt_list_sel_init (UIWinCntListSel *listwin)
{
	UIWinCntListSelPriv *priv = UI_WIN_CNT_LIST_SEL_GET_PRIVATE(listwin);
	GtkWidget *view = NULL;
	
	ui_comp_skwindow_parse_sk_conf(UI_COMP_SKWINDOW(listwin),
																 "Select",
																 "Cancel",
																 NULL,
																 NULL,
																 0);

	g_signal_connect(G_OBJECT(listwin),
									 "softkey-activated",
									 G_CALLBACK(ui_win_cnt_list_sel_softkey_cb),
									 NULL);							 

	return;
}

static void	
ui_win_cnt_list_sel_set_property	(GObject				*object,
																	 guint					 prop_id,
																	 const GValue		*value,
																	 GParamSpec			*pspec)
{
	UIWinCntListSelPriv *priv;

  priv = UI_WIN_CNT_LIST_SEL_GET_PRIVATE(UI_WIN_CNT_LIST_SEL(object));

  switch(prop_id)
  {
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
	
  return;
}

static void
ui_win_cnt_list_sel_get_property	(GObject			*object,
																	 guint				 prop_id,
																	 GValue				*value,
																	 GParamSpec		*pspec)
{
	UIWinCntListSelPriv *priv;

  priv = UI_WIN_CNT_LIST_SEL_GET_PRIVATE(UI_WIN_CNT_LIST_SEL(object));

  switch(prop_id)
  {
  	case PROP_SELECTION:
  		if (priv->sel_string)
  		{
  			g_value_set_string(value, priv->sel_string);
  		}
  		else
  		{
  			g_value_set_string(value, "");
  		}
  		break;
  		
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
  
  return;
}

GtkWidget *
ui_win_cnt_list_sel_new (void)
{
	return gtk_widget_new(UI_WIN_TYPE_CNT_LIST_SEL,
												NULL);
}

static void
ui_win_cnt_list_sel_softkey_cb (UIWinCntListSel	*listwin,
																gint						 nr,
																gpointer				 storage)
{
	UIWinCntListSelPriv *priv = UI_WIN_CNT_LIST_SEL_GET_PRIVATE(listwin);
	gboolean updated = FALSE;
  gboolean ret = FALSE;
 	
  if (nr == UI_COMP_SK_LEFT)
  {
  	GtkWidget *cntlview = NULL;
  	GemTreeView *gemtview = NULL;
  	GtkWidget *treeview = NULL;
		GtkTreeModel *model = NULL;
		
		cntlview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(listwin));
  	gemtview = ui_comp_cnt_lst_view_get_tree_view(UI_COMP_CNT_LST_VIEW(cntlview));
  	
  	treeview = gem_tree_view_get_view(gemtview);
  	model = gem_tree_view_get_model(gemtview);
  	if (treeview && model)
  	{
  		GtkTreePath *path = NULL;
  		GtkTreeIter iter;
  		ab_uid_t uid;
  		gint stg = 0;
  		const gchar *stg_str[] = {
  			"local",
  			"sim",
  			"network",
  		};
  		
  		gtk_tree_view_get_cursor(GTK_TREE_VIEW(treeview),
  														 &path,
  														 NULL);
  		
  		if (path)
  		{												 
				gtk_tree_model_get_iter(model,
	                              &iter,
	                              path);
				
				gtk_tree_model_get(model,
													 &iter,
													 UI_COMP_CLV_UID,
													 &uid,
													 UI_COMP_CLV_STORAGE,
													 &stg,
													 -1);
			
				if (priv->sel_string)
				{
					g_free(priv->sel_string);
				}					 
				
				priv->sel_string =
					g_strdup_printf("cnt://%d@%s",
													uid,
													stg_str[stg]);
													
	/*			g_print("%s(): retrieve uid(%d), storage(%d), uid_str(%s)\n",
								__FUNCTION__,
								uid,
								stg,
								priv->sel_string); 
	*/
				gtk_path_free(path);
  		}	
  	}
  	
    ret = TRUE;
  }
	else if (nr == UI_COMP_SK_RIGHT)
  {
    ret = TRUE;
  }

  if (ret)
  {
		gtk_widget_hide_all(GTK_WIDGET (listwin));			
	}

  return;
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
