/*	uiwincntedit.c - UI Window File Chooser headers
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <gtk/gtk.h>

#include <gemfilechooser.h>

#include <uicompskwindow.h>
#include "uiwinfilechooser.h"

typedef struct _UIWinFileChooserPriv UIWinFileChooserPriv;

struct _UIWinFileChooserPriv
{
	gchar *fname_sel;
};

#define UI_WIN_FILE_CHOOSER_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE((obj), \
																							 UI_WIN_TYPE_FILE_CHOOSER, \
																							 UIWinFileChooserPriv));

static void ui_win_file_chooser_class_init		(UIWinFileChooserClass	*klass);
static void ui_win_file_chooser_init					(UIWinFileChooser				*fcwin);
static void	ui_win_file_chooser_finalize			(GObject								*object);

static void	ui_win_file_chooser_softkey_cb				(UIWinFileChooser				*fcwin,
																									 gint										 nr,
																									 gpointer								 user_data);
static void	ui_win_file_chooser_file_selected_cb	(UIWinFileChooser				*fcwin,
																									 gpointer								 user_data);

static UICompSKWindowClass * parent_class = NULL;

GType
ui_win_file_chooser_get_type (void)
{
	static GType window_type = 0;

  if (!window_type)
	{
		static const GTypeInfo window_info =
		{
			sizeof (UIWinFileChooserClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) ui_win_file_chooser_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (UIWinFileChooser),
			0,			/* n_preallocs */
			(GInstanceInitFunc) ui_win_file_chooser_init,
			NULL,
		};
		
		window_type = g_type_register_static(UI_COMP_TYPE_SKWINDOW,
																				 "UIWinFileChooser",
																				 &window_info,
																				 0);
	}
	return window_type;
}

static void
ui_win_file_chooser_class_init (UIWinFileChooserClass * klass)
{
	GObjectClass *object_class;

  object_class = G_OBJECT_CLASS(klass);
	parent_class = g_type_class_peek_parent(klass);
  
  object_class->finalize = ui_win_file_chooser_finalize;

	g_type_class_add_private(klass,
													 sizeof(UIWinFileChooserPriv));

	return;
}

static void
ui_win_file_chooser_finalize (GObject *object)
{
	UIWinFileChooserPriv *priv = UI_WIN_FILE_CHOOSER_GET_PRIVATE(object);
	
	if (priv->fname_sel)
	{
		g_free(priv->fname_sel);
		priv->fname_sel = NULL;
	}
	
	G_OBJECT_CLASS(parent_class)->finalize(object);
	
	return;
}

static void
ui_win_file_chooser_init (UIWinFileChooser *fcwin)
{
	UIWinFileChooserPriv *priv = UI_WIN_FILE_CHOOSER_GET_PRIVATE(fcwin);
	GtkWidget *view = NULL;
	
  view = gem_file_chooser_new();
  gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(view),
  																	"/");
	g_signal_connect(G_OBJECT(view),
									 "file-selected",
									 G_CALLBACK(ui_win_file_chooser_softkey_cb),
									 NULL);		
	
	ui_comp_skwindow_set_view(UI_COMP_SKWINDOW(fcwin),
														view,
														"File Selections");
														
	ui_comp_skwindow_parse_sk_conf(UI_COMP_SKWINDOW(fcwin),
																 "Select",
																 "Back",
																 NULL,
																 NULL,
																 0);

	g_signal_connect(G_OBJECT(fcwin),
									 "softkey-activated",
									 G_CALLBACK(ui_win_file_chooser_softkey_cb),
									 NULL);		
									 
	priv->fname_sel = NULL;
	
	return;
}

GtkWidget *
ui_win_file_chooser_new (void)
{
	return gtk_widget_new(UI_WIN_TYPE_FILE_CHOOSER,
												NULL);
}

G_CONST_RETURN gchar *
ui_win_file_chooser_get_file_name	(UIWinFileChooser		*fcwin)
{
	UIWinFileChooserPriv *priv = UI_WIN_FILE_CHOOSER_GET_PRIVATE(fcwin);
	
	return priv->fname_sel;
}

static void
ui_win_file_chooser_softkey_cb (UIWinFileChooser	*fcwin,
																gint							 nr,
																gpointer					 user_data)
{
	UIWinFileChooserPriv *priv = UI_WIN_FILE_CHOOSER_GET_PRIVATE(fcwin);
  gboolean ret = FALSE;
 	
  if (nr == UI_COMP_SK_LEFT)
  {
		GList *files = NULL;
		GtkWidget *fcview = NULL;
		
		fcview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(fcwin));
		
		files = gem_file_chooser_get_current_selections(GEM_FILE_CHOOSER(fcview));
		if (files && g_list_length(files))
		{
/*			priv->fname_sel = g_build_filename(gem_file_chooser_get_current_path(GEM_FILE_CHOOSER(fcview)),
																			   g_list_nth_data(files, 0),
														 						 NULL);
*/
			priv->fname_sel = g_strdup((gchar *)g_list_nth_data(files, 0));
	    ret = TRUE;
		}
  }
	else if (nr == UI_COMP_SK_RIGHT)
  {
    ret = TRUE;
  }

  if (ret)
  {
		gtk_widget_hide_all(GTK_WIDGET(fcwin));			
	}

  return;
}

static void
ui_win_file_chooser_file_selected_cb (UIWinFileChooser	*fcwin,
																			gpointer					 user_data)
{
	UIWinFileChooserPriv *priv = UI_WIN_FILE_CHOOSER_GET_PRIVATE(fcwin);
	GList *files = NULL;
	GtkWidget *fcview = NULL;
		
	fcview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(fcwin));
		
	files = gem_file_chooser_get_current_selections(GEM_FILE_CHOOSER(fcview));
	if (files && g_list_length(files))
	{
/*		priv->fname_sel = g_build_filename(gem_file_chooser_get_current_path(GEM_FILE_CHOOSER(fcview)),
																		   g_list_nth_data(files, 0),
													 						 NULL);
*/
		priv->fname_sel = g_strdup((gchar *)g_list_nth_data(files, 0));
		gtk_widget_hide_all(GTK_WIDGET(fcwin));			
	}

  return;
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
