/*	contact - LiPS Address Book Application
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <abvalues.h>
#include <abtemplate.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <addressbook.h>
#include <abcontact.h>
#include <abcategory.h>
#include <uicompcnteditview.h>
#include <uicompcntlistview.h>
#include <uicompcatlistview.h>
#include <uicompskwindow.h>
#include <uicompwinmgr.h>
#include <uicompmainwindow.h>
#include <uicomptreeviewcnt.h>
#include <uicompmarkview.h>
#include "enabler.h"
#include "threads.h"
#include "gemnumchooser.h"
#include <gem.h>
#include <field-desc.h>
#define		ENABLER_SPEC_VER		1

static ab_sid_t sessions[NUM_SESSIONS];

static guint32	indexes[NUM_SESSIONS] = {
	0,	/* Local Flash Address Book */
	1,	/* SIM Card Address Book */
	2,	/* Network Address Book */
}; 
void ab_enabler_cnt_list_hash_table_update(UICompCntListView *cntview);

gboolean
ab_enabler_initialize (void)
{
	gint i;
	
	for (i = 0; i < NUM_SESSIONS; i++)
	{
		ab_err_t error = AB_ERROR_NONE;
		
		error = ab_open_session(indexes[i],
														&sessions[i],
														ENABLER_SPEC_VER);
		if (error != AB_ERROR_NONE)
		{
			g_print("%s(): open session failure. index = %d, error = %d\n",
							__FUNCTION__,
							indexes[i],
							error);
			sessions[i] = AB_ID_INVALID;
		}
	}
	return TRUE;
}

void
ab_enabler_finalize (void)
{
	gint i;
	for (i = 0; i < NUM_SESSIONS; i++)
	{
		if (sessions[i] != AB_ID_INVALID)
		{
			ab_close_session(sessions[i]);
		}
	}
	return;	
}

inline ab_sid_t
ab_enabler_get_session (ABSessionEnum sess)
{
	return sessions[sess];
}

gboolean
ab_enabler_ui_comp_list_view_find (GemTreeView			*cntview,
																	 ab_uid_t          uidcomp,
																	 GtkTreeIter       *iter)
{
	GtkTreeModel * model = NULL;
	model = gem_tree_view_get_model(cntview);
	if (model)
	{
		if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model),
																			iter))
		{
			gboolean res = TRUE;
			while (res)
			{
				ab_uid_t uid = AB_ID_INVALID;
				gboolean compare_res = FALSE;
				
				gtk_tree_model_get(GTK_TREE_MODEL(model),
													 iter,
													 UI_COMP_CNT_LV_UID,
													 &uid,
													 -1);							
				if (uidcomp == uid)
				{
					return TRUE;
				}				
				res = gtk_tree_model_iter_next(GTK_TREE_MODEL(model),
																			 iter);
			}
		}
	}
	return FALSE;
}

void
ab_enabler_ui_comp_list_view_delete (GemTreeView	*cntview,
																		 const ab_uid_t *uid)
{
  GtkListStore *model = NULL;
//      gdk_threads_enter();
  model = gem_tree_view_get_model(cntview);
  if (uid == NULL )
  {
    gtk_list_store_clear (model);
    return;
  }

  if (model)
  {
    GtkTreeIter iter;	
		ab_uid_t tmp_uid = (ab_uid_t)uid;
    gboolean gbb = ab_enabler_ui_comp_list_view_find (cntview, tmp_uid, &iter);
		/*g_print("%s() : gboolean = %s \n",__FUNCTION__,gbb ? "Ture" : "False");*/
    gboolean gb = gtk_list_store_remove (model, &iter);
  }
	ui_comp_cnt_list_view_update(cntview);
  return;
}

void 
ab_enabler_ui_comp_cnt_list_view_add (UICompCntListView			*cntview,
																			const ab_contact_t		*contact)
{
	return ab_enabler_ui_comp_cnt_list_view_update(cntview,
																								 contact,
																								 NULL);
}

void
ab_enabler_ui_comp_cnt_list_view_update (UICompCntListView		*cntview,
																				 const ab_contact_t		*contact,
																				 GtkTreeIter					*iter)
{
	GtkTreeModel * model = NULL;
	model = gem_tree_view_get_model (GEM_TREE_VIEW(cntview));
	if (model)
	{
		ab_uid_t uid = AB_ID_INVALID;
		gchar *name = NULL;
		gchar *picture = NULL;
		GdkPixbuf *pixbuf = NULL;
		
		ab_contact_get_uid (contact, &uid);
		ab_contact_get_full_name (contact, &name);
		ab_contact_get_picture (contact,&picture);   
		
		if (TRUE)
		{
			gchar * picpath = NULL;
			
			picpath = g_build_filename (gem_env_get_resource_path(),
																  "pixmaps",
														  	   picture,
																   NULL);
			gem_resource_load_pic(picpath,
														GEM_PIC_NORMAL);
			pixbuf = gem_resource_find_pic(picpath,
																		 GEM_PIC_NORMAL);
			g_free(picpath);
		} 

		if (iter)
		{
			gtk_list_store_set(GTK_LIST_STORE(model), 
												 iter,
												 UI_COMP_CNT_LV_UID,
												 uid,
												 UI_COMP_CNT_LV_NAME,
												 name,
												 UI_COMP_CNT_LV_PIC,
												 pixbuf,
												 UI_COMP_CNT_LV_STORAGE,
												 STORAGE_LOCAL,
												 -1);			
		}
		else
		{
			GtkTreeIter iter_new;
			gtk_list_store_append(GTK_LIST_STORE(model),
														&iter_new);
			gtk_list_store_set(GTK_LIST_STORE(model),
												 &iter_new,
												 UI_COMP_CNT_LV_UID,
												 uid,
												 UI_COMP_CNT_LV_NAME,
												 name,
												 UI_COMP_CNT_LV_PIC,
												 pixbuf,
												 UI_COMP_CNT_LV_STORAGE,
												 STORAGE_LOCAL,
												 -1);
		}		
		g_free(name);
		//ui_comp_cnt_list_view_update(cntview);
	}
	return;
}

gpointer
ab_enabler_cnt_list_view_load_contacts_thread (gpointer arg)
{
	ab_thread_t	*thread = (ab_thread_t *)arg;
	UICompCntListView *cntview = UI_COMP_CNT_LIST_VIEW(thread->user_data);
	GemNotification *notifier = GEM_NOTIFICATION(thread->notifier);
  ab_uid_t *uids_local = NULL;
  ab_uid_t *uids_sim = NULL;
	gint i;
  gint prg;
	gint count;
		
  ab_get_contact_uids(AB_ENABLER_SESS_LOCAL,
								    	NULL,
								    	&uids_local);
	
	ab_get_contact_uids(AB_ENABLER_SESS_SIM,
								    	NULL,
								    	&uids_sim);

	ab_thread_print(thread);
  
  count = 0;
	if (uids_local)
	{
		for (i = 0; uids_local[i] != NULL; i++)
		{
			count++;			
		}
  }
  
  if (uids_sim)
	{
		for (i = 0; uids_sim[i] != NULL; i++)
		{
			count++;			
		}
  }

  prg = 0;
  if (uids_local)
  {
		for (i = 0; uids_local[i] != NULL; i++, prg++)
		{
      ab_contact_t *contact = NULL;

      ab_get_contact(AB_ENABLER_SESS_LOCAL,
                     uids_local[i],
                     &contact);

      if (contact)
      {
        GDK_THREADS_ENTER();
  			ab_enabler_ui_comp_cnt_list_view_add(cntview,
    																				 contact);
				    																				 
        if (i % 10 == 0 || i == count - 1)
        {
          ui_comp_cnt_list_view_update(cntview);
      	  gem_notification_set_progross_fraction(notifier,
	      																				 ((gdouble)prg / count));
        }
		    GDK_THREADS_LEAVE();	

        ab_contact_free(contact);
      }
 	  }

    g_free(uids_local);
	}
	
	if (uids_sim)
	{
		for (i = 0; uids_sim[i] != NULL; i++)
		{
      ab_contact_t *contact = NULL;

      ab_get_contact(AB_ENABLER_SESS_SIM,
                     uids_sim[i],
                     &contact);

      if (contact)
      {
        GDK_THREADS_ENTER();
	  		ab_enabler_ui_comp_cnt_list_view_add_sim(cntview,
  	      																			 contact);		
        if (i % 10 == 0 || i == count - 1)
        {
    	    ui_comp_cnt_list_view_update(cntview);	
       	  gem_notification_set_progross_fraction(notifier,
  	    																				 ((gdouble)prg / count));
        }
		  	GDK_THREADS_LEAVE();
        
        ab_contact_free(contact);
      }
    }
    
    g_free(uids_sim);
	}

	ab_enabler_cnt_list_hash_table_update(UI_COMP_CNT_LIST_VIEW(cntview));

  gtk_widget_hide_all(GTK_WIDGET(notifier));

	return NULL;	
}

gpointer
ab_enabler_cnt_list_view_load_sim_contacts_thread (UICompCntListView *cntview,GemNotification *notifier)
{
//	ab_thread_t	*thread = (ab_thread_t *)arg;
//	UICompCntListView *cntview = UI_COMP_CNT_LIST_VIEW(thread->user_data);
//	GemNotification *notifier = GEM_NOTIFICATION(thread->notifier);
	ab_contact_t **contacts = NULL;
	gint i;
	gint count;
		
	ab_get_contacts(AB_ENABLER_SESS_SIM,
									NULL,
									&contacts);
	
	g_print("%s(): contacts = 0x%08x\n",
					__FUNCTION__,
					contacts);
//	ab_thread_print(thread);
	if (contacts)
	{
		for (i = 0, count = 0; contacts[i] != NULL; i++)
		{
			count++;			
		}
		
		for (i = 0; contacts[i] != NULL; i++)
		{
/*			g_print("%s(): i = %d\n",
							__FUNCTION__,
							i);
//			ab_contact_print(contacts[i]);
*/			
			gdk_threads_enter();
			ab_enabler_ui_comp_cnt_list_view_add_sim(cntview,
  																				 contacts[i]);
	    ui_comp_cnt_list_view_update(cntview);	

			gem_notification_set_progross_fraction(notifier,
	  																				 (0.5+(gdouble)i / (2*count)));
			gdk_threads_leave();	  									 
 
      /*if (i % 10 == 0)
      {
  			gdk_threads_enter();
	      ui_comp_cnt_list_view_update(cntview);	
        gdk_threads_leave();

  	  	gem_notification_set_progross_fraction(notifier,
	  																					 0.5+((gdouble)i / (2*count)));
      }
      if(i == count-1)
      {
  			gdk_threads_enter();
      	ui_comp_cnt_list_view_update(cntview);
        gdk_threads_leave();
      }*/
		}
	}
	
	ab_free_contacts(AB_ENABLER_SESS_SIM,
									 contacts);

	return NULL;	
}

void 
ab_enabler_ui_comp_cnt_list_view_add_sim (UICompCntListView			*cntview,
																					const ab_contact_t		*contact)
{
	return ab_enabler_ui_comp_cnt_list_view_update_sim(cntview,
																										 contact,
																										 NULL);
}

void
ab_enabler_ui_comp_cnt_list_view_update_sim (UICompCntListView		*cntview,
																					 	const ab_contact_t		*contact,
																				 		GtkTreeIter					*iter)
{
//	g_print("%s(): entering ......\n",__FUNCTION__);
	GtkTreeModel * model = NULL;

	model = gem_tree_view_get_model (GEM_TREE_VIEW(cntview));
	if (model)
	{
		ab_uid_t uid = AB_ID_INVALID;
		gchar *name = NULL;
		gchar *picture = NULL;
		GdkPixbuf *pixbuf = NULL;		
		ab_contact_get_uid (contact, &uid);

		ab_contact_get_full_name (contact, &name);
//		ab_contact_get_picture (contact,&picture);   
		
		if (TRUE)
		{
			gchar * picpath = NULL;			
			picpath = g_build_filename (gem_env_get_resource_path(),
																  "pixmaps",
														  	   "iconsim.png",
																   NULL);
			gem_resource_load_pic(picpath,
														GEM_PIC_NORMAL);
			pixbuf = gem_resource_find_pic(picpath,
																		 GEM_PIC_NORMAL);	
			g_free(picpath);
		} 

		if (iter)
		{
			gtk_list_store_set(GTK_LIST_STORE(model), 
												 iter,
												 UI_COMP_CNT_LV_UID,
												 uid,
												 UI_COMP_CNT_LV_NAME,
												 name,
												 UI_COMP_CNT_LV_PIC,
												 pixbuf,
												 UI_COMP_CNT_LV_STORAGE,
												 STORAGE_SIM,
												 -1);			
		}
		else
		{
			GtkTreeIter iter_new;			
			gtk_list_store_append(GTK_LIST_STORE(model),
														&iter_new);
			gtk_list_store_set(GTK_LIST_STORE(model),
												 &iter_new,
												 UI_COMP_CNT_LV_UID,
												 uid,
												 UI_COMP_CNT_LV_NAME,
												 name,
												 UI_COMP_CNT_LV_PIC,
												 pixbuf,
												 UI_COMP_CNT_LV_STORAGE,
												 STORAGE_SIM,
												 -1);					
		}
		
		g_free(name);
//		ui_comp_cnt_list_view_update(cntview);
	}	
	return;
}

void
ab_enabler_cnt_parse_from_edit_view (GemHashView *view,ab_contact_t **contact)
{	
  ab_sid_t sid;
 // ab_contact_t *contact = NULL;
  gint i;

  sid = ab_enabler_get_session(SESSION_LOCAL);
  if(*contact == NULL)
  {
   	ab_create_contact (sid, contact);
   	for (i = 0; i < AB_IV_NUM; i++)
	  {
	    const gchar *value = gem_hash_view_get_value (view,i*1000);
	    g_print("%s(): no.%d value = [%s]\n", __FUNCTION__,i, value ? value : "null");
	    if (value && strlen (value) > 0 && value!=NULL)
	    {
	   			switch(i)
					{
			 				case AB_IV_FULL_NAME : ab_contact_set_full_name(*contact, value);
										   break;
							case AB_IV_MOBILE_NUM : ab_contact_set_mobile_num(*contact, value); 
							 				 break;
							case AB_IV_HOME_NUM : ab_contact_set_home_num(*contact, value);
							  			 break;
							case AB_IV_BUSINESS_NUM : ab_contact_set_business_num(*contact, value);
											 break;
							case AB_IV_FAX_NUM :  ab_contact_set_fax_num(*contact, value);
											 break;
							case AB_IV_COMPANY : 	ab_contact_set_company(*contact, value);
											 break;
						  case AB_IV_DEPARTMENT : ab_contact_set_department(*contact, value);
						  				 break;
							case AB_IV_WEBSITE : ab_contact_set_website(*contact, value);
											 break;
							case AB_IV_EMAIL : ab_contact_set_email(*contact, value);		
											 break;	
							case AB_IV_PICTURE : ab_contact_set_picture(*contact, value);	
											 break;
							case AB_IV_RINGTONE : ab_contact_set_ringtone(*contact, value);		
												break;
							case AB_IV_HOME_STREET : ab_contact_set_home_street(*contact, value);
												break;
							case AB_IV_BUSI_STREET : ab_contact_set_business_street(*contact, value);
												break;																							        									      										           								       			
			 		}						
	    }
	  }
  }
  else 
  {
	  ab_fid_t *fid = NULL;
		ab_contact_list_fields(*contact,&fid); 
		if(fid!=NULL)
		{
		  for (i = 1; fid[i] != NULL; i++)
			{
				const gchar *value = gem_hash_view_get_value (view,fid[i]);
				if (value && strlen (value) > 0 && value!=NULL)
				{
					ab_contact_set_string_field	(*contact,fid[i],value);
				}
			}
		}
  }
 	ab_contact_print (*contact);
	return ;
}

void
ab_enabler_cnt_parse_from_edit_view_sim (GemHashView *view,ab_contact_t **contact)
{	
  ab_sid_t sid;
  gint i;
  sid = ab_enabler_get_session	(SESSION_SIM);
  if(*contact == NULL)
  {
  	ab_create_contact (sid, contact);
  }  
	for (i = 0; i < AB_IV_NUM; i++)
  {
    const gchar *value = gem_hash_view_get_value (view,i*1000);
    g_print("%s(): no.%d value = [%s]\n", __FUNCTION__,i, value ? value : "null");
    if (value && strlen (value) > 0	&&	value!=NULL)
    {
    	 g_print("%s(): set value\n", __FUNCTION__);
     		switch(i)
				{
		 				case AB_IV_FULL_NAME : ab_contact_set_full_name(*contact, value);
									   break;
						case AB_IV_MOBILE_NUM : ab_contact_set_mobile_num(*contact, value); 
						 				 break;																					        									      										           								       			
		 		}						
    }
#if 1
    else
    {
//      ab_contact_set_string_field (*contact, i + 2, "");
    }
#endif
  }
  ab_contact_print (*contact);
	return ;
}


void 
ab_enabler_cat_parse_from_edit_view(GemHashView *view,ab_category_t **category)
{
	ab_sid_t sid;
  gint i;
  sid = ab_enabler_get_session(0);
  if(*category == NULL)
  {
  	ab_create_category (sid, category);
  }  
	for (i = 0; i < NUM_CATS; i++)
  {
    const gchar *value = gem_hash_view_get_value (view,i);
    g_print("%s(): no.%d value = [%s]\n", __FUNCTION__,i, value ? value : "null");
    if (value && strlen (value) > 0	&&	value!=NULL)
    {
    	 g_print("%s(): set value\n", __FUNCTION__);
     		switch(i)
				{
		 				case CAT_NAME : ab_category_set_name(*category, value);
									   break;
						case CAT_PICTURE : ab_category_set_picture(*category, value); 
						 				 break;
						case CAT_RINGTONE : ab_category_set_ringtone(*category, value); 
						 				 break;						 																									        									      										           								       			
		 		}						
    }
#if 1
    else
    {
//      ab_contact_set_string_field (*contact, i + 2, "");
    }
#endif
  }
  ab_category_print (*category);
	return ;
}

/*void
ab_enabler_cnt_set_to_edit_view(GemHashView *view,ab_contact_t **contact)
{
	//set contact's name
    gchar *fieldstr = NULL;
    for (i = 0; i < NUM_AB_IV_CELLS; i++)
    {
    	
    }
    
    ab_contact_get_full_name	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_FULL_NAME,
												      fieldstr);					     									      										      
    //set contact's mobile phone
    ab_contact_get_mobile_num	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_MOBILE_NUM,
												      fieldstr);																		      												      
		//set contact's home number
    ab_contact_get_home_num	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_HOME_NUM,
												      fieldstr);	
		//set contact's business number
    ab_contact_get_business_num	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_BUSI_NUM,
												      fieldstr);	
		//set contact's fax number
    ab_contact_get_fax_num	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_FAXNUM,
												      fieldstr);	
		//set contact's company
    ab_contact_get_company	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_COMPANY,
												      fieldstr);
		//set contact's department
    ab_contact_get_department	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_DEPART,
												      fieldstr);
		//set contact's website
    ab_contact_get_website	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_WEBSITE,
												      fieldstr);		
		//set contact's email
    ab_contact_get_email	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_EMAIL,
												      fieldstr);											      												      													      												          
		//set contact's picture
    ab_contact_get_picture	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_PICTURE,
												      fieldstr);		
		//set contact's ringtone
    ab_contact_get_ringtone	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_RINGTONE,
												      fieldstr);													      
		//set contact's home street
    ab_contact_get_home_street	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_HADDR_STREET,
												      fieldstr);					      
		//set contact's business street
    ab_contact_get_business_street	(contact, &fieldstr);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
												      AB_IV_BADDR_STREET,
												      fieldstr);	
		//set contact's UID
		ab_uid_t id = AB_ID_INVALID;
		gchar *tmpuid = NULL;
  	tmpuid = g_new0(gchar, 6);
    g_sprintf (tmpuid, "%d",uid);
        
    g_print("%s() : uid = %d   \n",__FUNCTION__,uid);
    g_print("%s() : tmpuid = %s   \n",__FUNCTION__,tmpuid);
    gem_hash_view_set_value (GEM_HASH_VIEW(cntlistview),
											     	 AB_IV_UID,
											     	 tmpuid);	
}*/

void
ab_enabler_number_chooser_parse_from_contact (ABookNumberChooser *nc,
                                     			    const ab_contact_t *contact,int storage)
{
  GtkListStore *liststore = NULL;
  liststore = abook_number_chooser_get_model (nc);
// 	GtkWidget *window = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
//	gchar *storage = action_get_current_contact_storage(UI_COMP_SKWINDOW(window));

g_print("%s(): entering\n", __FUNCTION__);
  ab_contact_print (contact);

  if (TRUE)
  {
    gchar *m_num = NULL;
    gchar *h_num = NULL;
    gchar *b_num = NULL;
    GtkTreeIter iter;
    
    ab_fid_t fid = AB_ID_INVALID; 
    ab_ftid_t i = AB_IV_MOBILE_NUM;											 
		ab_contact_get_default_field(contact,i,&fid);      
		ab_contact_get_string_field(contact, fid, & m_num);
		if (m_num)
    {
      g_print ("%s(): mobile = %s\n", __FUNCTION__, m_num);
      gtk_list_store_append (liststore, &iter);
      gtk_list_store_set (liststore, &iter,
//                                                                                               COL_NC_TYPE, "<span color = \"Blue\">Mobile</span>",
                          COL_NC_TYPE, "Mobile", COL_NC_NUMBER, m_num, -1);
      g_free (m_num);
    }
    
g_print("%s() :storage = %d\n",__FUNCTION__,storage);		
		if((gint)storage==0)
		{
			i = AB_IV_HOME_NUM;											 
			ab_contact_get_default_field(contact,i,&fid);      
			ab_contact_get_string_field(contact, fid, & h_num);
	
			i = AB_IV_BUSINESS_NUM;											 
			ab_contact_get_default_field(contact,i,&fid);      
			ab_contact_get_string_field(contact, fid, & b_num);
		
	    if (h_num)
	    {
	      g_print ("%s(): home = %s\n", __FUNCTION__, h_num);
	      gtk_list_store_append (liststore, &iter);
	      gtk_list_store_set (liststore, &iter,
	//                                                                                               COL_NC_TYPE, "<span color = \"Blue\">Home</span>",
	                          COL_NC_TYPE, "Home", COL_NC_NUMBER, h_num, -1);
	      g_free (h_num);
	    }
	
	    if (b_num)
	    {
	      g_print ("%s(): business = %s\n", __FUNCTION__, b_num);
	      gtk_list_store_append (liststore, &iter);
	      gtk_list_store_set (liststore, &iter,
	//                                                                                               COL_NC_TYPE, "<span color = \"Blue\">Business</span>",
	                          COL_NC_TYPE, "Business", COL_NC_NUMBER, b_num, -1);
	      g_free (b_num);
	    }
		}
  }

  g_object_unref (liststore);

  abook_number_chooser_update (nc);

  return;
}

//void
//ab_mark_view_parse_contacts (UICompTreeViewCnt *view, gboolean flash_only)
//{
//  GtkTreeStore *tree = NULL;
//  enum
//  {
//    PIC_HOME,
//    PIC_OFFICE,
//    PIC_MOBILE,
//    NUM_PICS
//  };
//  gchar *pics[NUM_PICS] = {
//    "grp02.png",
//    "grp04.png",
//    "grp01.png",
//  };
//  GdkPixbuf *pixbufs[NUM_PICS] = {
//    NULL,
//    NULL,
//    NULL,
//  };
//
//  tree = abook_markview_get_model(view);
//
//  if (TRUE)
//  {
//    GtkTreeIter iter;
//
////              g_print("%s(): entering\n", __FUNCTION__);
//    gtk_tree_store_clear (tree);
//    gtk_tree_store_append (tree, &iter, NULL);
//    gtk_tree_store_set (tree, &iter,
//                        COL_MV_STRING, "Select All", COL_MV_CHECK, FALSE, -1);
//  }
//
//  if (TRUE)
//  {
//    gint i;
//
//    for (i = 0; i < NUM_PICS; i++)
//    {
//      gchar *picpath = NULL;
//
//      picpath = g_build_filename(gem_env_get_resource_path (), pics[i], NULL);
//
////                      g_print("%s(): path = %s\n", __FUNCTION__, picpath);
//      gem_resource_load_pic(picpath,
//														GEM_PIC_SMALL);
//      pixbufs[i] = gem_resource_find_pic (picpath, GEM_PIC_SMALL);
//
//      g_free (picpath);
//    }
//  }
//
//  if (TRUE)
//  {
//    ab_contact_t **results = NULL;
//
//    ab_get_contacts(ab_enabler_get_session(SESSION_LOCAL), NULL, &results);
//    if (results)
//    {
//      gint j;
//
//      for (j = 0; results[j] != NULL; j++)
//      {
//        gpointer uid = NULL;
//        gchar *name = NULL;
//        gchar *m_number = NULL;
//        gchar *h_number = NULL;
//        gchar *b_number = NULL;
//        gchar *pic = NULL;
//
//        ab_contact_get_int32_field(results[j], AB_IV_UID, (gpointer) & uid);
//        ab_contact_get_string_field(results[j], AB_IV_FULL_NAME, (gpointer) & name);
//        ab_contact_get_string_field(results[j], AB_IV_MOBILE_NUM, (gpointer) & m_number);
//        ab_contact_get_string_field(results[j], AB_IV_HOME_NUM, (gpointer) & h_number);
//        ab_contact_get_string_field(results[j], AB_IV_BUSINESS_NUM, (gpointer) & b_number);
//        ab_contact_get_string_field(results[j], AB_IV_PICTURE, (gpointer) & pic);
//
//        if (uid && name)
//        {
//          GtkTreeIter parnet_iter;
//
//          gtk_tree_store_append (tree, &parnet_iter, NULL);
//          gtk_tree_store_set (tree, &parnet_iter,
//                              COL_MV_STRING, name, COL_MV_CHECK, FALSE, -1);
//
//          if (TRUE)
//          {
//            gchar *uidstr = NULL;
//
//            //uidstr = g_new0 (gchar, 36 + 1);
//            uidstr = g_strdup_printf("%d",uid);
////            uuid_unparse (uid, uidstr);
//
//            gtk_tree_store_set (tree, &parnet_iter, COL_MV_UID, uidstr, -1);
//            g_free (uidstr);
//          }
//
//          if (FALSE)
//          {
//            GdkPixbuf *pixbuf = NULL;
//            gchar *picpath = NULL;
//
//            picpath = g_build_filename (gem_env_get_resource_path (), pic, NULL);
////                                              g_print("%s(): path = %s\n", __FUNCTION__, picpath);
//            gem_resource_load_pic (picpath, GEM_PIC_SMALL);
//            pixbuf = gem_resource_find_pic (picpath, GEM_PIC_SMALL);
//
//            gtk_tree_store_set (tree, &parnet_iter, COL_MV_PIC, pixbuf, -1);
//
////                                              g_object_unref(pixbuf);
//
//            g_free (picpath);
//          }
//
//          if (m_number != NULL && strlen (m_number) > 0)
//          {
//            GtkTreeIter iter;
//
//            gtk_tree_store_append (tree, &iter, &parnet_iter);
//            gtk_tree_store_set (tree, &iter,
//                                COL_MV_STRING, m_number,
//                                COL_MV_CHECK, FALSE,
//                                COL_MV_PIC, pixbufs[PIC_MOBILE], -1);
//
////                                              g_object_unref(pixbufs[PIC_MOBILE]);
//          }
//
//          if (h_number != NULL && strlen (h_number) > 0)
//          {
//            GtkTreeIter iter;
//
//            gtk_tree_store_append (tree, &iter, &parnet_iter);
//            gtk_tree_store_set (tree, &iter,
//                                COL_MV_STRING, h_number,
//                                COL_MV_CHECK, FALSE,
//                                COL_MV_PIC, pixbufs[PIC_HOME], -1);
//
////                                              (pixbufs[PIC_HOME]);
//          }
//
//          if (b_number != NULL && strlen (b_number) > 0)
//          {
//            GtkTreeIter iter;
//
//            gtk_tree_store_append (tree, &iter, &parnet_iter);
//            gtk_tree_store_set (tree, &iter,
//                                COL_MV_STRING, b_number,
//                                COL_MV_CHECK, FALSE,
//                                COL_MV_PIC, pixbufs[PIC_OFFICE], -1);
////                                              g_object_unref(pixbufs[PIC_OFFICE]);
//          }
//        }
//
//        g_free (uid);
//        g_free (name);
//        g_free (pic);
//        g_free (m_number);
//        g_free (h_number);
//        g_free (b_number);
//
////                              ab_contact_print(results[j]);
//        ab_contact_free (results[j]);
//      }
//
//      g_free (results);
//    }
//  }
//
//  if (flash_only)
//  {
//    ab_contact_t **results = NULL;
//
//    ab_get_contacts (ab_enabler_get_session(SESSION_LOCAL), NULL, &results);
//    if (results)
//    {
//      gint j;
//
//      for (j = 0; results[j] != NULL; j++)
//      {
//        gpointer uid = NULL;
//        gchar *name = NULL;
//        gchar *number = NULL;
//
//        ab_contact_get_string_field (results[j], AB_IV_UID, (gpointer) & uid);
//        ab_contact_get_string_field (results[j], AB_IV_LAST_NAME, (gpointer) & name);
//        ab_contact_get_string_field (results[j], AB_IV_MOBILE_NUM,
//                              (gpointer) & number);
//
//        if (uid && name)
//        {
//          GtkTreeIter parnet_iter;
//
//          gtk_tree_store_append (tree, &parnet_iter, NULL);
//          gtk_tree_store_set (tree, &parnet_iter,
//                              COL_MV_STRING, name, COL_MV_CHECK, FALSE, -1);
//
//          if (TRUE)
//          {
//            gchar *uidstr = NULL;
//
//            uidstr = g_strdup_printf("%d",uid);
//
//            gtk_tree_store_set (tree, &parnet_iter, COL_MV_UID, uidstr, -1);
//            g_free (uidstr);
//          }
//
//          if (number != NULL && strlen (number) > 0)
//          {
//            GtkTreeIter iter;
//
//            gtk_tree_store_append (tree, &iter, &parnet_iter);
//            gtk_tree_store_set (tree, &iter,
//                                COL_MV_STRING, number,
//                                COL_MV_CHECK, FALSE,
//                                COL_MV_PIC, pixbufs[PIC_MOBILE], -1);
////                                              g_object_unref(pixbufs[PIC_MOBILE]);
//          }
//        }
//
//        g_free (uid);
//        g_free (name);
//        g_free (number);
//
////                              ab_contact_print(results[j]);
//        ab_contact_free (results[j]);
//      }
//
//      g_free (results);
//    }
//  }
//  g_object_unref (tree);
//
//  abook_markview_update (view);
//
//  return;
//}


void
ab_enabler_cat_list_view_load_category(GtkWidget *widget)
{
//	ab_thread_t	*thread = (ab_thread_t *)arg;
	UICompCntListView *cntview = UI_COMP_CNT_LIST_VIEW(widget);
//	GemNotification *notifier = GEM_NOTIFICATION(thread->notifier);
	ab_category_t **categories = NULL;
	gint i;
	gint count;
		
	ab_get_categories(AB_ENABLER_SESS_LOCAL,
										NULL,
										&categories);
	
/*	g_print("%s(): contacts = 0x%08x\n",
					__FUNCTION__,
					contacts);
*/
	if (categories)
	{
		for (i = 0, count = 0; categories[i] != NULL; i++)
		{
			count++;			
		}
		
		for (i = 0; categories[i] != NULL; i++)
		{
			g_print("%s(): (local)i = %d\n",
							__FUNCTION__,
							i);											 

			ab_enabler_ui_comp_cat_list_view_add(cntview,
  																				 categories[i]);		
	    ui_comp_cnt_list_view_update(cntview);	

		}
	}
}

void 
ab_enabler_ui_comp_cat_list_view_add (UICompCntListView			*cntview,
																			const ab_category_t		*category)
{
	return ab_enabler_ui_comp_cat_list_view_update(cntview,
																								 category,
																								 NULL);
}

void
ab_enabler_ui_comp_cat_list_view_update (UICompCntListView		*cntview,
																				 const ab_category_t	*category,
																				 GtkTreeIter					*iter)
{
//	g_print("%s(): entering ......\n",__FUNCTION__);
	GtkTreeModel * model = NULL;

	model = gem_tree_view_get_model (GEM_TREE_VIEW(cntview));
	if (model)
	{
		ab_uid_t uid = AB_ID_INVALID;
		gchar *name = NULL;
		gchar *picture = NULL;
		GdkPixbuf *pixbuf = NULL;
		
		ab_category_get_uid (category, &uid);
		ab_category_get_name (category, &name);
		ab_category_get_picture (category,&picture);   
		
		if (TRUE)
		{
			gchar * picpath = NULL;
			
			picpath = g_build_filename (gem_env_get_resource_path(),
																  "pixmaps",
														  	   picture,
																   NULL);
g_print("%s(): picpath = %s \n",__FUNCTION__,picpath);																   
			gem_resource_load_pic(picpath,
														GEM_PIC_NORMAL);
			pixbuf = gem_resource_find_pic(picpath,
																		 GEM_PIC_NORMAL);
	
			g_free(picpath);
		} 

		if (iter)
		{
			gtk_list_store_set(GTK_LIST_STORE(model), 
												 iter,
												 UI_COMP_CAT_LV_UID,
												 uid,
												 UI_COMP_CAT_LV_NAME,
												 name,
												 UI_COMP_CAT_LV_PIC,
												 pixbuf,
												 -1);
			
		}
		else
		{
			GtkTreeIter iter_new;
			
			gtk_list_store_append(GTK_LIST_STORE(model),
														&iter_new);
			gtk_list_store_set(GTK_LIST_STORE(model),
												 &iter_new,
												 UI_COMP_CAT_LV_UID,
												 uid,
												 UI_COMP_CAT_LV_NAME,
												 name,
												 UI_COMP_CAT_LV_PIC,
												 pixbuf,
												 -1);
					
		}		
		g_free(name);
	}
	
	return;
}

void
ab_enabler_ui_comp_tree_view_cnt_parse_contacts (UICompMarkView *cnt_treeview, gboolean flash_only)
{
  GtkTreeStore *tree = NULL;
  enum
  {
    PIC_HOME,
    PIC_OFFICE,
    PIC_MOBILE,
    NUM_PICS
  };
  gchar *pics[NUM_PICS] = {
    "grp02.png",
    "grp04.png",
    "grp01.png",
  };
  GdkPixbuf *pixbufs[NUM_PICS] = {
    NULL,
    NULL,
    NULL,
  };
	tree = gem_tree_view_get_model(GEM_TREE_VIEW(cnt_treeview));

  if (TRUE)
  {
    ab_contact_t **results = NULL;
		ab_get_contacts(AB_ENABLER_SESS_LOCAL,
										NULL,
										&results);
    if (results)
    {
      gint j;

      for (j = 0; results[j] != NULL; j++)
      {
        ab_uid_t uid = AB_ID_INVALID;
        gchar *name = NULL;
        gchar *m_number = NULL;
        gchar *h_number = NULL;
        gchar *b_number = NULL;

				ab_contact_get_uid (results[j], &uid);
				ab_contact_get_full_name (results[j],&name);
				ab_contact_get_mobile_num (results[j], &m_number);
				ab_contact_get_home_num (results[j], &h_number);
				ab_contact_get_business_num (results[j], &b_number);
        if (uid && name)
        {
          GtkTreeIter parnet_iter;

          gtk_tree_store_append (tree, &parnet_iter, NULL);
          gtk_tree_store_set (tree, &parnet_iter,
                              MARK_COL_MV_STRING, name, MARK_COL_MV_CHECK, FALSE, -1);

          if (TRUE)
          {
            gchar *uidstr = NULL;
						uidstr = g_strdup_printf("%d",uid);
            gtk_tree_store_set (tree, &parnet_iter, COL_MV_UID, uidstr,MARK_COL_MV_STORAGE,STORAGE_LOCAL, -1);
           // g_free (uidstr);
          }

          if (m_number != NULL && strlen (m_number) > 0)
          {
            GtkTreeIter iter;

            gtk_tree_store_append (tree, &iter, &parnet_iter);
            gtk_tree_store_set (tree, &iter,
                                MARK_COL_MV_STRING, m_number,
                                MARK_COL_MV_CHECK, FALSE,
                                -1);
          }

          if (h_number != NULL && strlen (h_number) > 0)
          {
            GtkTreeIter iter;

            gtk_tree_store_append (tree, &iter, &parnet_iter);
            gtk_tree_store_set (tree, &iter,
                                MARK_COL_MV_STRING, h_number,
                                MARK_COL_MV_CHECK, FALSE,
                                -1);
          }
          if (b_number != NULL && strlen (b_number) > 0)
          {
            GtkTreeIter iter;

            gtk_tree_store_append (tree, &iter, &parnet_iter);
            gtk_tree_store_set (tree, &iter,
                                MARK_COL_MV_STRING, b_number,
                                MARK_COL_MV_CHECK, FALSE,
                                -1);
          }
        }
        g_free (name);
        g_free (m_number);
        g_free (h_number);
        g_free (b_number);
        ab_contact_free (results[j]);
      }

      g_free (results);
    }
  }
  
  if(TRUE)
  {
  	ab_contact_t **contacts_sim = NULL;
		ab_get_contacts(AB_ENABLER_SESS_SIM,
										NULL,
										&contacts_sim);
		 if (contacts_sim)
    {
      gint j;

      for (j = 0; contacts_sim[j] != NULL; j++)
      {
      	ab_uid_t uid = AB_ID_INVALID;
        gchar *name = NULL;
        gchar *m_number = NULL;

				ab_contact_get_uid (contacts_sim[j], &uid);
				ab_contact_get_full_name (contacts_sim[j],&name);
				ab_contact_get_mobile_num (contacts_sim[j], &m_number);
        if (uid && name)
        {
          GtkTreeIter parnet_iter;

          gtk_tree_store_append (tree, &parnet_iter, NULL);
          gtk_tree_store_set (tree, &parnet_iter,
                              MARK_COL_MV_STRING, name, MARK_COL_MV_CHECK, FALSE, -1);

          if (TRUE)
          {
            gchar *uidstr = NULL;
						uidstr = g_strdup_printf("%d",uid);
            gtk_tree_store_set (tree, &parnet_iter, COL_MV_UID, uidstr, MARK_COL_MV_STORAGE,STORAGE_SIM,-1);
           // g_free (uidstr);
          }

          if (m_number != NULL && strlen (m_number) > 0)
          {
            GtkTreeIter iter;

            gtk_tree_store_append (tree, &iter, &parnet_iter);
            gtk_tree_store_set (tree, &iter,
                                MARK_COL_MV_STRING, m_number,
                                MARK_COL_MV_CHECK, FALSE,
                                -1);
          }

        }
        g_free (name);
        g_free (m_number);
        ab_contact_free (contacts_sim[j]);
      }

      g_free (contacts_sim);
    }
  }
  ui_comp_mark_view_update (cnt_treeview);
  return;
}

void
ab_enabler_ui_comp_tree_view_cnt_parse_category (UICompTreeViewCnt *cnt_treeview, gboolean flash_only)
{
  GtkTreeStore *tree = NULL;
	tree = gem_tree_view_get_model(GEM_TREE_VIEW(cnt_treeview));

  if (TRUE)
  {
    ab_category_t **categories = NULL;

		ab_get_categories(AB_ENABLER_SESS_LOCAL,
											NULL,
											&categories);
    if (categories)
    {
      gint j;
      int i ;
      gchar *category_name;
      gchar *contact_name;
  		GdkPixbuf *pixbuf = NULL;
      gchar *pic;

      for (j = 0; categories[j] != NULL; j++)
      {
        ab_uid_t uid = AB_ID_INVALID;
      	ab_category_get_uid (categories[j],&uid);
        gchar *uidstr = NULL;
				uidstr = g_strdup_printf("%d",uid);
				g_print("%s(): category uid  (%d)\n",__FUNCTION__,uid);
				
      	ab_category_get_name (categories[j], &category_name);
      	ab_category_get_picture (categories[j],&pic);   

				if(TRUE)
				{
					gchar * picpath = NULL;
					picpath = g_build_filename (gem_env_get_resource_path(),
																		  "pixmaps",
																  	   pic,
																		   NULL);
					gem_resource_load_pic(picpath,
																GEM_PIC_NORMAL);
					pixbuf = gem_resource_find_pic(picpath,
																				 GEM_PIC_NORMAL);
			
					g_free(pic);
				} 
      
	 			ab_contact_t **contacts = NULL;
	    	if(flash_only)
	    	{
					ab_get_contacts_in_category(AB_ENABLER_SESS_LOCAL,
																			uid,
																			&contacts);
	    	}
	      GtkTreeIter parnet_iter;
	      gtk_tree_store_append (tree, &parnet_iter, NULL);
	      gtk_tree_store_set (tree, &parnet_iter,
	                          COL_MV_STRING, category_name,
	                          COL_MV_UID, uidstr,
                            COL_MV_PIC, pixbuf, -1);
				if (contacts)
				{
					for (i = 0; contacts[i] != NULL; i++)
					{
						ab_contact_get_full_name (contacts[i],&contact_name);
						
						ab_contact_get_picture (contacts[i],&pic);   

						if(TRUE)
						{
							gchar * picpath = NULL;
							
							picpath = g_build_filename (gem_env_get_resource_path(),
																				  "pixmaps",
																		  	   pic,
																				   NULL);
							gem_resource_load_pic(picpath,
																		GEM_PIC_NORMAL);
							pixbuf = gem_resource_find_pic(picpath,
																						 GEM_PIC_NORMAL);
					
							g_free(pic);
						} 
						
						if (uid && category_name)
		        {
		          if (TRUE)
		          {
		            gchar *uidstr = NULL;
								uidstr = g_strdup_printf("%d",uid);
		            gtk_tree_store_set (tree, &parnet_iter, COL_MV_UID, uidstr, -1);

		          }

							if (contact_name != NULL )
		          {
		            GtkTreeIter iter;
		            gtk_tree_store_append (tree, &iter, &parnet_iter);
		            gtk_tree_store_set (tree, &iter,
		                                COL_MV_STRING, contact_name,
		                                COL_MV_PIC, pixbuf, -1);
		          }		
		        }
          	g_free (contact_name);

					}
					ab_free_contacts(AB_ENABLER_SESS_LOCAL, contacts);					
				}			
      }

      g_free (categories);

    }
  }
  ui_comp_tree_view_cnt_update (cnt_treeview);

  return;
}

void
ab_enabler_ui_comp_tree_view_parse_selection_for_contact (UICompTreeViewCnt *view,
                                            							const gchar *uidstr)
{
  GtkTreeStore *tree = NULL;
	tree = gem_tree_view_get_model(GEM_TREE_VIEW(view));

  if (uidstr)
  {
    ab_category_t **results = NULL;

		ab_get_categories_for_contact (AB_ENABLER_SESS_LOCAL,atoi(uidstr),&results);                                     
    if (results)
    {
      gint i;
      gchar *category_name;
      gchar *contact_name;
      
      for (i = 0; results[i] != NULL; i++)
      {
	      ab_uid_t uid = AB_ID_INVALID;
      	ab_category_get_uid (results[i],&uid);
        gchar *uidstr = NULL;
				uidstr = g_strdup_printf("%d",uid);
				g_print("%s(): category uid  (%d)\n",__FUNCTION__,uid);
				
      	ab_category_get_name (results[i], &category_name);

        if (category_name)
        {
          gint n_children = 0;
          gint j;

          n_children =
            gtk_tree_model_iter_n_children (GTK_TREE_MODEL (tree), NULL);
          for (j = 1; j < n_children; j++)
          {
            GtkTreeIter iter;
            gchar *compname = NULL;
            gboolean ret = FALSE;

            gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (tree), &iter, NULL,
                                           j);
            gtk_tree_model_get (GTK_TREE_MODEL (tree), &iter, COL_MV_STRING,
                                &compname, -1);

            ret = (g_strcasecmp (category_name, compname) == 0);
            g_free (compname);

            if (ret)
            {
              gtk_tree_store_set (tree, &iter, COL_MV_CHECK, TRUE, -1);
              break;
            }
          }
        }

        g_free (category_name);
        ab_category_free (results[i]);
      }

      g_free (results);
    }
  }
  ui_comp_tree_view_cnt_update (view);
  return;
}

void ab_enabler_ui_comp_main_window(UICompInitView *mainview)
{
  gchar *pic[] = 
  {
	  "icon05.png",
	  "grp00.png",
		"icon03.png"
 	};
 	gchar *name[]=
 	{
 		"Contact",
 		"Category",
 		"Memory State"
 	};
 	GdkPixbuf *pixbuf = NULL;

 	gchar *picpath = NULL;
  GtkTreeStore *tree = NULL;
	tree = gem_tree_view_get_model(GEM_TREE_VIEW(mainview));
	GtkTreeIter iter;
 	int i =0;
 	
 	for(i=0; i< sizeof(pic)/sizeof(pic[0]); i++)
 	{
		picpath = g_build_filename (gem_env_get_resource_path(),
															  "pixmaps",
													  	   pic[i],
															   NULL);
		gem_resource_load_pic(picpath,
													GEM_PIC_NORMAL);
		pixbuf = gem_resource_find_pic(picpath,
																	 GEM_PIC_NORMAL);	
		gchar * uid = g_strdup_printf("%d",(i+1));
		
		gtk_list_store_append(GTK_LIST_STORE(tree),
													&iter);
		
	  gtk_list_store_set (GTK_LIST_STORE(tree),
	  									  &iter,
	                      UI_COMP_CNT_LV_NAME, name[i],
	                      UI_COMP_CNT_LV_UID, uid,
	                      UI_COMP_CNT_LV_PIC, pixbuf,
	                      -1);
//	 gchar * uidstr = NULL;	                      
//	 gtk_tree_model_get(tree,
//	  									&iter,
//	  									UI_COMP_CNT_LV_UID, &uidstr,-1);
//	 g_print("%s() uidstr(%s)\n",__FUNCTION__,uidstr);	
 	}                      
}

void
ab_enabler_ui_comp_mark_view_parse_field(UICompInitView *treeview,gchar *uid)
{
	GtkTreeStore *tree = NULL;
	tree = gem_tree_view_get_model(GEM_TREE_VIEW(treeview));

  if (TRUE)
  {
  	ab_ftid_t *ftids;
		gchar * label = NULL;
    ab_fid_t *fids = NULL;
		ab_contact_t *contact = NULL;
		ab_get_contact (AB_ENABLER_SESS_LOCAL,
										uid,
										&contact);

		if(contact)	
			ab_list_field_templates	(ab_enabler_get_session(SESSION_LOCAL),&ftids);
											
    if (ftids)
    {
      gint j;
      gint i ;
      gchar *label;
			gint attr;
			
      for (j = 0; ftids[j]!=NULL; j++)
      {
      	ab_field_template_get_attributes(ab_enabler_get_session(SESSION_LOCAL),ftids[j],&attr);
      	if(attr != FIELD_ATTR_MULTIPLE)
      	{
      		continue;
      	}
//      	if(ftids[j]==1 ||ftids[j]==3 || ftids[j]==15 || ftids[j]==16 )
//      		continue;
      	ab_field_template_get_label  (ab_enabler_get_session(SESSION_LOCAL),ftids[j],&label);
        gint extend = 	FIELD_MUL;
				
	      GtkTreeIter parnet_iter;
	      gtk_tree_store_append (tree, &parnet_iter, NULL);
	      gtk_tree_store_set (tree, &parnet_iter,
	                          MARK_COL_MV_STRING, label,
	                          MARK_COL_MV_UID, ftids[j],
	                          MARK_COL_MV_EXTEND, extend,
                            -1);
				ab_contact_list_fields_by_template(contact,ftids[j],&fids);  
				if (fids)
				{
					for (i = 0; fids[i] != NULL; i++)
					{
						gchar *fid_label = NULL;
						ab_contact_get_field_label (contact,fids[i],&fid_label);
						gchar *label_style = NULL;
						
						if(fid_label)
		        {
	        		if(ab_enabler_field_whether_default(contact,ftids[j],fids[i]))
	        		{
	        			label_style = g_strdup_printf("%s%s%s","<span color = \"Gray\">",fid_label,"</span>");
	        			extend = 	FIELD_NONE;

	        		}
	        		else 
	        		{
	        			label_style = g_strdup_printf("%s",fid_label);
	        			extend = 	FIELD_EXTEND;
	        		}
	            GtkTreeIter iter;
	            gtk_tree_store_append (tree, &iter, &parnet_iter);
	            gtk_tree_store_set (tree, &iter,
	            										MARK_COL_MV_UID, fids[i],
	                                MARK_COL_MV_STRING, label_style,
	                                MARK_COL_MV_EXTEND, extend,
	                                -1);
		        }
          	g_free (fid_label);
          	g_free (label_style);
          	

					}
				}
      }
    }
  }
  ui_comp_mark_view_update(treeview);

  return;
}

gboolean 
ab_enabler_field_whether_default(ab_contact_t	*contact,ab_ftid_t ftid,ab_fid_t	*fid)
{
	ab_fid_t	fid_default ;
	ab_contact_get_default_field(contact,ftid,&fid_default);
	if(fid_default == fid)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

void
ab_enabler_cnt_list_hash_table_update(UICompCntListView *cntview)
{
	GtkTreeModel *model = NULL;
	GtkTreeIter iter; 
	model = gem_tree_view_get_model(GEM_TREE_VIEW(cntview));
	if(gtk_tree_model_get_iter_first(model,&iter))
	{
		gchar *uid = NULL;
		do
		{
			gtk_tree_model_get(model, 
												 &iter,
												 UI_COMP_CNT_LV_UID,
												 &uid,
												 -1);
			if(uid)
			{
				ui_comp_cnt_list_hash_table_insert(cntview,&uid,gtk_tree_iter_copy(&iter));
			}												 
		}while(gtk_tree_model_iter_next(model,&iter));
	}
	 
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
