/*	contact - LiPS Address Book Application
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdio.h>
#include <glib.h>
#include <gdk/gdkkeysyms.h>
#include <gem.h>
#include <gemhashview.h>
#include <gemnumchooser.h>
#include "enabler.h"
#include "internal.h"
#include "threads.h"
#include <uicompmainwindow.h>
#include <uicompskwindow.h>
#include <uicompinitview.h>
#include <uicompcntlistview.h>
#include <uicompcnteditview.h>  
#include <uicompcnteditviewsim.h>
#include <uicompwinmgr.h>
#include <dbus.h>
#include <uicompcatcallbacks.h>
#include <uicomptreeviewcnt.h>
#include <uicompmarkview.h>
#include <uicompfieldview.h>
#include <field-desc.h>


static gboolean picture_flag = FALSE;
static gboolean ringtone_flag = FALSE;

static void ui_comp_file_chooser_selected_cb(GemFileChooser *fileview,gchar *kind);
static void ui_comp_file_chooser_softkey_activated_cb(GpeSKWindow *window,
																											gint nr,
																											gchar *kind);
static void ui_comp_field_manage_add();							
static void ui_comp_field_manage_modify();													
static void ui_comp_field_manage_delete();				
static void ui_comp_field_manage_deleteall();		
static void ui_comp_send_msg_multi();
static void ui_comp_delete_cnt_multi();	
void ui_comp_mem_status();
																											
GtkWidget *ui_comp_cnt_list_window_new();


static gchar * uidesc_field = " \
	<ui>\
		<popup  action=\"PopupLeft_M\">\
			<menuitem name=\"add\"						action=\"Add\"/>\
			<menuitem name=\"delete_all\"			action=\"DeleteAll\"/>\
		</popup>\
		<popup  action=\"PopupLeft_E\">\
			<menuitem name=\"modify\"					action=\"Modify\"/>\
			<menuitem name=\"delete\"					action=\"Delete\"/>\
		</popup>\
		<popup  action=\"PopupLeft\">\
			<menuitem name=\"send\"					action=\"Send_Multi\"/>\
			<menuitem name=\"delete\"					action=\"Delete_Multi\"/>\
		</popup>\
	</ui>\
";

static GtkActionEntry actions_field[] = {
	{"Add",		      	NULL, 	"_Add",	      	"", "none",ui_comp_field_manage_add},
  {"DeleteAll",			NULL, 	"_DeleteAll",		"", "none",ui_comp_field_manage_deleteall},
  {"Modify",	  		NULL,	  "_Modify",	  	"", "none",ui_comp_field_manage_modify},
	{"Delete",				NULL,		"_Delete",	    "",	"none",ui_comp_field_manage_delete},
	{"Send_Multi",		NULL,		"_Send Message","",	"none",ui_comp_send_msg_multi},
	{"Delete_Multi",	NULL,		"_Delete",	    "",	"none",ui_comp_delete_cnt_multi},
	{"PopupLeft",NULL, 	"Popup",				"", "none",NULL},
	{"PopupLeft_M",		NULL, 	"Popup",				"", "none",NULL},
	{"PopupLeft_E",		NULL, 	"Popup",				"", "none",NULL},
};

static void
main_window_softkey_activated_cb (GpeSKWindow		*window,
																	gint					 nr,
																	gpointer			 user_data)
{
	if(nr == UI_COMP_SK_LEFT)
	{
		gchar *uid = action_get_current_contact_uid(UI_COMP_SKWINDOW(window));
		GtkWidget *c_window = NULL;
		GtkWidget *view = NULL;
		g_print("%s() LINE(%d) uid(%d) \n",__FUNCTION__,__LINE__,atoi(uid));
		switch(atoi(uid))
		{
			case 1:
							c_window = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
							if(c_window==NULL)
							{
								c_window = ui_comp_cnt_list_window_new();
							}
							else
							{
								view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(c_window));	
								ui_comp_init_view_update (view);
							}
							gtk_widget_show_all(c_window);
							break;
			case 2:
						  action_cat_show_category_cb();
							break;
			case 3: ui_comp_mem_status();
							break;
		}
	}
	else if (nr == UI_COMP_SK_RIGHT)
	{
		GemApplication *app = gem_application_get_instance();		
		gem_application_quit(app);
	}
}
static void 
softkey_activated_field_manage_cb(GpeSKWindow		*window,
																	gint					 nr,
																	gpointer			 user_data)
{
	if(nr==UI_COMP_SK_LEFT )
	{
		const gchar *left_label =	gpe_skwindow_get_softkey_label (window, UI_COMP_SK_LEFT);
  	g_print("%s() left_label(%s)\n ",__FUNCTION__,left_label);

  	if( g_strcasecmp(left_label,"Option")==0)
  	{
			gpe_skwindow_set_softkey_label(GPE_SKWINDOW(window),UI_COMP_SK_RIGHT,"Cancel");
  	}
  	else if (g_strcasecmp(left_label,"Add")==0)
  	{
  		ui_comp_field_manage_add();
  	}
	}
	else if (nr == UI_COMP_SK_RIGHT)
	{
		gtk_widget_destroy(GTK_WIDGET(window));
	}	
	return;
	
}


static void
softkey_activated_cb (GpeSKWindow		*window,
											gint					 nr,
											gpointer			 user_data)
{
	g_print("%s(): entering\n",
					__FUNCTION__);
	if (nr == UI_COMP_SK_LEFT)
	{
		gchar *storage = NULL;
		GtkWidget *listwindow = NULL;
		listwindow = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
		if(listwindow == NULL)
			return;
		storage = action_get_current_contact_storage (UI_COMP_SKWINDOW(listwindow));
			
		GtkUIManager *uimanager = ui_comp_skwindow_get_ui_manager(UI_COMP_SKWINDOW(window));
		GtkAction *action;
		action = gtk_ui_manager_get_action(uimanager,"/PopupLeft/management/field_manage");
		
		int i = 0;
		if((gint)storage == 1 )
		{
			g_print("%s() LINE(%d) action[%d]=0x%08x\n",__FUNCTION__,
																									__LINE__,
																									i,
																									(guint32)action);
	    if (action)
	    {
				g_print("%s() LINE(%d) i = %d \n",__FUNCTION__,__LINE__,i);		    	
	      g_object_set (G_OBJECT (action),
	                    "visible",
	                    FALSE,
											NULL);
	    }
		}
		else
		{
			g_print("%s() LINE(%d) action[%d]=0x%08x\n",__FUNCTION__,
																									__LINE__,
																									i,
																									(guint32)action);
	    if (action)
	    {
				g_print("%s() LINE(%d) i = %d \n",__FUNCTION__,__LINE__,i);		    	
	      g_object_set (G_OBJECT (action),
	                    "visible",
	                    TRUE,
											NULL);
	    }
		
		}
		g_print("%s(): left activated\n",
						__FUNCTION__);
		gpe_skwindow_set_softkey_label(GPE_SKWINDOW(window),UI_COMP_SK_RIGHT,"Cancel");
	}
	else if (nr == UI_COMP_SK_RIGHT)
	{
		gtk_widget_hide_all(GTK_WIDGET(window));
	}	
	return;
}
static void
softkey_activated_mem_check_cb(GpeSKWindow		*window,
															gint					 nr,
															gpointer			 user_data)
{
	if (nr == UI_COMP_SK_LEFT)
	{
		g_print("%s(): left activated\n",
						__FUNCTION__);
	}
	else if (nr == UI_COMP_SK_RIGHT)
	{
		gtk_widget_destroy(GTK_WIDGET(window));
	}	
	return;
}

static void 
softkey_activated_treeview_cb(GpeSKWindow		*window,
															gint					 nr,
															gpointer			 user_data)
{
	g_print("%s(): entering\n",
					__FUNCTION__);
	if (nr == UI_COMP_SK_LEFT)
	{
		g_print("%s(): left activated\n",
						__FUNCTION__);
//		gpe_skwindow_set_softkey_label(GPE_SKWINDOW(window),UI_COMP_SK_RIGHT,"Cancel");
	}
	else if (nr == UI_COMP_SK_RIGHT)
	{
		gtk_widget_hide_all(GTK_WIDGET(window));
	}	
	return;
}
static void
softkey_activated_add_to_cat_cb(GpeSKWindow		*window,
																gint					 nr,
																gpointer			 user_data)
{
	if(nr == UI_COMP_SK_LEFT)
	{
		UICompTreeViewCnt *cnt_treeview = UI_COMP_TREE_VIEW_CNT(user_data);
		ab_uid_t cnt_uid = AB_ID_INVALID;
		GtkWidget *cnt_window = NULL;
	  GtkTreeModel *model = NULL;
		GtkTreeIter iter;
		gint i = 0;
		model = gem_tree_view_get_model(GEM_TREE_VIEW(cnt_treeview));
		gint child_count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
		
		cnt_window = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
	  cnt_uid = (gint)action_get_current_contact_uid (UI_COMP_SKWINDOW(cnt_window));
	  
		for(i=1;i<child_count;i++)
		{
        gchar *name = NULL;
        gchar *cat_uid = NULL;
        gboolean checked = FALSE;
        gtk_tree_model_iter_nth_child (model, &iter, NULL, i);
        gtk_tree_model_get (model, &iter,
                            COL_MV_UID, &cat_uid,
                            COL_MV_CHECK, &checked, -1);
				if(checked)      
				{
					g_print("%s(): cnt_uid(%d)  cat_uid(%s)\n",__FUNCTION__,cnt_uid,cat_uid);
				  if(cnt_uid!=AB_ID_INVALID && cat_uid!=AB_ID_INVALID)
				  { 
						ab_ref_contact_category(ab_enabler_get_session(SESSION_LOCAL),
																		cnt_uid,
																		atoi(cat_uid));	
					}     
				}
				else
				{
					ab_unref_contact_category(ab_enabler_get_session(SESSION_LOCAL),
																		cnt_uid,
																		(ab_uid_t)cat_uid);
				}                 
		}
	}
	else if (nr == UI_COMP_SK_RIGHT)
	{
		
	}
	gtk_widget_hide_all(GTK_WIDGET(window));
}

gchar *
action_get_current_contact_uid (UICompSKWindow *window)
{
	GtkWidget *view =NULL;
	GtkTreeModel *model = NULL;
 	GtkTreePath *path = NULL;
  GtkTreeViewColumn *col = NULL;
  gchar *uid = NULL;
  GtkTreeIter iter;
	
	view = ui_comp_skwindow_get_view(window);
	g_print("%s(): window = 0x%08x, view = 0x%08x\n",
					__FUNCTION__,
					(guint32)window,
					(guint32)view);		 	     
	model = gem_tree_view_get_model(GEM_TREE_VIEW(view));
	gtk_tree_view_get_cursor ((GtkTreeView *)gem_tree_view_get_view(GEM_TREE_VIEW(view)), &path, &col);
/*	g_print("%s(): model = 0x%08x ,path = 0x%08x, col = 0x%08x\n",
					__FUNCTION__,
					(guint32)model,
					(guint32)path,
					(guint32)col);*/
	
	if (gtk_tree_model_get_iter (model, &iter, path))
  {	
  	 gtk_tree_model_get (model, &iter, UI_COMP_CNT_LV_UID, &uid, -1);
// 		 g_print("%s():  ing  uid = %d ...\n",__FUNCTION__,uid);
     gtk_tree_path_free (path);  	 
  }
  return uid;
}

static gchar *
action_get_current_contact_storage (UICompSKWindow *window)
{
	GtkWidget *view =NULL;
	GtkTreeModel *model = NULL;
 	GtkTreePath *path = NULL;
  GtkTreeViewColumn *col = NULL;
  gchar *storage = NULL;
  GtkTreeIter iter;

	view = ui_comp_skwindow_get_view(window);
	model = gem_tree_view_get_model(GEM_TREE_VIEW(view));
	gtk_tree_view_get_cursor ((GtkTreeView *)gem_tree_view_get_view(GEM_TREE_VIEW(view)), &path, &col);
  if (gtk_tree_model_get_iter (model, &iter, path))
  {
  	gtk_tree_model_get (model, &iter, UI_COMP_CNT_LV_STORAGE,
											 	&storage, -1);
 		gtk_tree_path_free (path);  	  
  }  
  return storage;
}

static void
ui_comp_file_chooser_softkey_activated_cb(GpeSKWindow *window,
																					gint nr,
																					gchar *kind)
{
	GtkWidget *fileview = NULL;
	fileview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));	
	if(nr == UI_COMP_SK_LEFT)
	{
		GList *select_list = NULL;
		gchar *picpath = NULL;
		gchar *chooser = NULL;
		GtkWidget *EditWindow = NULL;
		GtkWidget *editview = NULL;
		select_list = gem_file_chooser_get_current_selections(GEM_FILE_CHOOSER(fileview));
		if(select_list==NULL || g_list_length (select_list) ==0)
		{
			return;
		}
		
		g_print("%s(): g_list_length (select_list) = %d \n",__FUNCTION__,g_list_length (select_list));
		
		chooser = g_list_nth_data (select_list, 0);
		
		picpath = g_build_filename(gem_file_chooser_get_current_path(GEM_FILE_CHOOSER(fileview)),
														   chooser,
														   NULL);
		EditWindow = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW_EDIT);
	  editview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(EditWindow));	
	 				 											 
  	if(chooser!=NULL && (g_strcasecmp(kind, "pic") == 0))
  	{
  		gem_hash_view_set_value	(GEM_HASH_VIEW(editview),AB_IV_PICTURE*1000,chooser);
  	} 
  	
  	if(chooser!=NULL && (g_strcasecmp(kind, "ring") == 0))
  	{
  		gem_hash_view_set_value	(GEM_HASH_VIEW(editview),AB_IV_RINGTONE*1000,chooser);
  	}	
	}
	else if (nr == UI_COMP_SK_RIGHT)
	{
	}
	
	gtk_widget_hide_all (GTK_WIDGET (window));
	g_object_disconnect(G_OBJECT(window),
										 "any_signal::softkey_activated",
										  ui_comp_file_chooser_softkey_activated_cb,
										  kind,
										  NULL);
	g_object_disconnect(G_OBJECT(fileview),
										 "any_signal::file_selected",
											ui_comp_file_chooser_selected_cb,
											kind,
											NULL);	
}

static void 
ui_comp_file_chooser_selected_cb(GemFileChooser *fileview,gchar *kind)
{
	GList *select_list = NULL;
	gchar *picpath = NULL;
	gchar *chooser = NULL;
	GtkWidget *EditWindow = NULL;
	GtkWidget *editview = NULL;
	GtkWidget *filewindow = NULL;
	filewindow = ui_comp_winmgr_get_window(UI_COMP_SKW_FILE_CHOOSER);
	select_list = gem_file_chooser_get_current_selections(fileview);
	if(select_list==NULL || g_list_length (select_list) ==0)
	{
		return;
	}
	
	g_print("%s(): g_list_length (select_list) = %d \n",__FUNCTION__,g_list_length (select_list));
	
	chooser = g_list_nth_data (select_list, 0);
	
	picpath = g_build_filename(gem_file_chooser_get_current_path(GEM_FILE_CHOOSER(fileview)),
													   chooser,
													   NULL);
	EditWindow = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW_EDIT);
  editview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(EditWindow));	

	if(chooser!=NULL && (g_strcasecmp(kind, "pic") == 0))
	{
		gem_hash_view_set_value	(GEM_HASH_VIEW(editview),AB_IV_PICTURE*1000,chooser);
	} 
	
	if(chooser!=NULL && (g_strcasecmp(kind, "ring") == 0))
	{
		gem_hash_view_set_value	(GEM_HASH_VIEW(editview),AB_IV_RINGTONE*1000,chooser);
	}	

	gtk_widget_hide_all (GTK_WIDGET (filewindow));
	g_object_disconnect(G_OBJECT(filewindow),
										 "any_signal::softkey_activated",
										  ui_comp_file_chooser_softkey_activated_cb,
										  kind,
										  NULL);
	g_object_disconnect(G_OBJECT(fileview),
										 "any_signal::file_selected",
											ui_comp_file_chooser_selected_cb,
											kind,
											NULL);	
}

static void
ui_comp_contact_view_softkey_modify_activated_cb (GpeSKWindow *window,
																									gint nr,
																									ab_contact_t *contact)
{
  GtkWidget *app = NULL;
  gboolean ret = FALSE;
  GtkWidget *view = NULL;  
  view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));
  
  if (nr == UI_COMP_SK_LEFT && !picture_flag && !ringtone_flag)
  {
    g_print ("%s(): Left Softkey      \n", __FUNCTION__);

    if (view && gem_hash_view_get_editable (GEM_HASH_VIEW(view)))
    {    	
			gchar *storage = NULL;
			GtkWidget *listwindow = NULL;
			listwindow = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);

			storage = action_get_current_contact_storage (UI_COMP_SKWINDOW(listwindow));
			
			switch((gint)storage)
			{
				case 0:		
								ab_enabler_cnt_parse_from_edit_view (GEM_HASH_VIEW(view), &contact);
								break;
				case 1: 
								ab_enabler_cnt_parse_from_edit_view_sim (GEM_HASH_VIEW(view), &contact);
								break;
				defautlt:
								break;				
			}      
      if (contact)
      {
        gboolean update = FALSE;
        ab_uid_t uid = AB_ID_INVALID;        
        GtkWidget *window = NULL;    
  			window = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);       
			  uid = (gint)action_get_current_contact_uid (UI_COMP_SKWINDOW(window));        
				        if (uid != AB_ID_INVALID)
        {
        	update = TRUE;
        }

        g_print ("%s(): update = %s\n", __FUNCTION__,
                 update ? "TRUE" : "FALSE");
        if (update)
        {
      		ab_update_contact(ab_enabler_get_session ((gint)storage), contact);          
         }
        GEM_HASH_VIEW(view)->status = GEM_HV_STATUS_SAVE;
      }
      ab_contact_free (contact);
    }
    ret = TRUE;
  }
  else if (nr == UI_COMP_SK_LEFT && picture_flag)    
  {
  	GtkWidget *fileview = NULL;
  	GtkWidget *filewindow = NULL;  	
  	filewindow = ui_comp_winmgr_get_window(UI_COMP_SKW_FILE_CHOOSER);
  	if(filewindow == NULL)
  	{  	  	
  		fileview = gem_file_chooser_new();
  		gem_file_chooser_set_selectable(GEM_FILE_CHOOSER(fileview),FALSE);
  		gchar *picpath = NULL;
  		picpath = g_build_filename(gem_env_get_resource_path(),
																 "pixmaps",
																  NULL,
																  NULL);
  		g_print("%s(): picpath = %s  \n",__FUNCTION__,picpath);
	  	gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fileview),picpath);  		
			filewindow = ui_comp_skwindow_new(fileview,
																				"Ok",
																				"Cancel",															
																				"",
																				NULL,
																				0,
																				NULL);																	 
			g_signal_connect(G_OBJECT(filewindow),
											 "softkey_activated",
											 G_CALLBACK(ui_comp_file_chooser_softkey_activated_cb),
											 "pic");	
			g_signal_connect(G_OBJECT(fileview),
											 "file_selected",
											 G_CALLBACK(ui_comp_file_chooser_selected_cb),
											 "pic");														 
			ui_comp_winmgr_register_window(UI_COMP_SKW_FILE_CHOOSER,
																		 filewindow);		
  	}						 
  	else
  	{
  		gchar *picpath = NULL;
  		fileview = ui_comp_skwindow_get_view (UI_COMP_SKWINDOW(filewindow));
  		picpath = g_build_filename(gem_env_get_resource_path(),
																 "pixmaps",
																  NULL,
																  NULL);
  		g_print("%s(): picpath = %s  \n",__FUNCTION__,picpath ? picpath : "null");
	  	gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fileview),picpath);
		
			g_signal_connect(G_OBJECT(filewindow),
											 "softkey_activated",
											 G_CALLBACK(ui_comp_file_chooser_softkey_activated_cb),
											 "pic");	
			g_signal_connect(G_OBJECT(fileview),
											 "file_selected",
											 G_CALLBACK(ui_comp_file_chooser_selected_cb),
											 "pic");	  	
  	}
													 							
		gtk_widget_show_all(filewindow); 	
  }   
  else if (nr == UI_COMP_SK_LEFT && ringtone_flag)    
  {
  	GtkWidget *fileview = NULL;
  	GtkWidget *filewindow = NULL;  	
  	filewindow = ui_comp_winmgr_get_window(UI_COMP_SKW_FILE_CHOOSER);
  	if(filewindow == NULL)
  	{  	  	
  		fileview = gem_file_chooser_new();
  		gem_file_chooser_set_selectable(GEM_FILE_CHOOSER(fileview),FALSE);
  		gchar *picpath = NULL;
  		picpath = g_build_filename(gem_env_get_resource_path(),
																 "sounds",
																  NULL,
																  NULL);
  		g_print("%s(): picpath = %s  \n",__FUNCTION__,picpath);
	  	gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fileview),picpath);  		
			filewindow = ui_comp_skwindow_new(fileview,
																				"Ok",
																				"Cancel",															
																				"",
																				NULL,
																				0,
																				NULL);																	 
			g_signal_connect(G_OBJECT(filewindow),
											 "softkey_activated",
											 G_CALLBACK(ui_comp_file_chooser_softkey_activated_cb),
											 "ring");	
			g_signal_connect(G_OBJECT(fileview),
											 "file_selected",
											 G_CALLBACK(ui_comp_file_chooser_selected_cb),
											 "ring");														 
			ui_comp_winmgr_register_window(UI_COMP_SKW_FILE_CHOOSER,
																		 filewindow);		
  	}						 
  	else
  	{
  		g_print("%s() ...............................\n",__FUNCTION__);
  		gchar *picpath = NULL;
  		fileview = ui_comp_skwindow_get_view (UI_COMP_SKWINDOW(filewindow));
  		picpath = g_build_filename(gem_env_get_resource_path(),
																 "sounds",
																  NULL,
																  NULL);
  		g_print("%s(): picpath = %s  \n",__FUNCTION__,picpath ? picpath : "null");
	  	gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fileview),picpath);
	  	
	  	g_signal_connect(G_OBJECT(filewindow),
											 "softkey_activated",
											 G_CALLBACK(ui_comp_file_chooser_softkey_activated_cb),
											 "ring");	
			g_signal_connect(G_OBJECT(fileview),
											 "file_selected",
											 G_CALLBACK(ui_comp_file_chooser_selected_cb),
											 "ring");	
  	}
													 							
		gtk_widget_show_all(filewindow); 	
  }  
   
  else if (nr == UI_COMP_SK_RIGHT)
  {
    g_print ("%s(): Right Softkey        nr = %d\n", __FUNCTION__,nr);
    if (view)
    {
      GEM_HASH_VIEW(view)->status = GEM_HV_STATUS_CANCEL;
    }
    ret = TRUE;
  }

  if (ret)
  {
		gtk_widget_hide_all (GTK_WIDGET (window));
  }	
  return;
}

static void
ui_comp_contact_view_softkey_add_activated_cb (GpeSKWindow *window,
                                        		   gint nr,
                                           		 gpointer storage)
{
  GemApplication *app = NULL;
  gboolean ret = FALSE;
  GtkWidget *view = NULL;
  GtkWidget *skwindow = NULL;
 	GtkWidget *listview = NULL;
 	
  skwindow = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
  listview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(skwindow));
 	view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));

  if (nr == UI_COMP_SK_LEFT &&  !picture_flag && !ringtone_flag)
  {
    g_print ("%s(): Left Softkey      \n", __FUNCTION__);
    if (view && gem_hash_view_get_editable (GEM_HASH_VIEW(view)))
    {
      ab_contact_t *contact = NULL;
      switch((gint)storage)
      {
      	case 0:			
      				 ab_enabler_cnt_parse_from_edit_view (GEM_HASH_VIEW(view), &contact);
      				 break;
      	case 1:
      				 ab_enabler_cnt_parse_from_edit_view_sim (GEM_HASH_VIEW(view), &contact);
      				 break;
      }
      ab_contact_print (contact);      
      if (contact)
      {
        ab_uid_t uid = AB_ID_INVALID;    
  			  		  
        gchar *tmpuid = NULL;
        tmpuid = g_new0(gchar, 6);
        ab_add_contact (ab_enabler_get_session	((gint)storage), contact,&uid);
        ab_contact_get_uid (contact,&uid);
  			g_sprintf (tmpuid, "%d",uid);
  				
  			GtkWidget *EditWindow = NULL;
  			GtkWidget *editview = NULL;
  				
  			EditWindow = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW_EDIT);
  			editview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(EditWindow)); 
  				
        gem_hash_view_set_value (GEM_HASH_VIEW(editview), AB_IV_UID*1000, tmpuid);
 				g_print("%s(): uid = %d     \n",__FUNCTION__,uid);     
         g_free (tmpuid);
       }
      ab_contact_free (contact);
    }
    ret = TRUE;
  }
 
  else if (nr == UI_COMP_SK_LEFT && picture_flag)    
  {
  	GtkWidget *fileview = NULL;
  	GtkWidget *filewindow = NULL;

  	filewindow = ui_comp_winmgr_get_window(UI_COMP_SKW_FILE_CHOOSER);
  	if(filewindow == NULL)
  	{  	  	
  		fileview = gem_file_chooser_new();
  		gem_file_chooser_set_selectable(GEM_FILE_CHOOSER(fileview),FALSE);
  		gchar *picpath = NULL;
  		picpath = g_build_filename(gem_env_get_resource_path(),
																 "pixmaps",
																  NULL,
																  NULL);
  		g_print("%s(): picpath = %s  \n",__FUNCTION__,picpath);
	  	gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fileview),picpath);  		
			filewindow = ui_comp_skwindow_new(fileview,
																				"Ok",
																				"Cancel",															
																				"",
																				NULL,
																				0,
																				NULL);																	 
			g_signal_connect(G_OBJECT(filewindow),
											 "softkey_activated",
											 G_CALLBACK(ui_comp_file_chooser_softkey_activated_cb),
											 "pic");	
			g_signal_connect(G_OBJECT(fileview),
											 "file_selected",
											 G_CALLBACK(ui_comp_file_chooser_selected_cb),
											 "pic");												 
			ui_comp_winmgr_register_window(UI_COMP_SKW_FILE_CHOOSER,
																		 filewindow);		
  	}	
   	else
  	{
  		g_print("%s() ...............................\n",__FUNCTION__);
  		gchar *picpath = NULL;
  		fileview = ui_comp_skwindow_get_view (UI_COMP_SKWINDOW(filewindow));
  		picpath = g_build_filename(gem_env_get_resource_path(),
																 "pixmaps",
																  NULL,
																  NULL);
  		g_print("%s(): picpath = %s  \n",__FUNCTION__,picpath ? picpath : "null");
	  	gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fileview),picpath);
			g_signal_connect(G_OBJECT(filewindow),
											 "softkey_activated",
											 G_CALLBACK(ui_comp_file_chooser_softkey_activated_cb),
											 "pic");	
			g_signal_connect(G_OBJECT(fileview),
											 "file_selected",
											 G_CALLBACK(ui_comp_file_chooser_selected_cb),
											 "pic");	  	
  	}					 
  	gtk_widget_show_all(filewindow);
  }  
   else if (nr == UI_COMP_SK_LEFT && ringtone_flag)    
  {
  	GtkWidget *fileview = NULL;
  	GtkWidget *filewindow = NULL;  	
  	filewindow = ui_comp_winmgr_get_window(UI_COMP_SKW_FILE_CHOOSER);
  	if(filewindow == NULL)
  	{  	  	
  		fileview = gem_file_chooser_new();
  		gem_file_chooser_set_selectable(GEM_FILE_CHOOSER(fileview),FALSE);
  		gchar *picpath = NULL;
  		picpath = g_build_filename(gem_env_get_resource_path(),
																 "sounds",
																  NULL,
																  NULL);
  		g_print("%s(): picpath = %s  \n",__FUNCTION__,picpath);
	  	gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fileview),picpath);  		
			filewindow = ui_comp_skwindow_new(fileview,
																				"Ok",
																				"Cancel",															
																				"",
																				NULL,
																				0,
																				NULL);																	 
			g_signal_connect(G_OBJECT(filewindow),
											 "softkey_activated",
											 G_CALLBACK(ui_comp_file_chooser_softkey_activated_cb),
											 "ring");	
			g_signal_connect(G_OBJECT(fileview),
											 "file_selected",
											 G_CALLBACK(ui_comp_file_chooser_selected_cb),
											 "ring");											 
			ui_comp_winmgr_register_window(UI_COMP_SKW_FILE_CHOOSER,
																		 filewindow);		
  	}
  	else
  	{
  		g_print("%s() ...............................\n",__FUNCTION__);
  		gchar *picpath = NULL;
  		fileview = ui_comp_skwindow_get_view (UI_COMP_SKWINDOW(filewindow));
  		picpath = g_build_filename(gem_env_get_resource_path(),
																 "sounds",
																  NULL,
																  NULL);
  		g_print("%s(): picpath = %s  \n",__FUNCTION__,picpath ? picpath : "null");
	  	gem_file_chooser_set_current_path(GEM_FILE_CHOOSER(fileview),picpath);
			g_signal_connect(G_OBJECT(filewindow),
											 "softkey_activated",
											 G_CALLBACK(ui_comp_file_chooser_softkey_activated_cb),
											 "ring");	
			g_signal_connect(G_OBJECT(fileview),
											 "file_selected",
											 G_CALLBACK(ui_comp_file_chooser_selected_cb),
											 "ring");	  	
  	}  						 
													 							
		gtk_widget_show_all(filewindow); 	
  }  
  else if (nr == UI_COMP_SK_RIGHT)
  {
    g_print ("%s(): Right Softkey        nr = %d\n", __FUNCTION__,nr);
    if (view)
    {
      GEM_HASH_VIEW(view)->status = GEM_HV_STATUS_CANCEL;
    }
    ret = TRUE;
  }

  if (ret)
  {
  	ui_comp_cnt_list_view_update(UI_COMP_CNT_LIST_VIEW(listview));
		gtk_widget_hide_all (GTK_WIDGET (window));			
	}

  return;
}

static void 
ui_comp_contact_modify_pic_entry_in_cb(GtkWidget *entry, GdkEventFocus *event,UICompSKWindow	*skwindow)
{ 
	g_print("%s(): entering... \n",__FUNCTION__);	
	g_print("%s(): entry = 0x%08x, skwindow = 0x%08x \n",__FUNCTION__,(guint32)entry, (guint32)skwindow);
	gpe_skwindow_set_softkey_label(GPE_SKWINDOW(skwindow),UI_COMP_SK_LEFT,"Modify");
	picture_flag = TRUE;
}

static void 
ui_comp_contact_modify_pic_entry_out_cb(GtkWidget *entry, GdkEventFocus *event,UICompSKWindow	*skwindow)
{ 
	gpe_skwindow_set_softkey_label(GPE_SKWINDOW(skwindow),UI_COMP_SK_LEFT,"Save");
	picture_flag = FALSE;
}

static void 
ui_comp_contact_modify_ring_entry_in_cb(GtkWidget *entry, GdkEventFocus *event,UICompSKWindow	*skwindow)
{ 
	g_print("%s(): entering... \n",__FUNCTION__);	
	g_print("%s(): entry = 0x%08x, skwindow = 0x%08x \n",__FUNCTION__,(guint32)entry, (guint32)skwindow);
	gpe_skwindow_set_softkey_label(GPE_SKWINDOW(skwindow),UI_COMP_SK_LEFT,"Modify");
	ringtone_flag = TRUE;
}

static void 
ui_comp_contact_modify_ring_entry_out_cb(GtkWidget *entry, GdkEventFocus *event,UICompSKWindow	*skwindow)
{ 
	gpe_skwindow_set_softkey_label(GPE_SKWINDOW(skwindow),UI_COMP_SK_LEFT,"Save");
	ringtone_flag = FALSE;
}

void action_cnt_add_contact_cb()
{
	GtkWidget *sub_window = NULL;
	GtkWidget *view = NULL;
	GemApplication *app = NULL;

	GtkWidget *cntview = NULL;	
//	g_print("\n%s() : invoke the ui_comp_cont_edit_view_new()\n",__FUNCTION__);
	cntview = ui_comp_cnt_edit_view_new();
  ui_comp_cnt_edit_view_constructor(NULL,cntview);
	
	 	
	sub_window = ui_comp_skwindow_new(cntview,
																"Save",
																"Cancel",															
																"",
																NULL,
																0,
																NULL);
	g_signal_connect(G_OBJECT(sub_window),
									 "softkey_activated",
									 G_CALLBACK(ui_comp_contact_view_softkey_add_activated_cb),
									 (gpointer)0);					 
									 
	g_signal_connect(G_OBJECT(sub_window),
										"hide",
										G_CALLBACK(ui_comp_contact_view_hide_cb),
										(gpointer)0);
	
	GemHashViewCell *cell = NULL;
	GtkWidget *entry = NULL;
	cell = gem_hash_view_get_item(GEM_HASH_VIEW(cntview),AB_IV_PICTURE*1000);		
	entry = gem_hash_view_cell_std_get_entry(GEM_HASH_VIEW_CELL_STD(cell));	
	 
	g_signal_connect(entry,
									"focus-in-event",
									G_CALLBACK(ui_comp_contact_modify_pic_entry_in_cb),
									UI_COMP_SKWINDOW(sub_window));																		 
	g_signal_connect(entry,
									"focus-out-event",
									G_CALLBACK(ui_comp_contact_modify_pic_entry_out_cb),
									UI_COMP_SKWINDOW(sub_window));						 
	cell = gem_hash_view_get_item(GEM_HASH_VIEW(cntview),AB_IV_RINGTONE*1000);		
	entry = gem_hash_view_cell_std_get_entry(GEM_HASH_VIEW_CELL_STD(cell));		
 
	g_signal_connect(entry,
									"focus-in-event",
									G_CALLBACK(ui_comp_contact_modify_ring_entry_in_cb),
									UI_COMP_SKWINDOW(sub_window));		
															 
	g_signal_connect(entry,
									"focus-out-event",
									G_CALLBACK(ui_comp_contact_modify_ring_entry_out_cb),
									UI_COMP_SKWINDOW(sub_window));										
									
	ui_comp_winmgr_register_window(UI_COMP_SKW_CNT_VIEW_EDIT,
																 sub_window);	
/*	g_signal_connect(G_OBJECT(sub_window),
									 "show",
									 G_CALLBACK(softkey_activated_cb),
									 NULL);*/
	gtk_widget_show_all(sub_window);
  return;
}

static void
ui_comp_contact_view_hide_cb (GtkWidget *widget, gpointer user_data)
{
  GtkWidget *ListWindow = NULL; 
  GtkWidget *EditWindow = NULL;
  GtkWidget *listview = NULL;
  GtkWidget *editview = NULL;
  const gchar *uidstr = NULL; 
  gint storage = (gint)user_data;    
  ListWindow = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
  listview =  ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(ListWindow)); 
  EditWindow = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW_EDIT);
	editview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(EditWindow));  
	uidstr = gem_hash_view_get_value(GEM_HASH_VIEW(editview),AB_IV_UID*1000);
  
  if (editview)
  {
    g_print ("%s():status = %s\n", __FUNCTION__,
             (GEM_HASH_VIEW(editview)->status == GEM_HV_STATUS_SAVE) ? "TRUE" : "FALSE");
    if (GEM_HASH_VIEW(editview)->status == GEM_HV_STATUS_SAVE)
    {
     
//      uidstr = gem_hash_view_get_value(GEM_HASH_VIEW(editview),AB_IV_UID*1000);
      if (uidstr)
      {
        GtkTreeIter iter;        
        ab_contact_t *contact = NULL;        
				ab_uid_t uid = atoi(uidstr);

  			ab_get_contact (ab_enabler_get_session(storage), uid, &contact);
      	ab_contact_print (contact);
        if (ab_enabler_ui_comp_list_view_find (GEM_TREE_VIEW(listview), uid, &iter))
        {
        	switch((gint)storage)
        	{
        		case 0: 
        						ab_enabler_ui_comp_cnt_list_view_update (UI_COMP_CNT_LIST_VIEW(listview), contact, &iter);
										break;
						case 1:
										ab_enabler_ui_comp_cnt_list_view_update_sim (UI_COMP_CNT_LIST_VIEW(listview), contact, &iter);
										break;
						default: break;				
        	}       
        }
        else
        {
        	g_print("%s(): listview = 0x%08x \n",__FUNCTION__,listview);
        	switch((gint)storage)
        	{
        		case 0: 
        						ab_enabler_ui_comp_cnt_list_view_update (UI_COMP_CNT_LIST_VIEW(listview), contact, NULL);
										break;
						case 1: 
										ab_enabler_ui_comp_cnt_list_view_update_sim (UI_COMP_CNT_LIST_VIEW(listview), contact, NULL);
										break;
						default: break;				
        	}
	        ui_comp_cnt_list_view_update(UI_COMP_CNT_LIST_VIEW(listview));              
        }
        gem_tree_view_set_search_column(GEM_TREE_VIEW(listview),UI_COMP_CNT_LV_NAME,GTK_SORT_ASCENDING,NULL);        
        ab_contact_free (contact);
      }
    }
  }
  return;
}
void 
action_cnt_modify_contact_local_cb(ab_contact_t *contact,ab_uid_t uid)
{
    GtkWidget *cntlistview = NULL;
    GtkWidget *sub_window = NULL;
    GemHashViewCell *cell = NULL;
    GtkWidget *entry = NULL;
    ab_fid_t *fid = NULL;
    gint i = 0;
    ab_ftid_t ftid = 0;
    ab_contact_print(contact);
    cntlistview = ui_comp_cnt_edit_view_new();   
    ui_comp_cnt_edit_view_constructor(contact,cntlistview);
      

		ab_contact_list_fields(contact,&fid);  
		
		for (i = 0; fid[i] != AB_ID_INVALID; i++)
		{
	    gchar *fieldstr = NULL;
	    
			ab_contact_get_ftid(contact,fid[i],&ftid);
			if(ftid==AB_IV_UID)
			{
				gint uid;
				ab_contact_get_int32_field(contact, fid[i], &uid);
//				g_print("%s(): (%d)\n", __FUNCTION__, uid);
    		fieldstr = g_strdup_printf("%d",uid);
			}
			else
			{
				ab_contact_get_string_field(contact, fid[i], &fieldstr);
			}
/*			g_print("%s(): %d, (%d, %d)--------------------fieldstr(%s)----------------\n",
							__FUNCTION__,
							i,
							fid[i],
							ftid,
							(fieldstr ? fieldstr : "(null)"));*/
			if(fieldstr!=NULL)
			{
				gem_hash_view_set_value(GEM_HASH_VIEW(cntlistview),
													      fid[i],
													      fieldstr);
			}	
			g_free(fieldstr);
		}        
		
		ab_category_t **categories = NULL;
		gchar *cat_content = g_new0(gchar,50);
		ab_get_categories_for_contact(ab_enabler_get_session(0),
																	(ab_uid_t)uid,
																	&categories);
		if(categories != NULL)
		{																	
			gint j = 0;	
			char *cat_name = "";																		
			for (j = 0; categories[j] != NULL; j++)					
			{
	    	ab_category_get_name (categories[j], &cat_name);
				if(categories[j+1] != NULL)
				{
					cat_content = g_strconcat(cat_content,cat_name,", ",NULL);
				}
				else
					cat_content = g_strconcat(cat_content,cat_name,NULL);
			}		
			gem_hash_view_set_value(GEM_HASH_VIEW(cntlistview),
												      AB_IV_CATEGORY_ID*1000,
												      cat_content);
	 		g_free(cat_name);
		}
 		g_free(cat_content);
		
		GEM_HASH_VIEW(cntlistview)->focused_item = AB_IV_MOBILE_NUM*1000;
		gem_hash_view_set_editable(GEM_HASH_VIEW(cntlistview),TRUE);		
    sub_window = ui_comp_skwindow_new(cntlistview,
																			"Save",
																			"Cancel",															
																			"",
																			NULL,
																			0,
																			NULL);
  	g_signal_connect(G_OBJECT(sub_window),
									   "softkey_activated",
									    G_CALLBACK(ui_comp_contact_view_softkey_modify_activated_cb),
								    	contact);
								    	
		g_signal_connect(G_OBJECT(sub_window),
										"hide",
										G_CALLBACK(ui_comp_contact_view_hide_cb),
										(gint)0);
		cell = gem_hash_view_get_item(GEM_HASH_VIEW(cntlistview),AB_IV_PICTURE*1000);		
		entry = gem_hash_view_cell_std_get_entry(GEM_HASH_VIEW_CELL_STD(cell));		
		g_signal_connect(entry,
										"focus-in-event",
										G_CALLBACK(ui_comp_contact_modify_pic_entry_in_cb),
										UI_COMP_SKWINDOW(sub_window));		
																 
		g_signal_connect(entry,
										"focus-out-event",
										G_CALLBACK(ui_comp_contact_modify_pic_entry_out_cb),
										UI_COMP_SKWINDOW(sub_window));	
		cell = gem_hash_view_get_item(GEM_HASH_VIEW(cntlistview),AB_IV_RINGTONE*1000);		
		entry = gem_hash_view_cell_std_get_entry(GEM_HASH_VIEW_CELL_STD(cell));		
 
		g_signal_connect(entry,
										"focus-in-event",
										G_CALLBACK(ui_comp_contact_modify_ring_entry_in_cb),
										UI_COMP_SKWINDOW(sub_window));		
																 
		g_signal_connect(entry,
										"focus-out-event",
										G_CALLBACK(ui_comp_contact_modify_ring_entry_out_cb),
										UI_COMP_SKWINDOW(sub_window));											
	  ui_comp_winmgr_register_window(UI_COMP_SKW_CNT_VIEW_EDIT,
																  sub_window);
		gtk_widget_show_all(sub_window);
//		free(fid);
		return;
}

void 
action_cnt_modify_contact_sim_cb(ab_contact_t *contact,ab_uid_t uid)
{
    GtkWidget *cntlistview = NULL;
    GtkWidget *sub_window = NULL;
    GemHashViewCell *cell = NULL;
    GtkWidget *entry = NULL;    
    ab_fid_t *fid = NULL;
    gchar *fieldstr = NULL;
    gint i = 0;
    ab_ftid_t ftid = 0;
    
    cntlistview = ui_comp_cnt_edit_view_sim_new();     
    ab_contact_print(contact);
		ab_contact_list_fields(contact,&fid);  
		
		for (i = 0; fid[i] != AB_ID_INVALID; i++)
		{
	    gchar *fieldstr = NULL;
	    
			ab_contact_get_ftid(contact,fid[i],&ftid);
			if(ftid==AB_IV_UID)
			{
				gint uid;
				ab_contact_get_int32_field(contact, fid[i], &uid);
//				g_print("%s(): (%d)\n", __FUNCTION__, uid);
    		fieldstr = g_strdup_printf("%d",uid);
			}
			else
			{
				ab_contact_get_string_field(contact, fid[i], &fieldstr);
			}
			/*g_print("%s(): %d, (%d, %d)--------------------fieldstr(%s)----------------\n",
							__FUNCTION__,
							i,
							fid[i],
							ftid,
							(fieldstr ? fieldstr : "(null)"));*/
			if(fieldstr!=NULL)
			{
				gem_hash_view_set_value(GEM_HASH_VIEW(cntlistview),
													      fid[i],
													      fieldstr);
			}		
			g_free(fieldstr);
		}  

		gem_hash_view_set_editable(GEM_HASH_VIEW(cntlistview),TRUE);
    sub_window = ui_comp_skwindow_new(cntlistview,
																			"Save",
																			"Cancel",															
																			"",
																			NULL,
																			0,
																			NULL);
  	g_signal_connect(G_OBJECT(sub_window),
									   "softkey_activated",
									    G_CALLBACK(ui_comp_contact_view_softkey_modify_activated_cb),
								    	contact);
		g_signal_connect(G_OBJECT(sub_window),
										"hide",
										G_CALLBACK(ui_comp_contact_view_hide_cb),
										(gpointer)1);
		cell = gem_hash_view_get_item(GEM_HASH_VIEW(cntlistview),AB_IV_PICTURE);		
		entry = gem_hash_view_cell_std_get_entry(GEM_HASH_VIEW_CELL_STD(cell));	
//	g_print("%s(): sub_window = 0x%08x,entry = 0x%08x  cell=0x%08x \n",__FUNCTION__,(guint32)sub_window,(guint32)entry,(guint32)cell);	 
		g_signal_connect(entry,
										"focus-in-event",
										G_CALLBACK(ui_comp_contact_modify_pic_entry_in_cb),
										UI_COMP_SKWINDOW(sub_window));
		g_signal_connect(entry,
										"focus-out-event",
										G_CALLBACK(ui_comp_contact_modify_pic_entry_out_cb),
										UI_COMP_SKWINDOW(sub_window));	
	  ui_comp_winmgr_register_window(UI_COMP_SKW_CNT_VIEW_EDIT,
																  sub_window);
		g_print("%s(): sub_window = 0x%08x \n",__FUNCTION__,sub_window);

  	gtk_widget_show_all(sub_window);
  //	free(fid);
  	return;
}

void action_cnt_modify_contact_cb()
{
  ab_contact_t *contact = NULL; 
  GtkWidget *window = NULL;
  ab_uid_t uid = AB_ID_INVALID;
  gint storage = -1;
  
  window = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
  uid = (ab_uid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(window));
  storage = (gint)action_get_current_contact_storage (UI_COMP_SKWINDOW(window));
  if (uid == AB_ID_INVALID)
  {
    return;
  }
	switch((gint)storage)
	{
		case 0: 
						ab_get_contact (ab_enabler_get_session(0), uid, &contact);
						action_cnt_modify_contact_local_cb(contact,uid);
						break;
		case 1: 
						ab_get_contact (ab_enabler_get_session(1), uid, &contact);
						action_cnt_modify_contact_sim_cb(contact,uid);
						break;
		default: 
		    		break;
	}
    return;
}

void cnt_delete_softkey_activated_cb(GpeSKWindow		*window,
																			gint					 nr,
																			gpointer			 user_data)
{
	g_print("%s(): entering......\n",
					__FUNCTION__);
	if (nr == UI_COMP_SK_LEFT)
	{
		 ab_uid_t *uid = (ab_uid_t *)user_data;
		 gchar *storage = NULL;
		 GtkWidget *listwindow = NULL;
		 listwindow = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
		 storage = action_get_current_contact_storage (UI_COMP_SKWINDOW(listwindow));
		 ab_remove_contact(ab_enabler_get_session((gint)storage), (ab_uid_t)uid);
		 
		 GtkWidget *ListWindow = NULL;
		 GtkWidget *listview = NULL;
		 
		 ListWindow = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
  	 listview =  ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(ListWindow)); 
		 ab_enabler_ui_comp_list_view_delete(GEM_TREE_VIEW(listview), uid);
	
		 ui_comp_cnt_list_view_update(UI_COMP_CNT_LIST_VIEW(listview));
	}
	else if (nr == UI_COMP_SK_RIGHT)
	{		
	}
	
	gtk_widget_hide_all (GTK_WIDGET (window));
}

void action_cnt_delete_contact_cb()
{
	GtkWidget *app = NULL;
	GtkWidget *dialog = NULL;
  ab_contact_t *contact = NULL; 
  GtkWidget *window = NULL;
  ab_uid_t uid = AB_ID_INVALID;
  gchar *storage = NULL;
  window = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
  uid = (ab_uid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(window));
  storage = action_get_current_contact_storage (UI_COMP_SKWINDOW(window));
  
	g_print("%s(): uid = %d \n",__FUNCTION__,uid); 	   
	 	     
  if (uid == AB_ID_INVALID)
  {
    return;
  }
		
  ab_get_contact (ab_enabler_get_session((gint)storage), uid, &contact);
	ab_contact_print(contact);
	
	dialog = g_object_new(UI_COMP_TYPE_SKWINDOW, NULL);
	gem_sk_window_set_delete_warning (dialog, "Confirm to delete it? \n\n");
	gem_sk_dialog_set_position (window,dialog, 50);
//	gem_sk_dialog_set_position (window,dialog, 150);
	
	g_signal_connect(G_OBJECT(dialog),
									 "softkey_activated",
									 G_CALLBACK(cnt_delete_softkey_activated_cb),
									 (gpointer) uid);
	gtk_widget_show_all(dialog);

  return;
}

void action_cnt_add_contact_sim_cb()
{
	GtkWidget *sub_window = NULL;
	GtkWidget *view = NULL;
	GemApplication *app = NULL;

	GtkWidget *cntview = NULL;	
//	g_print("\n%s() : invoke the ui_comp_cont_edit_view_new()\n",__FUNCTION__);
	cntview = ui_comp_cnt_edit_view_sim_new();
	sub_window = ui_comp_skwindow_new(cntview,
																"Save",
																"Cancel",															
																"",
																NULL,
																0,
																NULL);
	g_signal_connect(G_OBJECT(sub_window),
									 "softkey_activated",
									 G_CALLBACK(ui_comp_contact_view_softkey_add_activated_cb),
									 (gpointer)1);					
	g_signal_connect(G_OBJECT(sub_window),
										"hide",
										G_CALLBACK(ui_comp_contact_view_hide_cb),
										(gpointer)1);			
	/*GemHashViewCell *cell = NULL;
	GtkWidget *entry = NULL;
	cell = gem_hash_view_get_item(GEM_HASH_VIEW(cntview),AB_IV_PICTURE);		
	entry = gem_hash_view_cell_std_get_entry(GEM_HASH_VIEW_CELL_STD(cell));	
	g_print("%s(): sub_window = 0x%08x,entry = 0x%08x  cell=0x%08x \n",__FUNCTION__,(guint32)sub_window,(guint32)entry,(guint32)cell);	 
	g_signal_connect(entry,
									"focus-in-event",
									G_CALLBACK(ui_comp_contact_modify_entry_in_cb),
									UI_COMP_SKWINDOW(sub_window));																		 
	g_signal_connect(entry,
									"focus-out-event",
									G_CALLBACK(ui_comp_contact_modify_entry_out_cb),
									UI_COMP_SKWINDOW(sub_window));			*/
	ui_comp_winmgr_register_window(UI_COMP_SKW_CNT_VIEW_EDIT,
																 sub_window);	
/*	g_signal_connect(G_OBJECT(sub_window),
									 "show",
									 G_CALLBACK(softkey_activated_cb),
									 NULL);*/
	g_print("%s() : add contact is ing .\n",__FUNCTION__);				 
	gtk_widget_show_all(sub_window);
  return;
}
void action_cnt_list_contact_cb()
{
	GtkWidget *window = NULL;
	gchar *uid = NULL;
	gchar *storage = NULL;

	window = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
	uid = action_get_current_contact_uid(UI_COMP_SKWINDOW(window));
	storage = action_get_current_contact_storage(UI_COMP_SKWINDOW(window));
	switch((gint)storage)
	{
		case 0 : mw_cnt_list_view_row_cb(uid);            
						 break; 
		case 1:  mw_cnt_list_view_sim_row_cb(uid); 
						 break;
	}	
}

void action_cnt_message_contact_cb()
{
	ab_contact_t *contact = NULL;
  gchar *uid = NULL;
  GtkWidget *window = NULL;
  gchar *storage = NULL;
  
  int end = -1;
  window = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
  uid = action_get_current_contact_uid(UI_COMP_SKWINDOW(window));
  if (uid == NULL)
  {
    return;
  }
  
  storage = action_get_current_contact_storage(UI_COMP_SKWINDOW(window));
	switch((gint)storage)
	{
		case 0 : ab_get_contact (ab_enabler_get_session(0), (ab_uid_t)uid, &contact);  
						 end =  AB_IV_BUSINESS_NUM;
						 break; 
		case 1:  ab_get_contact (ab_enabler_get_session(1), (ab_uid_t)uid, &contact); 
						 end = AB_IV_MOBILE_NUM;
						 break;
	}	

  if (TRUE)
  {
    gint i;
    gint count;
    gchar *dialnum = NULL;
    
		ab_contact_print(contact);
    
    for (count = 0, i = AB_IV_MOBILE_NUM; i <= end ; i++)
    {
      gchar *value = NULL;
      ab_fid_t fid = AB_ID_INVALID;
  		ab_contact_get_default_field(contact,i,&fid);
			ab_contact_get_string_field(contact, fid, &value);
			
      if (value && strlen (value) > 0)
      {
        if (dialnum)
        {
          g_free (dialnum);
        }
        dialnum = g_strdup ((gchar *) value);
        count++;
        g_free (value);
      }
    }
g_print("%s() : the for loop is over................\n ",__FUNCTION__);

    if (count > 1)
    {
      GtkWidget *numberchooser = NULL;
      gint result;
      numberchooser = abook_number_chooser_new ();
      ab_enabler_number_chooser_parse_from_contact (ABOOK_NUMBER_CHOOSER(numberchooser), 
      																							contact,
      																							storage);
                                     			         
/*      g_signal_connect (G_OBJECT (numberchooser), "key-press-event",
                        G_CALLBACK (abook_skw_general_key_press_event_cb),
                        NULL);*/

      gtk_widget_grab_focus (numberchooser);
      gtk_widget_show_all (numberchooser);
      result = gtk_dialog_run (GTK_DIALOG (numberchooser));

      
      if (result == GTK_RESPONSE_OK)
      {
        dialnum =
          abook_number_chooser_get_selected (ABOOK_NUMBER_CHOOSER
                                             (numberchooser));
      }

      gtk_widget_destroy (numberchooser);
    }

    if (dialnum)
    {
      gchar *name = NULL;
      ab_fid_t fid = AB_ID_INVALID;											 
  		ab_contact_get_default_field(contact,AB_IV_FULL_NAME,&fid); 
			ab_contact_get_string_field(contact, fid, (gpointer) & name);

      g_print ("%s(): sending message to [%20s, %s]\n",
               __FUNCTION__, name ? name : "null", dialnum ? dialnum : "null");

      abook_iac_single_sms (name, dialnum);

      g_free (name);
    }

    g_free (dialnum);
  }

  ab_contact_free (contact);
  return;
}

void action_cnt_call_contact_cb()
{
	ab_contact_t *contact = NULL;
  gchar *uid = NULL;
  GtkWidget *window = NULL;
  gchar *storage = NULL;
  
  int end = -1;
  window = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
  uid = action_get_current_contact_uid(UI_COMP_SKWINDOW(window));
  if (uid == NULL)
  {
    return;
  }
  
  storage = action_get_current_contact_storage(UI_COMP_SKWINDOW(window));
	switch((gint)storage)
	{
		case 0 : ab_get_contact (ab_enabler_get_session(0), (ab_uid_t)uid, &contact);  
						 end =  AB_IV_BUSINESS_NUM;
						 break; 
		case 1:  ab_get_contact (ab_enabler_get_session(1), (ab_uid_t)uid, &contact); 
						 end = AB_IV_MOBILE_NUM;
						 break;
	}	

  if (TRUE)
  {
    gint i;
    gint count;
    gchar *dialnum = NULL;
    
    for (count = 0, i = AB_IV_MOBILE_NUM; i <= end ; i++)
    {
      gchar *value = NULL;
      ab_fid_t fid = AB_ID_INVALID;
  		ab_contact_get_default_field(contact,i,&fid);
			ab_contact_get_string_field(contact, fid, &value);
			
      if (value && strlen (value) > 0)
      {
        if (dialnum)
        {
          g_free (dialnum);
        }
        dialnum = g_strdup ((gchar *) value);
        count++;
        g_free (value);
      }
    }

    if (count > 1)
    {
      GtkWidget *numberchooser = NULL;
      gint result;
      numberchooser = abook_number_chooser_new ();
      ab_enabler_number_chooser_parse_from_contact (ABOOK_NUMBER_CHOOSER(numberchooser), 
      																							contact,
      																							storage);
                                     			         
/*      g_signal_connect (G_OBJECT (numberchooser), "key-press-event",
                        G_CALLBACK (abook_skw_general_key_press_event_cb),
                        NULL);*/

      gtk_widget_grab_focus (numberchooser);
      gtk_widget_show_all (numberchooser);
      result = gtk_dialog_run (GTK_DIALOG (numberchooser));

      
      if (result == GTK_RESPONSE_OK)
      {
        dialnum =
          abook_number_chooser_get_selected (ABOOK_NUMBER_CHOOSER
                                             (numberchooser));
      }

      gtk_widget_destroy (numberchooser);
    }

    if (dialnum)
    {
      gchar *name = NULL;
      gchar *picfile = NULL;
      gchar *picpath = NULL;
      ab_fid_t fid = AB_ID_INVALID;											 
  		ab_contact_get_default_field(contact,AB_IV_FULL_NAME,&fid); 
			ab_contact_get_string_field(contact, fid, (gpointer) & name);
			
			if(name == NULL)
			{
				name = g_strdup("");
			}
               
			switch((gint)storage)
			{
				case 0 :fid = AB_ID_INVALID;											 
					  		ab_contact_get_default_field(contact,AB_IV_PICTURE,&fid); 
								ab_contact_get_string_field(contact, fid, (gpointer) & picfile);
								break;     
				case 1: picfile = "iconsim.png"; 
								break;
			}	                           
    
g_print ("%s(): call to [%20s, %s,%s]\n",
	   			__FUNCTION__, name ? name : "null", dialnum ? dialnum : "null",picpath ? picpath : "null"); 
    
			if (picfile == NULL)
      {
        picfile = g_strdup("");
      }       
      else
      {      	
				picpath = g_build_filename (gem_env_get_resource_path(),
																	  "pixmaps",
															  	   picfile,
																	   NULL);
															   		
	//			g_free(picfile);																	   
      }   
				g_print("%s() : picpath = %s.......................\n",__FUNCTION__,picpath);		
     	abook_iac_dialer (name, picpath, dialnum);

      g_free(name);
      g_free(picpath);
    }
    g_free(dialnum);
    
  }
  ab_contact_free (contact);
  return;
}

void
action_cnt_mark_cb (void)
{
	GtkWidget *treeview = NULL;
	GtkWidget *sub_window = NULL;
	sub_window = ui_comp_winmgr_get_window (UI_COMP_SKW_CAT_MARK_VIEW);
	
	if(sub_window == NULL)
	{
		g_print("%s() sub_window is NULL; create a new one \n",__FUNCTION__);
	  treeview = ui_comp_mark_view_new ();
		ui_comp_tree_view_add_headline(treeview,"All");
	  ab_enabler_ui_comp_tree_view_cnt_parse_contacts(treeview, FALSE);
	  
	  gem_tree_view_set_search_column(GEM_TREE_VIEW(treeview),MARK_COL_MV_STRING,GTK_SORT_ASCENDING,NULL);
		sub_window = ui_comp_skwindow_new(treeview,
																			"Opt",
																			"Back",															
																			uidesc_field,
																			actions_field,
																			G_N_ELEMENTS(actions_field),
																			"Contact");
																			
		ui_comp_winmgr_register_window (UI_COMP_SKW_CNT_MARK_VIEW,sub_window);																		
																			
		g_signal_connect(G_OBJECT(sub_window),
										 "softkey_activated",
										 G_CALLBACK(softkey_activated_treeview_cb),
										 NULL);									
	}										
	else
	{
		treeview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(sub_window));
		ui_comp_tree_view_cnt_update(treeview);
	}
	
  gtk_widget_show_all (sub_window);
  return;
}
void
action_cnt_sort_by_group_cb()
{
	 GtkWidget *treeview = NULL;
	GtkWidget *sub_window = NULL;
	sub_window = ui_comp_winmgr_get_window (UI_COMP_SKW_CAT_MEMBER_VIEW);
	if(sub_window == NULL)
	{
		g_print("%s() sub_window is NULL; create a new one \n",__FUNCTION__);
	  treeview = ui_comp_tree_view_cnt_new ();
	  ui_comp_tree_view_set_checkcolumn_visible(treeview,FALSE);
	  ab_enabler_ui_comp_tree_view_cnt_parse_category(treeview, TRUE);
		sub_window = ui_comp_skwindow_new(treeview,
																			"",
																			"Back",															
																			NULL,
																			NULL,
																			0,
																			"Category");
																			
		ui_comp_winmgr_register_window (UI_COMP_SKW_CAT_MEMBER_VIEW,sub_window);																		
																			
		g_signal_connect(G_OBJECT(sub_window),
										 "softkey_activated",
										 G_CALLBACK(softkey_activated_treeview_cb),
										 NULL);									
	}										
	else
	{
		treeview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(sub_window));
		ui_comp_tree_view_cnt_update(treeview);
	}
  gtk_widget_show_all (sub_window);
  return;
}


static void
cat_add_hide_cb(GtkWidget *widget, gpointer user_data)
{
	GtkWidget *cnt_edit_window = NULL;
	GtkWidget *cnt_edit_view = NULL;
	GtkWidget *cnt_list_window = NULL;
	gchar *uid = NULL;
	cnt_edit_window = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW_EDIT);
	if(cnt_edit_window)
	{
		cnt_edit_view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(cnt_edit_window));
		cnt_list_window = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
		uid = action_get_current_contact_uid (UI_COMP_SKWINDOW(cnt_list_window));
		ab_category_t **categories = NULL;
		gchar *cat_content = g_new0(gchar,50);
		ab_get_categories_for_contact(ab_enabler_get_session(0),
																	(ab_uid_t)uid,
																	&categories);
		gint j = 0;	
		char *cat_name = "";						
		if(categories)
		{												
			for (j = 0; categories[j] != NULL; j++)					
			{
		  	ab_category_get_name (categories[j], &cat_name);
		//			g_strlcat(cat_content,cat_name,strlen(cat_name));
		//			g_strlcat(cat_content,",",1);
				if(categories[j+1] != NULL)
				{
					cat_content = g_strconcat(cat_content,cat_name,", ",NULL);
				}
				else
					cat_content = g_strconcat(cat_content,cat_name,NULL);
			}		
			gem_hash_view_set_value(GEM_HASH_VIEW(cnt_edit_view),
												      AB_IV_CATEGORY_ID*1000,
												      cat_content);
			g_free(cat_name);
		}
	}
}

void 
action_add_to_category_cb(void)
{
	GtkWidget *treeview = NULL;
	GtkWidget *sub_window = NULL;
	GtkWidget *cnt_window = NULL;
	GtkWidget *cnt_view = NULL;
	GemHashViewCell *cell = NULL;
	const gchar *cnt_uid;
	cnt_window = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW_EDIT);
	cnt_view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(cnt_window));
	cell = gem_hash_view_get_item (GEM_HASH_VIEW(cnt_view),AB_IV_UID*1000);
//  cnt_uid = action_get_current_contact_uid (UI_COMP_SKWINDOW(cnt_window));
  cnt_uid = gem_hash_view_cell_get_value (cell);
   
	treeview = ui_comp_tree_view_cnt_new();
	ui_comp_tree_view_set_checkcolumn_visible(treeview,TRUE);
	ui_comp_tree_view_add_headline(treeview,"All");
  ab_enabler_ui_comp_tree_view_cnt_parse_category(treeview, FALSE);

//  gchar *uidstr = NULL;
//	uidstr = g_strdup_printf("%d",cnt_uid);
  ab_enabler_ui_comp_tree_view_parse_selection_for_contact (treeview,cnt_uid);
  
	sub_window = ui_comp_skwindow_new(treeview,
																	"Confirm",
																	"Back",															
																	NULL,
																	NULL,
																	0,
																	"Category");
	g_signal_connect(G_OBJECT(sub_window),
									 "softkey_activated",
									 G_CALLBACK(softkey_activated_add_to_cat_cb),
									 treeview);				
	g_signal_connect(G_OBJECT(sub_window),
									 "hide",
									 G_CALLBACK(cat_add_hide_cb),
									 NULL);
									 															
  gtk_widget_show_all (sub_window);
  return;
}

gboolean
menu_popup_key_press_cb (GtkWidget *widget,
                         GdkEventKey *event, 
                         gpointer user_data)
{

  UICompSKWindow *sk_window = (UICompSKWindow *) user_data;
	
  if (event->keyval == GDK_F4 || event->keyval == GDK_KP_F4)
  {
  	gtk_menu_popdown (GTK_MENU (widget));
		gpe_skwindow_set_softkey_label(GPE_SKWINDOW(sk_window),UI_COMP_SK_RIGHT,"Back");
  //  gtk_widget_grab_focus (GTK_WIDGET (sk_window->window));
    return TRUE;
  }
  
  return FALSE;
}
gboolean
mark_view_selection_func(GtkTreeSelection *selection,
	                       GtkTreeModel *model,
	                       GtkTreePath *path,
	                       gboolean path_currently_selected,
	                       gpointer data)
{
	GtkTreeIter iter;
	GpeSKWindow *window = GPE_SKWINDOW(data);
	if(gtk_tree_model_get_iter(model,&iter,path))
	{
		gchar *uid;
		gchar *extend;
		GtkUIManager *uimanager = NULL;
		GtkWidget *menu_left = NULL;
		gtk_tree_model_get(model,&iter,MARK_COL_MV_EXTEND,&extend,-1);
		if(!path_currently_selected)
		{
			if((gint)extend == 	FIELD_MUL)
			{
				gpe_skwindow_set_softkey_label(window, UI_COMP_SK_LEFT, "Option");
				uimanager = ui_comp_skwindow_get_ui_manager(UI_COMP_SKWINDOW(window));
				menu_left = gtk_ui_manager_get_widget(uimanager,"/PopupLeft_M/");
				if(menu_left)
				{
					g_signal_connect (G_OBJECT (menu_left),
  			                    "key-press-event",
	  		                     G_CALLBACK (menu_popup_key_press_cb),
	  	                     (gpointer) window);
					gpe_skwindow_set_softkey_menu(GPE_SKWINDOW(window),
																				UI_COMP_SK_LEFT,
																				menu_left);
				}
	
			}
			else if((gint)extend == FIELD_NONE)
			{
				gpe_skwindow_set_softkey_label(window, UI_COMP_SK_LEFT, "");
				uimanager = ui_comp_skwindow_get_ui_manager(UI_COMP_SKWINDOW(window));
				gpe_skwindow_set_softkey_menu(GPE_SKWINDOW(window),
																				UI_COMP_SK_LEFT,
																				NULL);
				gpe_skwindow_set_softkey_label(window,UI_COMP_SK_RIGHT,"Back");
			}
			else if((gint)extend ==	FIELD_EXTEND)
			{
				gpe_skwindow_set_softkey_label(window, UI_COMP_SK_LEFT, "Option");
				uimanager = ui_comp_skwindow_get_ui_manager(UI_COMP_SKWINDOW(window));
				menu_left = gtk_ui_manager_get_widget(uimanager,"/PopupLeft_E/");
				if(menu_left)
				{
					g_signal_connect (G_OBJECT (menu_left),
  			                    "key-press-event",
	  		                     G_CALLBACK (menu_popup_key_press_cb),
	  	                     (gpointer) window);
					gpe_skwindow_set_softkey_menu(GPE_SKWINDOW(window),
																				UI_COMP_SK_LEFT,
																				menu_left);
				}
			}
		}
	}
	return TRUE;
}	                   

void ui_comp_row_activated_field_manage_cb (GtkTreeView *treeview,
                                            GtkTreePath *path,
                                            GtkTreeViewColumn *col,
                                            gpointer user_data)
{
	GtkTreeModel *model = NULL;

	GtkTreeIter iter;
	model = gem_tree_view_get_model(GEM_TREE_VIEW(user_data));
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gchar *extend;
    gtk_tree_model_get (model, &iter,
                        MARK_COL_MV_EXTEND, &extend,
                        -1);
    if((gint)extend == FIELD_EXTEND)
    {
    	ui_comp_field_manage_modify();
    }
	}	
}                                            
  

void 
action_cnt_field_manage_cb()
{
	GtkWidget *treeview = NULL;
	GtkWidget *sub_window = NULL;
	GtkWidget *cntlistWindow = NULL;
	const gchar *uid ;
	ab_contact_t *contact = NULL;
	gchar *full_name = NULL;
	GtkTreeSelection *selection = NULL;
	cntlistWindow = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
	sub_window = ui_comp_winmgr_get_window (UI_COMP_SKW_FIELD_VIEW);
	g_print("%s() sub_window is NULL; create a new one \n",__FUNCTION__);
  treeview = ui_comp_mark_view_new ();
  ui_comp_mark_view_set_checkcolumn_visible(UI_COMP_MARK_VIEW(treeview),FALSE);
  
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gem_tree_view_get_view(GEM_TREE_VIEW(treeview))));
 
  uid = (gint)action_get_current_contact_uid (UI_COMP_SKWINDOW(cntlistWindow));
  ab_get_contact (AB_ENABLER_SESS_LOCAL,(ab_uid_t)uid,&contact);
  ab_contact_get_full_name(contact,&full_name);  
	  
	ab_enabler_ui_comp_mark_view_parse_field(treeview,uid);
	
	gem_tree_view_set_search_column(GEM_TREE_VIEW(treeview),MARK_COL_MV_UID,GTK_SORT_ASCENDING,NULL);
	
	sub_window = ui_comp_skwindow_new(treeview,
																		"Option",
																		"Back",												
																		uidesc_field,
																		actions_field,
																		G_N_ELEMENTS(actions_field),
																		full_name);
  gtk_tree_selection_set_select_function(selection,mark_view_selection_func,sub_window,NULL);
  
 																			
	ui_comp_winmgr_register_window (UI_COMP_SKW_FIELD_VIEW,sub_window);																		
					
	g_signal_connect(G_OBJECT(gem_tree_view_get_view(GEM_TREE_VIEW(treeview))),
									 "row-activated",
									 G_CALLBACK(ui_comp_row_activated_field_manage_cb),
									 treeview);								
																		
	g_signal_connect(G_OBJECT(sub_window),
									 "softkey_activated",
									 G_CALLBACK(softkey_activated_field_manage_cb),
									 NULL);									
  gtk_widget_show_all (sub_window);
  return;
}

static gchar * uidesc = " \
	<ui>\
		<popup  action=\"PopupLeft\">\
		<menu name=\"comunication\"					action=\"Comunication\">\
			<menuitem name=\"call_cnt\"				action=\"CallCnt\"/>\
			<menuitem name=\"message_cnt\"		action=\"MessageCnt\"/>\
		</menu>\
		<menu name=\"management\"           action=\"Management\">\
			<menuitem name=\"add_cnt\"				action=\"AddCnt\"/>\
			<menuitem name=\"add_cnt_sim\"		action=\"AddCntSim\"/>\
			<menuitem name=\"modify_cnt\"			action=\"ModifyCnt\"/>\
			<menuitem name=\"delete_cnt\"			action=\"DeleteCnt\"/>\
			<menuitem name=\"field_manage\"		action=\"FieldManage\"/>\
    </menu>\
    <menu name=\"advanced\"							action=\"Advanced\">\
    	<menuitem name=\"copy_contact\"		action=\"CopyCnt\"/>\
    	<menuitem name=\"send_contact\"		action=\"SendCnt\"/>\
		</menu>\
			<menuitem name=\"mark_cnt\"				action=\"MarkCnt\"/>\
			<menuitem name=\"sort\"						action=\"Sort\"/>\
		</popup>\
	</ui>\
";

//			<menuitem name=\"category\"			action=\"Category\"/>\
//			<menuitem name=\"add_to_cat\"			action=\"AddToCAT\"/>\
//     		<separator/>\
//			<menuitem name=\"list_cnt\"			action=\"ListCnt\"/>\

static GtkActionEntry actions[] = {
	{"Comunication",	NULL,	"_Comunication",	"",	"none",NULL},
	{"CallCnt",			NULL, 	"_Call",					"", "none",action_cnt_call_contact_cb },
	{"MessageCnt",	NULL, 	"_Message",	"", "none",action_cnt_message_contact_cb },
	{"Management",	NULL, 	"_Manage",		"", "none",NULL},
  {"AddCnt",			NULL, 	"_Add Local",			"", "none",action_cnt_add_contact_cb },
  {"AddCntSim",		NULL, 	"_Add SIM",				"", "none",action_cnt_add_contact_sim_cb },
  {"ModifyCnt",		NULL, 	"_Modify","", "none",action_cnt_modify_contact_cb},
  {"DeleteCnt",		NULL, 	"_Delete","", "none",action_cnt_delete_contact_cb},
  {"FieldManage",	NULL, 	"_Field Manage","", "none",action_cnt_field_manage_cb},
  {"Advanced",		NULL, 	"_Advanced",			"",	"none",NULL},
  {"CopyCnt",			NULL,		"_Copy",	"",	"none",NULL},
	{"SendCnt",			NULL,		"_Send",	"",	"none",NULL},
  {"MarkCnt",			NULL, 	"_Mark/Unmark",		"", "none",action_cnt_mark_cb},
	{"Sort",				NULL,		"_Sort by Group",	"",	"none",action_cnt_sort_by_group_cb},
	{"PopupLeft",		NULL, 	"Popup",					"", "none",NULL},
};

//	{"ListCnt",			NULL, "_Detail",				"", "none",action_cnt_list_contact_cb },
//  {"Category",		NULL, "_Category",		"", "none", action_cat_show_category_cb},
//	{"AddToCAT",		NULL, "_Add to Category",		"", "none", action_add_to_category_cb},

GtkWidget *
ui_comp_main_window_new()
{
	GtkWidget *window = NULL;
	GtkWidget *mainview = NULL;
	GtkWidget *sktitle = NULL;

	mainview = ui_comp_init_view_new(); 
	
	gem_tree_view_searchentry_visible(FALSE);
//	ui_comp_tree_view_set_checkcolumn_visible(mainview,FALSE);
	ab_enabler_ui_comp_main_window(UI_COMP_INIT_VIEW(mainview));
	
	window = ui_comp_skwindow_new(mainview,
																"OK",
																"Exit",															
																NULL,
																NULL,
																0,
																"Contacts");	

//	g_print("%s():  cntview = 0x%08x \n",__FUNCTION__,(guint32)maiview);																	
	g_signal_connect(G_OBJECT(window),
									 "softkey_activated",
									 G_CALLBACK(main_window_softkey_activated_cb),
									 NULL);	
	ui_comp_winmgr_register_window(UI_COMP_SKW_INIT_VIEW,
																 window);															 
	return window;
}

void
cnt_list_window_show_cb (GtkWidget	*widget,
												 gpointer		 user_data)
{
	GtkWidget *view = NULL;
	ab_thread_t *thread1 = NULL;
	view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(widget));	

	g_print("%s () :  widget = 0x%08x  ............\n",__FUNCTION__,widget);	
#if 1
	thread1 = ab_thread_create(ab_enabler_cnt_list_view_load_contacts_thread,
														(gpointer)view,
									 					TRUE);																						
  gem_notification_set_text(GEM_NOTIFICATION(thread1->notifier),
                            "Loading ...");
	ab_thread_run(thread1);

#else
	thread1 = ab_thread_create(ab_enabler_cat_list_view_load_category_thread,
														(gpointer)view,
									 					TRUE);																						
	ab_thread_run(thread1);
# endif
	return;
}

GtkWidget *
ui_comp_cnt_list_window_new()
{
	GtkWidget *window = NULL;
	GtkWidget *cntview = NULL;
	GtkWidget *sktitle = NULL;

	cntview = ui_comp_cnt_list_view_new(); 
	gem_tree_view_set_search_column(GEM_TREE_VIEW(cntview),UI_COMP_CNT_LV_NAME,GTK_SORT_ASCENDING,NULL);
	
  gpe_skwindow_set_menu_fullscreen(TRUE);
	gem_tree_view_searchentry_visible(TRUE);							
	window = ui_comp_skwindow_new(cntview,
																"Options",
																"Back",															
																uidesc,
																actions,
																G_N_ELEMENTS(actions),
																"Contacts");	
//	g_signal_connect(G_OBJECT(window),
//									 "show",
//									 G_CALLBACK(cnt_list_window_show_cb),
//									 NULL);
	cnt_list_window_show_cb(window,NULL);
	g_print("%s():  cntview = 0x%08x \n",__FUNCTION__,(guint32)cntview);
	g_signal_connect(G_OBJECT(window),
									 "softkey_activated",
									 G_CALLBACK(softkey_activated_cb),
									 NULL);	
	ui_comp_winmgr_register_window(UI_COMP_SKW_CNT_VIEW,
																 window);															 
	return window;
}

static void
ui_comp_field_manage_add_cb(GpeSKWindow		*window,
														gint					 nr,
														gpointer			 user_data)
{
	if(nr==UI_COMP_SK_LEFT)
	{
		ab_ftid_t ftid = AB_ID_INVALID;
		gchar *uid = AB_ID_INVALID;
		ab_contact_t *contact = NULL;
		ab_fid_t fid = AB_ID_INVALID;
		GtkWidget *cntlistWindow = NULL;
		GtkWidget *fieldview = NULL;
		
		fieldview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));		
		ftid = (gint)user_data;
		cntlistWindow = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
	  uid = (ab_uid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(cntlistWindow));
    ab_get_contact (AB_ENABLER_SESS_LOCAL,(ab_uid_t)uid,&contact);
	  
		if(contact)
		{
			gchar *value = NULL;
			ab_contact_add_field(contact, 
													 ftid,
													 &fid);
													 
	    value = gem_hash_view_get_value(GEM_HASH_VIEW(fieldview),
												      				FIELD_VALUE);

			ab_contact_set_string_field(contact,
																	fid,
																	value);
			value = gem_hash_view_get_value(GEM_HASH_VIEW(fieldview),
	    										  					FIELD_LABEL);
			ab_contact_set_field_label(contact,fid,value);																	
																	
			ab_update_contact (AB_ENABLER_SESS_LOCAL,contact);													
/*			g_print("%s() LINE(%d) ftid(%d) fid(%d) uid(%d) value(%s)\n",__FUNCTION__,
																																	 __LINE__,
																																	 ftid,
																																	 fid,
																																	 uid,
																													 				 value);			*/
			GtkTreeModel *model = NULL;
		 	GtkTreePath *path = NULL;
		  GtkTreeViewColumn *col = NULL;
		  GtkWidget *field_list_window = NULL;
		  GtkWidget *field_list_view = NULL;
		  GtkTreeIter iter_parent;
		  
		  field_list_window = ui_comp_winmgr_get_window (UI_COMP_SKW_FIELD_VIEW);
		  field_list_view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(field_list_window));	
		  
			model = gem_tree_view_get_model(GEM_TREE_VIEW(field_list_view));
			gtk_tree_view_get_cursor ((GtkTreeView *)gem_tree_view_get_view(GEM_TREE_VIEW(field_list_view)), &path, &col);
	
			if (gtk_tree_model_get_iter (model, &iter_parent, path))
			{
				GtkTreeIter iter;
      	gtk_tree_store_append (GTK_TREE_STORE(model), &iter, &iter_parent);
        gtk_tree_store_set (GTK_TREE_STORE(model), &iter,
        										MARK_COL_MV_UID, fid,
                            MARK_COL_MV_STRING, value,
                            MARK_COL_MV_EXTEND, FIELD_EXTEND,
                            -1);
			}			
			gtk_tree_path_free (path);  																											 				 														
		}	  
	}
	else if(nr==UI_COMP_SK_RIGHT)
	{
		
	}
	gtk_widget_destroy (GTK_WIDGET(window));
	return;
}											

static void ui_comp_field_manage_add()
{
  GtkWidget *window = NULL;
  gchar *label = NULL;
  ab_uid_t uid = AB_ID_INVALID;
  GtkWidget *fieldview = NULL;
  GtkWidget *fieldwindow = NULL;
  window = ui_comp_winmgr_get_window (UI_COMP_SKW_FIELD_VIEW);
  uid = (ab_uid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(window));
  
	g_print("%s(): uid = %d \n",__FUNCTION__,uid); 	   
	 	     
  if (uid == AB_ID_INVALID)
  {
    return;
  }
	ab_field_template_get_label  (ab_enabler_get_session(SESSION_LOCAL),uid,&label);
	
	fieldview = ui_comp_field_view_new();
	gem_hash_view_set_value(GEM_HASH_VIEW(fieldview),
										      FIELD_NAME,
										      label);		
	
	fieldwindow = ui_comp_skwindow_new(fieldview,
																		"Confirm",
																		"Back",															
																		NULL,
																		NULL,
																		0,
																		"New Field");	
	
	g_signal_connect(G_OBJECT(fieldwindow),
									 "softkey_activated",
									 G_CALLBACK(ui_comp_field_manage_add_cb),
									 (gpointer) uid);
									 
	ui_comp_winmgr_register_window(UI_COMP_SKW_FIELD_VIEW_EDIT,
																 window);		
	gtk_widget_show_all(fieldwindow);
  return;
}
void 
ui_comp_field_manage_modify_cb(GpeSKWindow		*window,
															 gint					 nr,
															 gpointer			 user_data)
{
	if(nr==UI_COMP_SK_LEFT)
	{
		ab_ftid_t ftid = AB_ID_INVALID;
		gchar *uid = AB_ID_INVALID;
		ab_contact_t *contact = NULL;
		ab_fid_t fid = AB_ID_INVALID;
		GtkWidget *cntlistWindow = NULL;
		GtkWidget *fieldview = NULL;
		
		fieldview = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(window));		
		fid = (gint)user_data;
		cntlistWindow = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
	  uid = (ab_uid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(cntlistWindow));
    ab_get_contact (AB_ENABLER_SESS_LOCAL,(ab_uid_t)uid,&contact);
	  
		if(contact)
		{
			gchar *value = NULL;
													 
	    value = gem_hash_view_get_value(GEM_HASH_VIEW(fieldview),
												      				FIELD_VALUE);

			ab_contact_set_string_field(contact,
																	fid,
																	value);
																	
			value = gem_hash_view_get_value(GEM_HASH_VIEW(fieldview),
	    										  					FIELD_LABEL);
			ab_contact_set_field_label(contact,fid,value);																	
																	
			ab_update_contact (AB_ENABLER_SESS_LOCAL,contact);		
			
			
			GtkTreeModel *model = NULL;
		 	GtkTreePath *path = NULL;
		  GtkTreeViewColumn *col = NULL;
		  GtkWidget *field_list_window = NULL;
		  GtkWidget *field_list_view = NULL;
		  GtkTreeIter iter;
		  
		  field_list_window = ui_comp_winmgr_get_window (UI_COMP_SKW_FIELD_VIEW);
		  field_list_view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(field_list_window));	
		  
			model = gem_tree_view_get_model(GEM_TREE_VIEW(field_list_view));
			gtk_tree_view_get_cursor ((GtkTreeView *)gem_tree_view_get_view(GEM_TREE_VIEW(field_list_view)), &path, &col);
	
			if (gtk_tree_model_get_iter (model, &iter, path))
			{
	      gtk_tree_store_set (GTK_TREE_STORE(model), &iter,
				                    MARK_COL_MV_STRING, value,
				                    -1);
			}
			gtk_tree_path_free (path);  	
														
		}	  
	}
	
	else if(nr==UI_COMP_SK_RIGHT)
	{
		
	}
	gtk_widget_destroy (GTK_WIDGET(window));
	return;
}


static void 
ui_comp_field_manage_modify()
{
 	GtkWidget *window = NULL;
  gchar *label = NULL;
  gchar *fid_label = NULL;
  gchar *value = NULL;
	ab_ftid_t ftid = AB_ID_INVALID;
  ab_fid_t fid = AB_ID_INVALID;
  ab_uid_t uid = AB_ID_INVALID;
  GtkWidget *cntlistWindow = NULL;
  GtkWidget *fieldview = NULL;
  GtkWidget *fieldwindow = NULL;
  ab_contact_t *contact = NULL;
  window = ui_comp_winmgr_get_window (UI_COMP_SKW_FIELD_VIEW);
  fid = (ab_fid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(window));
  
	g_print("%s(): fid = %d \n",__FUNCTION__,fid); 	   
	 	     
  if (fid == AB_ID_INVALID)
  {
    return;
  }
  
  cntlistWindow = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
  uid = (ab_uid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(cntlistWindow));
  ab_get_contact (AB_ENABLER_SESS_LOCAL,uid,&contact);
  
  if(contact==NULL)
  {
  	return;
  }
  
  ab_contact_get_ftid	(contact,fid,&ftid);
  
	ab_field_template_get_label(ab_enabler_get_session(SESSION_LOCAL),ftid,&label);
	
	ab_contact_get_field_label(contact,fid,&fid_label);
	fieldview = ui_comp_field_view_new();
	
	gem_hash_view_set_value(GEM_HASH_VIEW(fieldview),
										      FIELD_NAME,
										      label);
										      		
	gem_hash_view_set_value(GEM_HASH_VIEW(fieldview),
										      FIELD_LABEL,
										      fid_label);		
										      
	ab_contact_get_string_field(contact,fid,&value);	
	
	gem_hash_view_set_value(GEM_HASH_VIEW(fieldview),
										      FIELD_VALUE,
										      value);											      
										      
	fieldwindow = ui_comp_skwindow_new(fieldview,
																		"Confirm",
																		"Back",															
																		NULL,
																		NULL,
																		0,
																		"Modify Field");	
	
	g_signal_connect(G_OBJECT(fieldwindow),
									 "softkey_activated",
									 G_CALLBACK(ui_comp_field_manage_modify_cb),
									 (gpointer) fid);
									 
	ui_comp_winmgr_register_window(UI_COMP_SKW_FIELD_VIEW_EDIT,
																 window);		
	gtk_widget_show_all(fieldwindow);
  return;
}

void
ui_comp_field_delete_cb(GpeSKWindow		*window,
											  gint					 nr,
											  gpointer			 user_data)
{
	if(nr==UI_COMP_SK_LEFT)
	{
		ab_fid_t fid = (ab_fid_t)user_data;
	  ab_contact_t *contact = NULL; 
	  GtkWidget *CntWindow = NULL;
	  ab_uid_t uid = AB_ID_INVALID;
	  CntWindow = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
	  uid = (ab_uid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(CntWindow));
	  ab_get_contact (AB_ENABLER_SESS_LOCAL, uid, &contact);
	  if(contact)
	  {
  		ab_contact_remove_field	(contact,fid);
			ab_update_contact (AB_ENABLER_SESS_LOCAL,contact);
						
			GtkTreeModel *model = NULL;
		 	GtkTreePath *path = NULL;
		  GtkTreeViewColumn *col = NULL;
		  GtkWidget *field_list_window = NULL;
		  GtkWidget *field_list_view = NULL;
		  GtkTreeIter iter;
		  
		  field_list_window = ui_comp_winmgr_get_window (UI_COMP_SKW_FIELD_VIEW);
		  field_list_view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(field_list_window));	
		  
			model = gem_tree_view_get_model(GEM_TREE_VIEW(field_list_view));
			gtk_tree_view_get_cursor ((GtkTreeView *)gem_tree_view_get_view(GEM_TREE_VIEW(field_list_view)), &path, &col);
			if (gtk_tree_model_get_iter (model, &iter, path))
			{
   			gtk_tree_store_remove (GTK_TREE_STORE(model), &iter);
			}
			
			gtk_tree_path_up (path);
      gtk_tree_view_set_cursor (GTK_TREE_VIEW(gem_tree_view_get_view(GEM_TREE_VIEW(field_list_view))), path, NULL, TRUE);
      gtk_tree_path_free (path);
	  }
	}
	else if (nr==UI_COMP_SK_RIGHT)
	{
	}
	gtk_widget_destroy (GTK_WIDGET(window));
}

static void 
ui_comp_field_manage_delete()
{
	GtkWidget *dialog = NULL;
  ab_fid_t fid = AB_ID_INVALID;
	GtkWidget *fieldWindow = NULL;
  fieldWindow = ui_comp_winmgr_get_window (UI_COMP_SKW_FIELD_VIEW);
	fid = (ab_fid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(fieldWindow));
	 	     
  if (fid == AB_ID_INVALID)
  {
    return;
  }
	
	dialog = g_object_new(UI_COMP_TYPE_SKWINDOW, NULL);
	gem_sk_window_set_delete_warning (dialog, "Confirm to delete this field? \n\n");
	gem_sk_dialog_set_position (fieldWindow,dialog, 50);
	
	g_signal_connect(G_OBJECT(dialog),
									 "softkey_activated",
									 G_CALLBACK(ui_comp_field_delete_cb),
									 (gpointer) fid);
	gtk_widget_show_all(dialog);

  return;	
}

void
ui_comp_field_deleteall_cb(GpeSKWindow		*window,
											  gint					 nr,
											  gpointer			 user_data)
{
	if(nr==UI_COMP_SK_LEFT)
	{
		ab_ftid_t ftid = (ab_fid_t)user_data;
	  ab_contact_t *contact = NULL; 
	  GtkWidget *CntWindow = NULL;
	  ab_uid_t uid = AB_ID_INVALID;
	  CntWindow = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_VIEW);
	  uid = (ab_uid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(CntWindow));
	  ab_get_contact (AB_ENABLER_SESS_LOCAL, uid, &contact);
	  if(contact)
	  {		      
			GtkTreeModel *model = NULL;
		 	GtkTreePath *path = NULL;
		 	GtkTreePath *path_tmp = NULL;
		  GtkTreeViewColumn *col = NULL;
		  GtkWidget *field_list_window = NULL;
		  GtkWidget *field_list_view = NULL;
		  GtkTreeIter iter;
		  GtkTreeIter iter_child;
		  GtkTreeIter iter_tmp;
	  	ab_fid_t *fids = NULL;
	  	gint j =0 ;
	  	gboolean flag = TRUE;
			ab_contact_list_fields_by_template(contact,ftid,&fids); 
	  	for (j = 0; fids[j]!=AB_ID_INVALID; j++)
      {
      	if(!ab_enabler_field_whether_default(contact,ftid,fids[j]))
      	{
      		ab_contact_remove_field	(contact,fids[j]);
					ab_update_contact (AB_ENABLER_SESS_LOCAL,contact);
	     	}
      }
		  field_list_window = ui_comp_winmgr_get_window (UI_COMP_SKW_FIELD_VIEW);
		  field_list_view = ui_comp_skwindow_get_view(UI_COMP_SKWINDOW(field_list_window));	
		  
			model = gem_tree_view_get_model(GEM_TREE_VIEW(field_list_view));
			gtk_tree_view_get_cursor ((GtkTreeView *)gem_tree_view_get_view(GEM_TREE_VIEW(field_list_view)), &path, &col);
			if (gtk_tree_model_get_iter (model, &iter, path))
			{
				gint i = 0 ;
				gint count = gtk_tree_model_iter_n_children (model,&iter);
				for(i = 1;i < count;i++)				
				{
					gtk_tree_model_iter_nth_child (model,&iter_child,&iter,1);
					gtk_tree_store_remove (GTK_TREE_STORE(model), &iter_child);
				}
			}
            
			gtk_tree_view_set_cursor (GTK_TREE_VIEW(gem_tree_view_get_view(GEM_TREE_VIEW(field_list_view))), path, NULL, TRUE);
      gtk_tree_path_free (path);
	  }
	}
	else if (nr==UI_COMP_SK_RIGHT)
	{
	}
	gtk_widget_destroy (GTK_WIDGET(window));
}											  
											  

static void ui_comp_field_manage_deleteall()
{
	GtkWidget *dialog = NULL;
  ab_ftid_t ftid = AB_ID_INVALID;
	GtkWidget *fieldWindow = NULL;
  fieldWindow = ui_comp_winmgr_get_window (UI_COMP_SKW_FIELD_VIEW);
	ftid = (ab_fid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(fieldWindow));
	 	     
  if (ftid == AB_ID_INVALID)
  {
    return;
  }
	
	dialog = g_object_new(UI_COMP_TYPE_SKWINDOW, NULL);
	gem_sk_window_set_delete_warning (dialog, "Delete all of this field? \n\n");
	gem_sk_dialog_set_position (fieldWindow,dialog, 50);
	
	g_signal_connect(G_OBJECT(dialog),
									 "softkey_activated",
									 G_CALLBACK(ui_comp_field_deleteall_cb),
									 (gpointer) ftid);
	gtk_widget_show_all(dialog);
}

static void ui_comp_send_msg_multi()
{
  GtkWidget *sk_window = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_MARK_VIEW);
  GtkWidget *view = ui_comp_skwindow_get_view (UI_COMP_SKWINDOW(sk_window));
  GPtrArray *number_pairs = NULL;

  g_print ("%s(): entering\n", __FUNCTION__);

  number_pairs = g_ptr_array_new ();

  if (view)
  {
    GtkTreeModel *model = NULL;
		model = gem_tree_view_get_model(GEM_TREE_VIEW(view));
		
    if (model)
    {
      GtkTreeIter toplevel;

      if (gtk_tree_model_get_iter_first (model, &toplevel))
      {
        do
        {
          gchar *name = NULL;
          GtkTreeIter iter;

          gtk_tree_model_get (model, &toplevel, COL_MV_STRING, &name, -1);

//                                      g_print("%s(): name = %s\n", __FUNCTION__, name);
          if (gtk_tree_model_iter_children (model, &iter, &toplevel))
          {
            do
            {
              gchar *number = NULL;
              gboolean checked = FALSE;

              gtk_tree_model_get (model, &iter,
                                  COL_MV_STRING, &number,
                                  COL_MV_CHECK, &checked, -1);

              if (checked)
              {
                NumberPair *pair = NULL;

//                                                              g_print("%s(): number = %s\n", __FUNCTION__, number);
                pair = number_pair_new (name, number);
                if (pair)
                {
                  g_ptr_array_add (number_pairs, (gpointer) pair);
                }
              }

              g_free (number);
            } while (gtk_tree_model_iter_next (model, &iter));
          }

          g_free (name);
        } while (gtk_tree_model_iter_next (model, &toplevel));
      }
    }
  }
  abook_iac_multi_sms (number_pairs);
  if (TRUE)
  {
    gint i;
    gint count;

    count = number_pairs->len;
    for (i = 0; i < count; i++)
    {
      NumberPair *pair = NULL;

      pair = (NumberPair *) g_ptr_array_remove_index_fast (number_pairs, 0);
      g_print ("%s(): pair [%20s, %s]\n", __FUNCTION__, pair->name,
               pair->number);

      number_pair_free (pair);
    }
  }

  g_ptr_array_free (number_pairs, TRUE);

  gtk_widget_hide_all (sk_window);

  return;

}


gpointer
ui_comp_cnt_mark_view_delete_contacts_thread (gpointer arg)
{
  ab_uid_t *uids_local = NULL;
  ab_uid_t *uids_sim = NULL;
	ab_thread_t	*thread = (ab_thread_t *)arg;
	GtkTreeModel *cntlist_model = NULL;
	GtkWidget *cntlistWindow = NULL;
	GtkWidget *cntlistView = NULL;
	cntlistWindow = ui_comp_winmgr_get_window(UI_COMP_SKW_CNT_VIEW);
	cntlistView = ui_comp_skwindow_get_view (UI_COMP_SKWINDOW(cntlistWindow));
	cntlist_model = gem_tree_view_get_model(GEM_TREE_VIEW(cntlistView));
	
	
	ab_enabler_cnt_list_hash_table_update(UI_COMP_CNT_LIST_VIEW(cntlistView));
	
	UICompMarkView *view = UI_COMP_MARK_VIEW(thread->user_data);
	GemNotification *notifier = GEM_NOTIFICATION(thread->notifier);
	
	gem_notification_set_progross_fraction(notifier,0);

  ab_get_contact_uids(AB_ENABLER_SESS_LOCAL,
								    	NULL,
								    	&uids_local);
  ab_get_contact_uids(AB_ENABLER_SESS_SIM,
								    	NULL,
								    	&uids_sim);
									    	
	gint count = 0;
	gint i = 0;
	if (uids_local)
	{
		for (i = 0; uids_local[i] != AB_ID_INVALID; i++)
		{
			count++;			
		}
  }
  if (uids_sim)
	{
		for (i = 0; uids_sim[i] != AB_ID_INVALID; i++)
		{
			count++;			
		}
  }
  
  if (view && count > 0)
  {
    GtkTreeModel *model = NULL;
  	GList *rr_list = NULL;
  	GList *node = NULL;
  	model = gem_tree_view_get_model(GEM_TREE_VIEW(view));
//  	gtk_tree_model_foreach(model,(GtkTreeModelForeachFunc) foreach_func,&rr_list);
    GtkTreeIter toplevel;
  	if (gtk_tree_model_get_iter_first (model, &toplevel))
    {
    	gint prg = 0;
      gchar *uidstr = NULL;
      gboolean checked = FALSE;
      gchar *storage = NULL;
     	GtkTreePath *path = NULL;
			GtkTreeViewColumn *col = NULL;
	    GtkTreeRowReference *rowref;
	    GtkTreeIter *iter_list;
  		GArray *array_local = NULL;
  		GArray *array_sim = NULL;
			guint32 *uids_local = NULL;
			guint32 *uids_sim = NULL;
			ab_uid_t endflag = AB_ID_INVALID;
			array_local = g_array_new(FALSE,
																TRUE,
																sizeof(guint32));
			array_sim = g_array_new(FALSE,
															TRUE,
															sizeof(guint32));
																
      do
      {
      	path = gtk_tree_model_get_path (model,&toplevel);
        gtk_tree_model_get (model, &toplevel,
                            COL_MV_UID, &uidstr,
                            COL_MV_CHECK, &checked,
                            UI_COMP_CNT_LV_STORAGE, &storage,
                            -1);
        if (uidstr && checked)
        {
          ab_uid_t uid = atoi(uidstr);
          if((gint)storage==0)
          {
  	   			g_array_append_val(array_local, uid);
          }
          else if ((gint)storage==1)
          {
          	g_array_append_val(array_sim, uid);
          }

			    rowref = gtk_tree_row_reference_new(model,path);
			    rr_list = g_list_append(rr_list,rowref);
			    	  
	    		iter_list = ui_comp_cnt_list_hash_table_get(cntlistView,&uid);
	    
	    		GDK_THREADS_ENTER();
	    		if(iter_list)
	    		{
						gtk_list_store_remove(GTK_LIST_STORE(cntlist_model),iter_list);
	    		}
					ui_comp_cnt_list_hash_table_remove(cntlistView,&uid);
//					gdouble percentage = (gdouble)prg / count;
//
//					gem_notification_set_progross_fraction(notifier,percentage);
					GDK_THREADS_LEAVE();
        }
				GDK_THREADS_ENTER();
				gdouble percentage = (gdouble)prg / count;
				gem_notification_set_progross_fraction(notifier,percentage);
				GDK_THREADS_LEAVE();
        
				prg++;

      } while (gtk_tree_model_iter_next (model, &toplevel));
			if(uidstr!=NULL)
			{
//        	g_free (uidstr);
			}
      
			g_array_append_val(array_local, endflag);      
  		g_array_append_val(array_sim, endflag);
		
			uids_local = (guint32 *)g_array_free(array_local, FALSE);
			uids_sim = (guint32 *)g_array_free(array_sim, FALSE);
			
			ab_remove_contacts(AB_ENABLER_SESS_LOCAL, uids_local);
			ab_remove_contacts(AB_ENABLER_SESS_SIM, uids_sim);
//			GDK_THREADS_ENTER();
//	  	gem_notification_set_progross_fraction(notifier,0.95);
//	  	GDK_THREADS_LEAVE();

		  gtk_tree_path_free (path);
    }
  	
  	
		gint i = 0;  	
  	for(node = rr_list;  i<g_list_length(rr_list) ; node = g_list_next(node),i++)
  	{
  		GtkTreePath *path_delete;
  		path_delete = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data);
  		if(path_delete)
  		{
  			GtkTreeIter iter;
  			if(gtk_tree_model_get_iter(GTK_TREE_MODEL(model),&iter,path_delete))
  			{
			  	GDK_THREADS_ENTER();
					gtk_tree_store_remove(GTK_TREE_STORE(model),&iter);
//					gem_notification_set_progross_fraction(notifier,
//																	((gdouble)0.05* i / g_list_length(rr_list) + 0.95));
//g_print("%s() LINE(%d) p(%d) count(%d) prg(%d)\n",__FUNCTION__,__LINE__,i,count,(gdouble)i / ( 2 * count));																	
      	  GDK_THREADS_LEAVE();																			 
  			}
  		}
		  gtk_tree_path_free (path_delete);
  	}
  	if(rr_list)
  	{
  		g_list_free(rr_list);
  	}
  	GDK_THREADS_ENTER();
  	ui_comp_cnt_list_view_update (UI_COMP_CNT_LIST_VIEW(cntlistView));
  	ui_comp_mark_view_update (UI_COMP_MARK_VIEW(view));
  	GDK_THREADS_LEAVE();
  }
  
  gtk_widget_hide_all(GTK_WIDGET(notifier));

	return NULL;	
}


static void ui_comp_cnt_multi_delete_cb(GpeSKWindow		*window,
																			  gint					 nr,
																			  gpointer			 user_data)
{
	gtk_widget_destroy (GTK_WIDGET(window));
	if(nr == UI_COMP_SK_LEFT)
	{
		
		ab_thread_t *thread1 = NULL;
	  GtkWidget *sk_window = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_MARK_VIEW);
	  GtkWidget *view = ui_comp_skwindow_get_view (UI_COMP_SKWINDOW(sk_window));
		
		thread1 = ab_thread_create(ui_comp_cnt_mark_view_delete_contacts_thread,
															(gpointer)view,
										 					TRUE);																						
	  gem_notification_set_text(GEM_NOTIFICATION(thread1->notifier),
	                            "Deleting ...");
		ab_thread_run(thread1);
		
	}
	else if (nr==UI_COMP_SK_RIGHT)
	{
	}

}

static void ui_comp_delete_cnt_multi()
{
	GtkWidget *dialog = NULL;
//  ab_fid_t fid = AB_ID_INVALID;
	GtkWidget *fieldWindow = NULL;
  fieldWindow = ui_comp_winmgr_get_window (UI_COMP_SKW_CNT_MARK_VIEW);
//	fid = (ab_fid_t)action_get_current_contact_uid (UI_COMP_SKWINDOW(fieldWindow));
//	 	     
//  if (fid == AB_ID_INVALID)
//  {
//    return;
//  }
	
	dialog = g_object_new(UI_COMP_TYPE_SKWINDOW, NULL);
	gem_sk_window_set_delete_warning (dialog, "Confirm to delete these contact? \n\n");
	gem_sk_dialog_set_position (fieldWindow,dialog, 50);
	
	g_signal_connect(G_OBJECT(dialog),
									 "softkey_activated",
									 G_CALLBACK(ui_comp_cnt_multi_delete_cb),
									 NULL);
	gtk_widget_show_all(dialog);

  return;	
}	
void ui_comp_mem_status()
{
	GtkWidget *window = NULL;
	GtkWidget *memory_view = NULL;
	GtkWidget *sktitle = NULL;
  ab_uid_t *uids_local = NULL;
  ab_uid_t *uids_sim = NULL;
	
	memory_view = ui_comp_mem_status_view_new(); 
	
	window = ui_comp_skwindow_new(memory_view,
																"",
																"Back",															
																NULL,
																NULL,
																0,
																"Memory Status");	
	ui_comp_winmgr_register_window(UI_COMP_SKW_MEM_CHECK, window);	
//	g_print("%s():  cntview = 0x%08x \n",__FUNCTION__,(guint32)maiview);																	
	g_signal_connect(G_OBJECT(window),
									 "softkey_activated",
									 G_CALLBACK(softkey_activated_mem_check_cb),
									 NULL);	
									 
 	ab_get_contact_uids(AB_ENABLER_SESS_LOCAL,
								    	NULL,
								    	&uids_local);
  ab_get_contact_uids(AB_ENABLER_SESS_SIM,
								    	NULL,
								    	&uids_sim);
									    	
	gint count_local = 0;
	gint count_sim = 0;
	gint i = 0;
	if (uids_local)
	{
		for (i = 0; uids_local[i] != AB_ID_INVALID; i++)
		{
			count_local++;			
		}
  }
  if (uids_sim)
	{
		for (i = 0; uids_sim[i] != AB_ID_INVALID; i++)
		{
			count_sim++;			
		}
  }									 
	ui_comp_mem_status_progress_set_local(memory_view, count_local, 2000);					
	ui_comp_mem_status_progress_set_sim(memory_view, count_sim,150);				 
													 
	gtk_widget_show_all(window);
}


/*
vi:ts=2:nowrap:ai:expandtab 
*/
