/*	contact - LiPS Address Book Application
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */
#include <gtk/gtk.h>
#include <stdio.h>
#include <glib.h>
#include <gdk/gdkkeysyms.h>
#include <gpe/gpeskwindow.h>
#include <uicompskwindow.h>

typedef struct _UICompSKWindowPriv UICompSKWindowPriv;

struct _UICompSKWindowPriv
{
	GtkWidget 		*label;
	GtkWidget 		*viewport;
	GtkWidget			*view;
	GtkUIManager	*uimgr;
};

#define UI_COMP_SKWINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), \
																					 UI_COMP_TYPE_SKWINDOW, \
																					 UICompSKWindowPriv));
static void ui_comp_skwindow_class_init		(UICompSKWindowClass	*klass);
static void ui_comp_skwindow_init					(UICompSKWindow				*skwindow);
static void	ui_comp_skwindow_destroy			(GtkObject						*object);

static GpeSKWindowClass * parent_class = NULL;

GType
ui_comp_skwindow_get_type (void)
{
	static GType skwindow_type = 0;

  if (!skwindow_type)
	{
		static const GTypeInfo skwindow_info =
		{
			sizeof (UICompSKWindowClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) ui_comp_skwindow_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (UICompSKWindow),
			0,			/* n_preallocs */
			(GInstanceInitFunc) ui_comp_skwindow_init,
			NULL,
		};
		skwindow_type = g_type_register_static(GPE_TYPE_SKWINDOW,
																					 "UICompSKWindow",
																					 &skwindow_info,
																					 0);
	}
	return skwindow_type;
}

static void
ui_comp_skwindow_class_init (UICompSKWindowClass * klass)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*)klass;

  parent_class = g_type_class_peek_parent(klass);

	g_type_class_add_private(klass,
													 sizeof(UICompSKWindowPriv));

  object_class->destroy = ui_comp_skwindow_destroy;

	return;
}

static void
ui_comp_skwindow_destroy (GtkObject *object)
{
	UICompSKWindow * skwindow = UI_COMP_SKWINDOW(object);

  (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
  
  return;
}

static void
ui_comp_skwindow_init (UICompSKWindow * skwindow)
{
	UICompSKWindowPriv *priv = UI_COMP_SKWINDOW_GET_PRIVATE(skwindow);
	priv->label = NULL;
	priv->viewport = NULL;
	priv->view	= NULL;
	priv->uimgr	= NULL;
	return;
}

static void
ui_comp_skwindow_set_view (UICompSKWindow		*skwindow,
													 GtkWidget				*view,
													 gchar					  *title)
{
	UICompSKWindowPriv *priv = UI_COMP_SKWINDOW_GET_PRIVATE(skwindow);
	g_return_if_fail(view);
	
	GtkWidget *vbox = NULL;
	vbox = gtk_vbox_new(FALSE, 0);

	GtkWidget *label = NULL;
	GtkRcStyle *style = NULL;
	GtkWidget *viewport = NULL;
	gchar *picpath = NULL;
	viewport = gtk_viewport_new(NULL,NULL);
	if(title)
	{
		label = gtk_label_new (title);
	}
	else
		label = gtk_label_new(NULL);
	gtk_widget_set_size_request (GTK_WIDGET (label), 240, 28);
//		gtk_box_pack_start(GTK_BOX(hbox1), la bel, FALSE, FALSE, 3);
	picpath = g_build_filename(gem_env_get_resource_path(),
													   "pixmaps",
													   "top_background.png",
													   NULL);
													  
	style = gtk_widget_get_modifier_style (viewport);
  style->bg_pixmap_name[GTK_STATE_NORMAL] = picpath;
  gtk_widget_modify_style (viewport, style);
	PangoFontDescription *font_desc = pango_font_description_from_string ("Sans 16");
  gtk_widget_modify_font (label, font_desc);
  pango_font_description_free (font_desc);
  GdkColor color;
  gdk_color_parse ("white", &color);
  gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &color);
	gtk_label_set_justify (label,
                         GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
	
	gtk_container_add(GTK_CONTAINER(viewport), label);
	gtk_box_pack_start(GTK_BOX(vbox), viewport, FALSE, FALSE, 0);
	priv->viewport = viewport;
	priv->label = label;

	gtk_box_pack_start(GTK_BOX(vbox), view, TRUE, TRUE, 0);

	gtk_container_add(GTK_CONTAINER(skwindow), vbox);
//		gtk_container_add(GTK_CONTAINER(skwindow), eventbox);
	
	priv->view = view;
	return;
}

gboolean
skw_menu_popup_key_press_cb (GtkWidget *widget,
                             GdkEventKey *event, 
                             gpointer user_data)
{

  UICompSKWindow *sk_window = (UICompSKWindow *) user_data;
	
  if (event->keyval == GDK_F4 || event->keyval == GDK_KP_F4)
  {
  	gtk_menu_popdown (GTK_MENU (widget));
		gpe_skwindow_set_softkey_label(GPE_SKWINDOW(sk_window),UI_COMP_SK_RIGHT,"Back");
  //  gtk_widget_grab_focus (GTK_WIDGET (sk_window->window));
    return TRUE;
  }
  return FALSE;
}

static void
ui_comp_skwindow_parse_sk_conf (UICompSKWindow	*skwindow,
																const gchar			*left_sk_label,
																const gchar			*right_sk_label,
																const gchar			*sk_uidesc,
																GtkActionEntry	*sk_actions,
																guint						 n_sk_actions)
{
	UICompSKWindowPriv *priv = UI_COMP_SKWINDOW_GET_PRIVATE(skwindow);
	GtkActionGroup	*action_group = NULL;
	GtkWidget *menu_left = NULL;
	GtkWidget *menu_right = NULL;
	GError *error = NULL;
	if (sk_actions)
	{
		priv->uimgr	 = gtk_ui_manager_new();
		g_print("%s() LINE(%d) priv->uimgr(0x%08x)\n",__FUNCTION__,__LINE__,(guint32)priv->uimgr);	
		action_group = gtk_action_group_new ("Actions");
    
		gtk_action_group_add_actions(action_group,
																 sk_actions,
																 n_sk_actions,
																 NULL);
	/*	g_print("%s(): n_sk_actions = %d\n",
						__FUNCTION__,
						n_sk_actions);
	*/
		gtk_ui_manager_insert_action_group(priv->uimgr,
																			 action_group,
																			 0);
		if (!gtk_ui_manager_add_ui_from_string(priv->uimgr,
																					 sk_uidesc,
																					 -1,
																					 &error))     
		{
	      g_print("%s(): failed to parse menu desc: %s\n",
	      				__FUNCTION__,
	      				error->message);
	      g_error_free(error);
	      return;
		}
		menu_left = gtk_ui_manager_get_widget(priv->uimgr,
																					"/PopupLeft");
		if (menu_left)
		{
			g_signal_connect (G_OBJECT (menu_left),
		                    "key-press-event",
		                     G_CALLBACK (skw_menu_popup_key_press_cb),
		                     (gpointer) skwindow);
		                     
			gpe_skwindow_set_softkey_menu(GPE_SKWINDOW(skwindow),
																		UI_COMP_SK_LEFT,
																		menu_left);
		}
		else
		{
			gpe_skwindow_set_softkey_menu(GPE_SKWINDOW(skwindow),
																		UI_COMP_SK_LEFT,
																		NULL);
		}
		
		menu_right = gtk_ui_manager_get_widget(priv->uimgr,
																				"/PopupRight");

		if (menu_right)
		{
			gpe_skwindow_set_softkey_menu(GPE_SKWINDOW(skwindow),
																		UI_COMP_SK_RIGHT,
																		menu_right);
		}
		else
		{
			gpe_skwindow_set_softkey_menu(GPE_SKWINDOW(skwindow),
																		UI_COMP_SK_RIGHT,
																		NULL);
		}
	}
	else
	{
		priv->uimgr = NULL;
		gpe_skwindow_set_softkey_menu(GPE_SKWINDOW(skwindow),
																	UI_COMP_SK_LEFT,
																	NULL);
		gpe_skwindow_set_softkey_menu(GPE_SKWINDOW(skwindow),
																	UI_COMP_SK_RIGHT,
																	NULL);
	}	
	gpe_skwindow_set_softkey_label(GPE_SKWINDOW(skwindow), 
																 UI_COMP_SK_LEFT,
																 left_sk_label);
	
  gpe_skwindow_set_softkey_label(GPE_SKWINDOW(skwindow),
  															 UI_COMP_SK_RIGHT,
  															 right_sk_label);
	return;
}

static void
ui_comp_skwindow_show_cb (GtkWidget *widget, gpointer user_data)
{
	GtkWidget *viewport = NULL;
	GtkWidget *label = NULL;
	const gchar *text = NULL;

	label = ui_comp_skwindow_get_label(UI_COMP_SKWINDOW(widget));
	text = gtk_label_get_text(GTK_LABEL(label));
	if(!(strcmp(text,"")))
	{
  	viewport = ui_comp_skwindow_get_viewport(UI_COMP_SKWINDOW(widget));
  	gtk_widget_hide_all(viewport);
	}	
}

GtkWidget *
ui_comp_skwindow_new (GtkWidget					*view,
											const gchar				*left_sk_label,
											const gchar				*right_sk_label,
											const gchar				*sk_uidesc,
											GtkActionEntry		*sk_actions,
											guint							 n_sk_actions,
											const gchar					 		*title)
{
  GtkWidget *window = NULL;
  window = gtk_widget_new(UI_COMP_TYPE_SKWINDOW,
													NULL);
	g_return_val_if_fail(window, NULL);
	
	ui_comp_skwindow_set_view(UI_COMP_SKWINDOW(window),
														view,
														title);
	ui_comp_skwindow_parse_sk_conf(UI_COMP_SKWINDOW(window),
																 left_sk_label,
																 right_sk_label,
																 sk_uidesc,
																 sk_actions,
																 n_sk_actions);
	g_signal_connect(G_OBJECT(window),
									 "show",
									 G_CALLBACK (ui_comp_skwindow_show_cb),
									 NULL);																 
	return window;
}

GtkWidget *
ui_comp_skwindow_get_viewport (UICompSKWindow	*skwindow)
{
	UICompSKWindowPriv *priv = UI_COMP_SKWINDOW_GET_PRIVATE(skwindow);
	return priv->viewport;
}

GtkWidget *
ui_comp_skwindow_get_label (UICompSKWindow	*skwindow)
{
	UICompSKWindowPriv *priv = UI_COMP_SKWINDOW_GET_PRIVATE(skwindow);
	return priv->label;
}

GtkWidget *
ui_comp_skwindow_get_view (UICompSKWindow	*skwindow)
{
	UICompSKWindowPriv *priv = UI_COMP_SKWINDOW_GET_PRIVATE(skwindow);
	return priv->view;
}

GtkUIManager *
ui_comp_skwindow_get_ui_manager (UICompSKWindow	*skwindow)
{
	UICompSKWindowPriv *priv = UI_COMP_SKWINDOW_GET_PRIVATE(skwindow);
	return priv->uimgr;
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
